#!/bin/bash

# Copyright 2012 Tsagkrasoulis Dimosthenis dimostsag@gmail.com
#
# This file is part of Drugster.
# 
#    Drugster is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Drugster is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
#    dms: Copyright (c) <2002> The Regents of the University of California.
#    All rights reserved.
#
#    sphgen_cpp: Andrew Magis <amagis@cise.ufl.edu>
#    Last change: Time-stamp: <26-Jul-2006 10:26:54 amagis>

INSTALL_DIR=`pwd`
ENTERED_DIR=''
choice=5

#functions
function install_mint {
	sudo apt-get install -q -y \
			openbabel \
			perl \
			perl-tk \
			libtk-tablematrix-perl \
			libtk-dirselect-perl \
			libfile-homedir-perl \
			liblist-moreutils-perl \
			libsys-statistics-linux-perl \
			cpanminus
	sudo cpanm -q Sys::Info
	echo $'\nInstalling Dependencies for Drugster Finished.\n'
	echo $'Please enter the full path where you want Drugster to be installed. A folder named Drugster_1.00 will be created there. (defaults to current directory. It is strongly recommended to install on a folder at your Home directory, since write access in the Drugster directory is necessary for some functions): '
	read ENTERED_DIR
	if [ -n "$ENTERED_DIR" ]; then
		INSTALL_DIR=$ENTERED_DIR
		echo "Installing Drugster to $INSTALL_DIR..."
		if [ ! -d "$INSTALL_DIR" ]; then
			mkdir -p "$INSTALL_DIR"
		fi
		cp -a ../Drugster_1.00 "$INSTALL_DIR"
		INSTALL_DIR="$INSTALL_DIR/Drugster_1.00"
	fi
	#echo -n "$INSTALL_DIR/Drugster_1.00" > "$INSTALL_DIR/Drugster_1.00/.drugster_dir"
	echo $'\nInstalling Drugster finished.'
	echo $'\nAdditional Software Installation...\n'
	local IN_GR='y'
	echo -n 'Do you want to install Gromacs [Y/n]? '
	read IN_GR
	if  [ "$IN_GR" != 'n' ]; then
		sudo apt-get install -q -y gromacs grace
		echo $'\nInstalling Gromacs finished.\n'
	fi
	local IN_PY='y'
	echo -n 'Do you want to install Pymol [Y/n]? '
	read IN_PY
	if  [ "$IN_PY" != 'n' ]; then
		sudo apt-get install -q -y pymol
		echo $'\nInstalling pymol finished.'
	fi
	echo $'\nNOTE: You will need to download and install Pdb2pqr, Ligbuilder and Dock\nfollowing the instructions found in the appropriate websites.\nThis script will install the necessary dependencies for the above applications\nand also the supplementary files that are needed for running Dock.\nIf Dock is not installed, please make sure to install it in the folder that you enter here.\n'
	local DOCK_DIR="`pwd`/dock"
	echo $'Please enter the Dock installation path: '
	read ENTERED_DIR
	echo $'Installing Dependencies and supplementary files for Pdb2pqr, Ligbuilder and Dock6, ...\n'
	sudo apt-get install -q -y \
			python \
			python-dev \
			g++ \
			flex \
			gfortran \
			bison
	echo ''
	if [ -n "$ENTERED_DIR" ]; then
		DOCK_DIR=$ENTERED_DIR
	fi
	if [ ! -d "$DOCK_DIR" ]; then
		mkdir -p "$DOCK_DIR/bin"
	fi
	cd ./Supplementary_Files/dms
	sudo make install
	cd ..
	cd ..
	cd ./Supplementary_Files/sphgen_cpp
	sudo make
	cd ..
	cd ..
	cp -a "./Supplementary_Files/dms" "$DOCK_DIR/bin/dms"
	cp -a "./Supplementary_Files/sphgen_cpp" "$DOCK_DIR/bin/sphgen_cpp"
	echo $'\nInstalling Dependencies and supplementary files for Pdb2pqr, Ligbuilder and Dock6 finished.\n'
	local IN_L='y'
	echo -n 'Do you want to create an application launcher for Drugster [Y/n]? '
	read IN_L
	if  [ "$IN_L" != 'n' ]; then
		mkdir -p ~/.local/share/applications
		echo -e "[Desktop Entry]\nVersion=1.00\nType=Application\nName=Drugster\nComment=Software Suite for Computer Assisted Drug Discovery\nCategories=Scientific;Tools;\nIcon=$INSTALL_DIR/Icons/Drugster.png\nExec=bash -ic 'perl $INSTALL_DIR/drugster.pl'\nTerminal=true\n" > ~/.local/share/applications/drugster.desktop
		echo $'\nCreating Launcher finished.'
	fi
	echo $'\nInstallation Process Finished.\n'
	echo $'Please check output for any (non ignored) error messages.\nIf none is found... Enjoy!\n'
}

function install_ubuntu {
	install_mint
}

function install_fedora {
	sudo yum install -y \
			gcc \
			openbabel \
			perl \
			perl-CGI \
			perl-Tk \
			perl-Tk-TableMatrix \
			perl-Tk-DirSelect \
			perl-File-HomeDir \
			perl-List-MoreUtils \
			perl-Sys-Statistics-Linux \
			perl-Test-Simple \
			perl-App-cpanminus
	sudo cpanm -q Sys::Info
	echo $'\nInstalling Dependencies for Drugster Finished.\n'
	echo $'Please enter the full path where you want Drugster to be installed. A folder named Drugster_1.00 will be created there. (defaults to current directory. It is strongly recommended to install on a folder at your Home directory, since write access in the Drugster directory is necessary for some functions): '
	read ENTERED_DIR
	if [ -n "$ENTERED_DIR" ]; then
		INSTALL_DIR=$ENTERED_DIR
		echo "Installing Drugster to $INSTALL_DIR..."
		if [ ! -d "$INSTALL_DIR" ]; then
			mkdir -p "$INSTALL_DIR"
		fi
		cp -a ../Drugster_1.00 "$INSTALL_DIR"
		INSTALL_DIR="$INSTALL_DIR/Drugster_1.00"
	fi
	#echo -n "$INSTALL_DIR/Drugster_1.00" > "$INSTALL_DIR/Drugster_1.00/.drugster_dir"
	echo $'\nInstalling Drugster finished.'
	echo $'\nAdditional Software Installation...\n'
	local IN_GR='y'
	echo -n 'Do you want to install Gromacs [Y/n]? '
	read IN_GR
	if  [ "$IN_GR" != 'n' ]; then
		sudo yum install -y gromacs grace
		echo $'\nInstalling Gromacs finished.\n'
	fi
	local IN_PY='y'
	echo -n 'Do you want to install Pymol [Y/n]? '
	read IN_PY
	if  [ "$IN_PY" != 'n' ]; then
		sudo yum install -y pymol
		echo $'\nInstalling pymol finished.'
	fi
	echo $'\nNOTE: You will need to download and install Pdb2pqr, Ligbuilder and Dock\nfollowing the instructions found in the appropriate websites.\nThis script will install the necessary dependencies for the above applications\nand also the supplementary files that are needed for running Dock.\nIf Dock is not installed, please make sure to install it in the folder that you enter here.\n'
	local DOCK_DIR="`pwd`/dock"
	echo $'Please enter the Dock installation path: '
	read ENTERED_DIR
	echo $'Installing Dependencies and supplementary files for Pdb2pqr, Ligbuilder and Dock6, ...\n'
	sudo yum install -y \
			gcc-c++ \
			python \
			python-devel \
			flex \
			gcc-gfortran \
			byacc
	echo ''
	if [ -n "$ENTERED_DIR" ]; then
		DOCK_DIR=$ENTERED_DIR
	fi
	if [ ! -d "$DOCK_DIR" ]; then
		mkdir -p "$DOCK_DIR/bin"
	fi
	cd ./Supplementary_Files/dms_fedora
	sudo make install
	cd ..
	cd ..
	cd ./Supplementary_Files/sphgen_cpp
	sudo make
	cd ..
	cd ..
	cp -a "./Supplementary_Files/dms_fedora" "$DOCK_DIR/bin/dms"
	cp -a "./Supplementary_Files/sphgen_cpp" "$DOCK_DIR/bin/sphgen_cpp"
	echo $'\nInstalling Dependencies and supplementary files for Pdb2pqr, Ligbuilder and Dock6 finished.\n'
	local IN_L='y'
	echo -n 'Do you want to create an application launcher for Drugster [Y/n]? '
	read IN_L
	if  [ "$IN_L" != 'n' ]; then
		mkdir -p ~/.local/share/applications
		echo -e "[Desktop Entry]\nVersion=1.00\nType=Application\nName=Drugster\nComment=Software Suite for Computer Assisted Drug Discovery\nCategories=Scientific;Tools;\nIcon=$INSTALL_DIR/Icons/Drugster.png\nExec=bash -ic 'perl $INSTALL_DIR/drugster.pl'\nTerminal=true\n" > ~/.local/share/applications/drugster.desktop
		echo $'\nCreating Launcher finished.'
	fi
	echo $'\nInstallation Process Finished.\n'
	echo $'Please check output for any (non ignored) error messages.\nIf none is found... Enjoy!\n'
}

function install_opensuse {
	sudo zypper -n install \
			make \
			xorg-x11-devel \
			gcc \
			openbabel \
			perl \
			perl-Tk \
			perl-Tk-devel \
			perl-File-HomeDir \
			perl-List-MoreUtils \
			perl-CPANPLUS
	sudo cpanp i \
		      CPANPLUS::Dist::MM \
		      Tk::TableMatrix \
		      Tk::DirSelect \
		      UNIVERSAL::require \
		      Sys::Statistics::Linux \
		      Unix::Processors \
		      Test::Sys::Info \
		      Config::General \
		      Sys::Info::Base \
		      Sys::Info::Driver::Linux \
		      Sys::Info
	echo $'\nInstalling Dependencies for Drugster Finished.\n'
	echo $'Please enter the full path where you want Drugster to be installed. A folder named Drugster_1.00 will be created there. (defaults to current directory. It is strongly recommended to install on a folder at your Home directory, since write access in the Drugster directory is necessary for some functions): '
	read ENTERED_DIR
	if [ -n "$ENTERED_DIR" ]; then
		INSTALL_DIR=$ENTERED_DIR
		echo "Installing Drugster to $INSTALL_DIR..."
		if [ ! -d "$INSTALL_DIR" ]; then
			mkdir -p "$INSTALL_DIR"
		fi
		cp -a ../Drugster_1.00 "$INSTALL_DIR"
		INSTALL_DIR="$INSTALL_DIR/Drugster_1.00"
	fi
	#echo -n "$INSTALL_DIR/Drugster_1.00" > "$INSTALL_DIR/Drugster_1.00/.drugster_dir"
	echo $'\nInstalling Drugster finished.'
	echo $'\nAdditional Software Installation...\n'
	local IN_GR='y'
	echo -n 'Do you want to install Gromacs [Y/n]? '
	read IN_GR
	if  [ "$IN_GR" != 'n' ]; then
		sudo zypper -n install gromacs xmgrace
		echo $'\nInstalling Gromacs finished.\n'
	fi
	local IN_PY='y'
	echo -n 'Do you want to install Pymol [Y/n]? '
	read IN_PY
	if  [ "$IN_PY" != 'n' ]; then
		sudo zypper -n install pymol
		echo $'\nInstalling pymol finished.'
	fi
	echo $'\nNOTE: You will need to download and install Pdb2pqr, Ligbuilder and Dock\nfollowing the instructions found in the appropriate websites.\nThis script will install the necessary dependencies for the above applications\nand also the supplementary files that are needed for running Dock.\nIf Dock is not installed, please make sure to install it in the folder that you enter here.\n'
	local DOCK_DIR="`pwd`/dock"
	echo $'Please enter the Dock installation path: '
	read ENTERED_DIR
	echo $'Installing Dependencies and supplementary files for Pdb2pqr, Ligbuilder and Dock6, ...\n'
	sudo zypper -n install \
			gcc-c++ \
			python \
			python-devel \
			flex \
			gcc-fortran \
			bison
	echo ''
	if [ -n "$ENTERED_DIR" ]; then
		DOCK_DIR=$ENTERED_DIR
	fi
	if [ ! -d "$DOCK_DIR" ]; then
		mkdir -p "$DOCK_DIR/bin"
	fi
	cd ./Supplementary_Files/dms
	sudo make install
	cd ..
	cd ..
	cd ./Supplementary_Files/sphgen_cpp
	sudo make
	cd ..
	cd ..
	cp -a "./Supplementary_Files/dms" "$DOCK_DIR/bin/dms"
	cp -a "./Supplementary_Files/sphgen_cpp" "$DOCK_DIR/bin/sphgen_cpp"
	echo $'\nInstalling Dependencies and supplementary files for Pdb2pqr, Ligbuilder and Dock6 finished.\n'
	local IN_L='y'
	echo -n 'Do you want to create an application launcher for Drugster [Y/n]? '
	read IN_L
	if  [ "$IN_L" != 'n' ]; then
		mkdir -p ~/.local/share/applications
		echo -e "[Desktop Entry]\nVersion=1.00\nType=Application\nName=Drugster\nComment=Software Suite for Computer Assisted Drug Discovery\nCategories=Scientific;Tools;\nIcon=$INSTALL_DIR/Icons/Drugster.png\nExec=bash -ic 'perl $INSTALL_DIR/drugster.pl'\nTerminal=true\n" > ~/.local/share/applications/drugster.desktop
		echo $'\nCreating Launcher finished.'
	fi
	echo $'\nInstallation Process Finished.\n'
	echo $'Please check output for any (non ignored) error messages.\nIf none is found... Enjoy!\n'
}

# Main
echo $'\nDrugster v1.00 Installation\n'
echo $'This script will install/update the necessary packages using the native OS Package Manager.\nFor a list of these packages please check the manual or inspect the script source code.\nAn active internet connection is required.\n'

echo "Choose your Linux Distribution:"
echo "1. Linux Mint"
echo "2. Ubuntu"
echo "3. Fedora"
echo "4. OpenSuse"
echo -n "Please enter the corresponding number [1,2,3 or 4] or Ctrl-C to Quit: "
while [ $choice -eq 5 ]; do
read choice
if [ $choice -eq 1 ] ; then
	echo $'\nInstalling Dependencies for Linux Mint...\n'
	install_mint
else                   
        if [ $choice -eq 2 ] ; then
                echo $'\nInstalling Dependencies for Ubuntu...\n'
		install_ubuntu
        else
         
                if [ $choice -eq 3 ] ; then
			echo $'\nInstalling Dependencies for Fedora...\n'
                        install_fedora
                else
			if [ $choice -eq 4 ] ; then
				echo $'\nInstalling Dependencies for openSUSE...\n'
                        	install_opensuse
                	else
                        	echo "1. Linux Mint"
				echo "2. Ubuntu"
				echo "3. Fedora"
				echo "4. OpenSuse"
				echo -n "Please enter the corresponding number [1,2,3 or 4] or Ctrl-C to Quit: "
				choice=5
			fi

                fi   
        fi
fi

done
