#!/usr/bin/perl

# Copyright (C) 2014  Paraskevas Fakourelis fakourelis@ceid.upatras.gr
#
#  This file is part of DrugOn.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    dms: Copyright (c) <2002> The Regents of the University of California.
#    All rights reserved.
#
#    sphgen_cpp: Andrew Magis <amagis@cise.ufl.edu>
#    Last change: Time-stamp: <26-Jul-2006 10:26:54 amagis>
#
#    opal-python: Copyright (c) 2005-2008 The Regents of the University of California.
#    All rights reserved.

# Use
use warnings;
use strict;
use Tk;
use Tk::ROText;
use Tk::LabFrame;
use Tk::LabEntry;
use Tk::JPEG;
use Tk::PNG;
use Tk::MsgBox;
use Tk::TableMatrix::Spreadsheet;
use Tk::NoteBook;
use Tk::Balloon;
use File::HomeDir;
use File::Spec;
use Cwd;
use Tk::DirSelect;
use Tk::DialogBox;
use List::MoreUtils qw(uniq);
use Tk::ProgressBar;
use Time::HiRes qw/gettimeofday tv_interval/;
use Scalar::Util qw(openhandle);
use Sys::Statistics::Linux::MemStats;
use Sys::Info::OS;
use Tk::BrowseEntry; 
use List::Util qw[min max];
#use LWP::Simple;
use WWW::Mechanize;

# Global Variables
my ($selected_pocket_par_file, $selected_grow_par_file, $selected_link_par_file, $selected_process_par_file);
my $ligbuilder_has_error = 0;
my ($selected_cavity_par_file, $selected_build_par_file,$selected_build_db_folder);
my ($selected_text_editor_files, $selected_molecular_viewer_files);
my  $button_size_w = 25;
my ($preferred_terminal, $preferred_text_editor, $preferred_mol_viewer,$preferred_graph_viewer);
my ($pdb2pqr_path,$gromacs_path,$ligbuilder_path, $ligbuilder_20_path, $pharmACOphore_path);
my ($default_pocket_param_file, $default_grow_param_file, $default_link_param_file, $default_process_param_file,$default_cavity_param_file,$default_build_param_file,$default_gromacs_param_file);
my $fragment_db_path;
my ($grow_param_expand, $link_param_expand) = (0, 0);
my $default_working_directory;
my $pid;
my $enable_fun=0;
my $progress;
my $enable_log = 0;
my $default_log_file;
my @time = gettimeofday;
my $steps = 0;
my $running_id;
my @complete_steps = qw/14 1 1 6 7/;
my $stdbuf_str = "stdbuf -oL";
my $working_directory = File::HomeDir->my_documents;
my $drugOns_directory = getcwd();
my $running_bulb = 0;
my $refresh_rate=1000;
my $cpus;
my @perc;
my %prev_stats;
my $mem_usage;
my $swap_usage;
my $os = Sys::Info::OS->new();
my $arch = $os->bitness;
my $build_function = '-Automatic';
my ($selected_pre_file,$selected_pre_save_file);
my $pre_rem_hetatm=1;
my $pre_pdb2pqr=1;
my @pre_remote=qw/1 0/;
my @neutralize = qw/1 1/;
my $pdb2pqr_ff = 'parse';
my $selected_mdrun_par_file;
my @em_checks = qw/1 1 1 0/;
my $gromacs_ff = 'amber03p-amber94n';
my $gromacs_boxtype = 'cubic';
my $gromacs_watermodel = 'tip3p';
my $sol_box_distance = 0.5;
my $initial_center;
my ($selected_gro_in_file,$selected_gro_out_file);
my ($selected_em_md_in_pdb_file,$selected_em_md_in_mol2_file,$selected_em_md_out_folder);
my ($selected_pha_pr_file,$selected_pha_li_file,$selected_pha_save_file,$selected_pha_input_pr_file,$selected_pha_input_li_file,$pha_save_file_dir);
my $pha_def_options=1;
my $selected_pha_conf_file;
my $conf_cust_file;
my $pharmacophore_ff= 'default';
my $pharmacophore_directory = getcwd();
my $preprocess_current_step = 0; my $pre_complete_steps = 7; my $pre_steps = 0;
my $gromacs_has_error = 0; my $gromacs_current_step = 0; my $gromacs_complete_steps = 14; my $gromacs_steps = 0;
my @fast_switches;
my $running_fast_id;
my $add_spc = 0;
my $phaconfig ;


# set correct $drugOns_directory, chdir to $working_directory
{
	my $abs_drugOns_file = File::Spec->rel2abs($0);
	my ($volume,$directories,$drug_file) = File::Spec->splitpath($abs_drugOns_file);
	$drugOns_directory = $directories;
	chdir "$working_directory" or die "Can't cd to $working_directory: $!\n";
}			

#
# Construction
#

# Main Window
my $mw = new MainWindow;
$mw->withdraw;
$mw->geometry("800x800+0+0");
$mw->update;


# Icons
my $new_button_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/document--plus.png'));
my $browse_button_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/folder2.png'));
my $folder_button_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/folder1.png'));
my $save_as_button_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/save.png'));
my $print_button_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/printer.png'));
my $fragment_databases_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/Database.png'));
my $parameter_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/hammer-screwdriver.png'));
my $preferences_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/wrench.png'));
my $terminal_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/terminal.png'));
my $text_editor_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/txt.png'));
my $molecular_viewer_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/molecule1.png'));
my $molecular_viewer_plus_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/molecule1_plus.png'));
my $help_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/help.png'));
my $help_ligb_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/Website.png'));
my $help_web_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/about.png'));
my $plus_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/plus.png'));
my $minus_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/minus.png'));
my $program_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/DrugOn.png'));
my $arrow_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/arrow.png'));
my $arrow_180_button_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/arrow-180.png'));
my $eraser_image = $mw->Photo(-format => 'png' , -file => File::Spec->catfile($drugOns_directory,'Icons/eraser.png'));
my $bulb_on_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/light-bulb.png'));
my $bulb_off_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/light-bulb-off.png'));
my $on_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/eye_b_on.png'));
my $off_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/eye_b.png'));
my $concat_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/document-view.png'));
my $exclamation_image = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/exclamation-white.png'));
my $down_arrow = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/Arrow_Down.png'));
my $check = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/check.png'));
my $uncheck = $mw->Photo(-format => 'png', -file => File::Spec->catfile($drugOns_directory,'Icons/uncheck.png'));
$mw->iconimage ($program_image);


# Menu Bar
my $menubar = $mw->Menu;
$mw->configure(-menu => $menubar);

my $menu_file = $menubar -> cascade(-label=>"File", -tearoff => 0);
my $menu_edit = $menubar -> cascade(-label =>"Edit", -tearoff => 0);
my $menu_open = $menubar -> cascade(-label =>"Open", -tearoff => 0);
my $menu_help = $menubar -> cascade(-label =>"Help", -tearoff => 0);


$menu_file -> command(-label => "Load Output", -underline => 0, -accelerator=> 'Ctrl+l', -command => [\&menu_file_callback,'load']);
$menu_file -> command(-label => "Save Output As...", -underline => 12, -accelerator=> 'Ctrl+a', -command => [\&menu_file_callback,'save_as']);
$menu_file -> command(-label => "Print Output", -underline => 0, -accelerator=> 'Ctrl+p', -command => [\&menu_file_callback,'print']);
$menu_file -> command(-label => "Clear Output", -underline => 0, -accelerator=> 'Ctrl+c', -command => [\&menu_file_callback,'clear']);
$menu_file -> separator();
$menu_file -> command(-label => "Toggle Auto Logging", -underline => 8, -accelerator=> 'Ctrl+u', -command => [\&menu_file_callback,'log']);
$menu_file -> separator();
$menu_file -> command(-label => "Quit", -underline => 0, -accelerator=> 'Ctrl+q', -command => \&quit_callback);
my $menu_edit_m = $mw -> Menu(-type=>'normal', -tearoff=>0); 
$menu_edit_m -> command(-label => "Handle Databases", -underline => 3, -accelerator=> 'Ctrl+g', -command => [\&menu_edit_callback,'fragment']);
my $menu_edit_param_sub = $menu_edit_m -> cascade(-label=>"Parameter Files", -underline=>0, -tearoff => 0);
$menu_edit_param_sub -> command(-label => "Gromacs MDRun", -command => [\&menu_edit_callback,'parameter_mdrun']);
$menu_edit_param_sub -> command(-label => "LigBuilder v1.2", -command => [\&menu_edit_callback,'parameter_lig_1']);
$menu_edit_param_sub -> command(-label => "LigBuilder v2.0", -command => [\&menu_edit_callback,'parameter_lig_2']);
$menu_edit_m -> separator();
$menu_edit_m -> command(-label => "Preferences", -underline => 7, -accelerator=> 'Ctrl+n', -command => [\&menu_edit_callback,'preferences']);
$menu_edit->configure(-menu=>$menu_edit_m);
$menu_open -> command(-label => "File Manager", -underline => 0, -accelerator=> 'Ctrl+f', -command => [\&menu_open_callback,'file']);
$menu_open -> command(-label => "Text Editor", -underline => 6, -accelerator=> 'Ctrl+d', -command => [\&menu_open_callback,'text_editor']);
$menu_open -> command(-label => "Molecular Viewer", -underline => 9, -accelerator=> 'Ctrl+v', -command => [\&menu_open_callback,'molecular_viewer']);
$menu_help -> command(-label => "Help", -underline => 0, -accelerator=> 'F1', -command => [\&menu_help_callback,'help']);
$menu_help -> command(-label => "DrugOn Website", -command => [\&menu_help_callback,'program_website']);
$menu_help -> separator();
$menu_help -> command(-label => "PDB2PQR Website", -command => [\&menu_help_callback,'pdb2pqr_website']);
$menu_help -> command(-label => "Gromacs Website", -command => [\&menu_help_callback,'gromacs_website']);
$menu_help -> command(-label => "LigBuilder Website", -command => [\&menu_help_callback,'ligbuilder_2_0_website']);
$menu_help -> separator();
$menu_help -> command(-label => "About", -underline => 0, -command => [\&menu_help_callback,'about']);

# Main Balloon
my $main_balloon = $mw->Balloon();

# Info Bar
my $info_frame = $mw->Frame(-borderwidth=>2, -relief=>'raised')->pack(-side => 'bottom', -anchor => 'sw', -expand => 0, -fill => 'x');
my $pb = $info_frame->ProgressBar(-variable=>\$progress,-width => 17,-blocks=>100,-gap=> 2, -colors   => &PBcolor)->pack(-fill=>'x', -expand =>1, -side=>'left',-padx => 1, -pady => 2);
$main_balloon->attach($info_frame, -msg => "Progress Bar");
my $pb_label= $info_frame->Label(-text=>'0%',-justify=>'right',-width=>6,-font=>['Courier', '9', 'bold'],-anchor=>'e',-relief=>'sunken',-borderwidth=>1)->pack(-side=>'left', -pady => 2,-padx=>1);
$main_balloon->attach($pb_label, -msg => "Progress Bar");
my $pb__bulb_label= $info_frame->Label(-image => $bulb_off_image,-anchor=>'e')->pack(-side=>'left', -pady => 1,-padx=>1);
$main_balloon->attach($pb__bulb_label, -msg => "Running...");
my $log_icon = $info_frame->Button(-borderwidth=>0,-relief=>'flat',-image=> $off_image, -command => [\&log_button_callback])->pack(-side=>'right',-pady => 1,-padx=>4);
$main_balloon->attach($log_icon, -msg => "Toggle Auto Logging");
&create_stats;
my $swap_prog = $info_frame->ProgressBar(-variable => \$swap_usage,-blocks=>10,-gap=> 2,-colors => &PBcolor, -length => 17,-anchor => 's')->pack(-side=>'right', -pady => 2,-padx=>0);
$main_balloon->attach($swap_prog, -msg => "Swap usage");
my $mem_prog = $info_frame->ProgressBar(-variable => \$mem_usage,-blocks=>10,-gap=> 2,-colors => &PBcolor, -length => 17,-anchor => 's')->pack(-side=>'right', -pady => 2,-padx=>3);
$main_balloon->attach($mem_prog, -msg => "Memory usage");
my $cpu_prog = $info_frame->ProgressBar(-variable => \$perc[0],-blocks=>10,-gap=> 2,-colors => &PBcolor, -length => 17,-anchor => 's')->pack(-side=>'right', -pady => 2,-padx=>0);
$main_balloon->attach($cpu_prog, -msg => "CPU usage");
my $cpu_label = $info_frame->Label(-text=>'%CPU',-font=>['Courier', '9', 'bold'],-relief=>'sunken',-width=>7,-anchor=>'e',-relief=>'sunken',-borderwidth=>1)->pack(-side=>'right', -pady => 2,-padx=>2);
$main_balloon->attach($cpu_label, -msg => "CPU usage");


# Icon Menu, Working Directory
my $iconmenubar = $mw->Frame(-borderwidth=>2, -relief=>'raised')->pack(-side => 'top', -fill=>'x', -anchor => 'nw');
my $subframe1_iconmenubar = $iconmenubar->Frame()->pack(-side => 'left', -fill=>'none', -anchor => 'nw', -padx=> 5);
my $iconmenu_button1 = $subframe1_iconmenubar->Button(-text => 'Load', -image => $browse_button_image, -command => [\&menu_file_callback,'load'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button1, -msg => 'Load Output', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button2 = $subframe1_iconmenubar->Button(-text => 'Save_as', -image => $save_as_button_image, -command => [\&menu_file_callback,'save_as'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button2, -msg => 'Save Output As', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button3 = $subframe1_iconmenubar->Button(-text => 'Print', -image => $print_button_image, -command => [\&menu_file_callback,'print'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button3, -msg => 'Print Output', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button4 = $subframe1_iconmenubar->Button(-text => 'Text', -image => $text_editor_button_image, -command => [\&menu_open_callback,'text_editor'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button4, -msg => 'Open Text Editor', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button5 = $subframe1_iconmenubar->Button(-text => 'Term', -image => $terminal_button_image, -command => [\&menu_open_callback,'terminal'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button5, -statusmsg => 'Open Terminal', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button6 = $subframe1_iconmenubar->Button(-text => 'Clear', -image => $eraser_image, -command => [\&menu_file_callback,'clear'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button6, -msg => 'Clear Output', -initwait => 400, -balloonposition => 'mouse');
my $subframe2_iconmenubar = $iconmenubar->Frame()->pack(-side => 'left', -fill=>'none', -anchor => 'nw', -padx=> 5);
my $iconmenu_button8 = $subframe2_iconmenubar->Button(-text => 'Pref', -image => $preferences_button_image, -command => [\&menu_edit_callback,'preferences'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button8, -msg => 'Preferences', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button9 = $subframe2_iconmenubar->Button(-text => 'FDb', -image => $fragment_databases_button_image, -command => [\&menu_edit_callback,'fragment'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button9, -msg => 'Handle Databases', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button10 = $subframe2_iconmenubar->Button(-text => 'Mol', -image => $molecular_viewer_button_image, -command => [\&menu_open_callback,'molecular_viewer'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button10, -msg => 'Open Molecular Viewer', -initwait => 400, -balloonposition => 'mouse');
my $subframe3_label = $iconmenubar->Frame(-label=> 'DrugOn pro')->pack(-side => 'left', -fill=>'none', -anchor => 'w', -padx=>150);
my $subframe4_iconmenubar = $iconmenubar->Frame()->pack(-side => 'right', -fill=>'none', -anchor => 'ne', -padx=> 5);
my $iconmenu_button11 = $subframe4_iconmenubar->Button(-text => 'Help', -image => $help_button_image, -command => [\&menu_help_callback,'help'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button11, -msg => 'Help', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button12 = $subframe4_iconmenubar->Button(-text => 'Website', -image => $help_ligb_button_image, -command => [\&menu_help_callback,'program_website'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button12, -msg => 'DrugOns Website', -initwait => 400, -balloonposition => 'mouse');
my $iconmenu_button13 = $subframe4_iconmenubar->Button(-text => 'About', -image => $help_web_button_image, -command => [\&menu_help_callback,'about'])->pack( -side => 'left', -padx => 0);
$main_balloon->attach($iconmenu_button13, -msg => 'About...', -initwait => 400, -balloonposition => 'mouse');




# App, System Windows
my $panedwin1 = $mw -> Panedwindow(-orient => 'vertical', -borderwidth=>1, -relief=>'raised') -> pack( -fill=>'both', -expand => 5) ;
my $button_frame = $panedwin1->Frame()->pack(-side => 'top', -fill=>'both', -anchor => 'nw',-expand=>1);
my $Tab_frame = $button_frame->NoteBook()->pack(-side => 'top', -fill=>'both',-expand=>1, -pady => 2, -padx => 2);
my $pre_tab_frame = $Tab_frame ->add('Pre',-label => "Input\npreparation");
my $em_gromacs_frame = $Tab_frame  ->add('EM', -label => "Receptor\noptimization");
my $buildlig_frame = $Tab_frame  ->add('Build', -label => "Build\n    ligands    ");
my $pha_tab_frame =  $Tab_frame ->add('pha', -label => "Pharmacophore");



# preprocess buttons
my $pre_frame = $pre_tab_frame->Frame()->pack(-side => 'left', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $pre_button = $pre_frame->Button(-text => "Input preparation", -width => $button_size_w,-command => [\&preprocess_callback,'pdb2pqr'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($pre_button, -msg => 'Input preparation', -initwait => 400, -balloonposition => 'mouse');
my $pre_interrupt_frame = $pre_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 0);
my $pre_interrupt_button = $pre_interrupt_frame->Button(-text => 'Abort', -width => $button_size_w, -command => [\&preprocess_callback,'interrupt'], -state => 'disabled')->pack( -side => 'top', -expand => 1, -pady => 1);
$main_balloon->attach($pre_interrupt_button, -msg => 'Interrupt Running Command', -initwait => 400, -balloonposition => 'mouse');
my $pre_pdb_frame = $pre_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 1);
my $pre_input_frame = $pre_pdb_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $pre_input_entry = $pre_input_frame->LabEntry(-label => '   Input PDB File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pre_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3,  -expand => 1);
my $pre_input_phantom_frame = $pre_input_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $pre_input_browse_button = $pre_input_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_pre_file_callback,'in'])->pack( -side => 'right');
$main_balloon->attach($pre_input_browse_button, -msg => 'Open PDB File', -initwait => 400, -balloonposition => 'mouse');
my $pre_output_frame = $pre_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $pre_output_entry = $pre_output_frame->LabEntry(-label => 'Output PDB File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pre_save_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $pre_output_phantom_frame = $pre_output_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $pre_output_browse_button = $pre_output_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_pre_file_callback,'out'])->pack( -side => 'right');
$main_balloon->attach($pre_output_browse_button, -msg => 'Save PDB File', -initwait => 400, -balloonposition => 'mouse');
my $pre_options_frame = $pre_tab_frame->LabFrame(-label => 'Input Preparation Options')->pack(-side => 'right', -anchor => 'n',-fill=>'y', -expand => 1, -fill => 'both', -pady => 1);
my $pre_rem_hetatm_frame = $pre_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 0, -fill => 'x', -pady => 1);
my $pre_rem_hetatm_fantom_frame = $pre_rem_hetatm_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $pre_rem_hetatm_checkbox = $pre_rem_hetatm_frame->Checkbutton(-selectimage => $check ,-image=> $uncheck,-variable => \$pre_rem_hetatm)->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $pre_rem_hetatm_label = $pre_rem_hetatm_frame->Label(-text => '                 Remove Heteroatoms from PDB:')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $pre_pdb2pqr_options_frame = $pre_options_frame->LabFrame(-label => 'PDB2PQR Options')->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 0, -padx => 4);
my $pre_ff_frame = $pre_pdb2pqr_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 2);
my $pre_ff_entry = $pre_ff_frame->BrowseEntry( -label => '            Force Field: ', -labelPack => [ -side => "left" ],-arrowimage=>$down_arrow, -textvariable => \$pdb2pqr_ff, -insertofftime => 0);
$pre_ff_entry->insert("end", "amber");
$pre_ff_entry->insert("end", "charmm");
$pre_ff_entry->insert("end", "parse");
$pre_ff_entry->insert("end", "tyl06");
$pre_ff_entry->insert("end", "peoepb");
$pre_ff_entry->insert("end", "swanson");
$pre_ff_entry->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1, -pady => 2);
my $pre_ff_fantom_frame = $pre_ff_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $pre_neutralc_frame = $pre_pdb2pqr_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 0);
my $pre_neutralc_fantom_frame = $pre_neutralc_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $pre_neutralc_checkbox = $pre_neutralc_frame->Checkbutton(-selectimage => $check ,-image=> $uncheck, -variable => \$neutralize[0])->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $pre_neutralc_label = $pre_neutralc_frame->Label(-text => 'Neutralize C Terminus (Only with PARSE):')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $pre_neutraln_frame = $pre_pdb2pqr_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 0);
my $pre_neutraln_fantom_frame = $pre_neutraln_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $pre_neutraln_checkbox = $pre_neutraln_frame->Checkbutton(-selectimage => $check ,-image=> $uncheck, -variable => \$neutralize[1])->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $pre_neutraln_label = $pre_neutraln_frame->Label(-text => 'Neutralize N Terminus (Only with PARSE):')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);


# em buttons
my $em_frame = $em_gromacs_frame->Frame()->pack(-side => 'left', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $em_button = $em_frame->Button(-text => "Receptor optimization", -width => $button_size_w, -command => [\&gromacs_mdrun_callback,'mdrun'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($em_button, -msg => 'Receptor optimization', -initwait => 400, -balloonposition => 'mouse');
my $em_interrupt_frame = $em_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 4);
my $em_interrupt_button = $em_interrupt_frame->Button(-text => 'Abort', -width => $button_size_w, -command => [\&gromacs_mdrun_callback,'interrupt'], -state => 'disabled')->pack( -side => 'top', -expand => 1, -pady => 1);
$main_balloon->attach($em_interrupt_button, -msg => 'Interrupt Running Command', -initwait => 400, -balloonposition => 'mouse');
my $em_pdb_frame = $em_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 1);
my $em_input_frame = $em_pdb_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $em_input_entry = $em_input_frame->LabEntry(-label => '   Input PDB File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_gro_in_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $em_input_phantom_frame = $em_input_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $em_input_browse_button = $em_input_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_gro_file_callback,'in'])->pack( -side => 'right');
$main_balloon->attach($em_input_browse_button, -msg => 'Open PDB File', -initwait => 400, -balloonposition => 'mouse');
my $em_output_frame = $em_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $em_output_entry = $em_output_frame->LabEntry(-label => 'Output PDB File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_gro_out_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $em_output_phantom_frame = $em_output_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $em_output_browse_button = $em_output_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_gro_file_callback,'out'])->pack( -side => 'right');
$main_balloon->attach($em_output_browse_button, -msg => 'Save PDB File', -initwait => 400, -balloonposition => 'mouse');
my $em_options_frame = $em_gromacs_frame->LabFrame(-label => 'Gromacs EM Options')->pack(-side => 'right', -anchor => 'n', -expand => 1, -fill => 'y', -pady => 0);
my $em_ff_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 2);
my $em_ff_entry = $em_ff_frame->BrowseEntry( -label => '                      Force Field: ', -labelPack => [ -side => "left" ],-arrowimage=>$down_arrow, -textvariable => \$gromacs_ff, -insertofftime => 0);
$em_ff_entry->insert("end", "amber03p-amber94n");
$em_ff_entry->insert("end", "amber94");
$em_ff_entry->insert("end", "amber96p-amber94n");
$em_ff_entry->insert("end", "amber99p-amber94n");
$em_ff_entry->insert("end", "amber99SBp-amber94n");
$em_ff_entry->insert("end", "amber99SB-ILDNp-amber94n");
$em_ff_entry->insert("end", "amberGS");
$em_ff_entry->insert("end", "charmm27");
$em_ff_entry->insert("end", "gromos96 43a1");
$em_ff_entry->insert("end", "gromos96 43a2");
$em_ff_entry->insert("end", "gromos96 45a3");
$em_ff_entry->insert("end", "gromos96 53a5");
$em_ff_entry->insert("end", "gromos96 53a6");
$em_ff_entry->insert("end", "opls-aa/l");
$em_ff_entry->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1);
my $em_ff_fantom_frame = $em_ff_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $em_box_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $em_box_entry = $em_box_frame->BrowseEntry( -label => '                         Box Type: ', -labelPack => [ -side => "left" ],-arrowimage=>$down_arrow, -textvariable => \$gromacs_boxtype, -insertofftime => 0);
$em_box_entry->insert("end", "cubic");
$em_box_entry->insert("end", "triclinic");
$em_box_entry->insert("end", "octahedron");
$em_box_entry->insert("end", "dodecahedron");
$em_box_entry->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1);
my $em_box_fantom_frame = $em_box_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $em_d_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $em_d_entry = $em_d_frame->LabEntry(-label => '          Sol-Box Distance: ', -labelPack => [ -side => "left" ], -textvariable => \$sol_box_distance, -background => 'white', -insertofftime => 0, -justify => 'right', -width => 4)->pack( -side => 'left',-fill=>'x', -padx => 3,  -expand => 1);
my $em_d_phantom_frame = $em_d_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 25, -pady => 0);
my $em_solvate_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 0);
my $em_solvate_fantom_frame = $em_solvate_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $em_solvate_checkbox = $em_solvate_frame->Checkbutton(-selectimage => $check ,-image=> $uncheck, -variable => \$em_checks[0])->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $em_solvate_label = $em_solvate_frame->Label(-text => '                       Solvate Protein in Water:')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $em_water_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $em_water_entry = $em_water_frame->BrowseEntry( -label => '                    Water Model: ', -labelPack => [ -side => "left" ], -arrowimage=>$down_arrow,-textvariable => \$gromacs_watermodel, -insertofftime => 0);
$em_water_entry->insert("end", "tip3p");
$em_water_entry->insert("end", "tip4p");
$em_water_entry->insert("end", "spc");
$em_water_entry->insert("end", "spce");
$em_water_entry->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1);
my $em_water_fantom_frame = $em_water_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $em_ions_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 0);
my $em_ions_fantom_frame = $em_ions_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $em_ions_checkbox = $em_ions_frame->Checkbutton( -selectimage => $check ,-image=> $uncheck,-variable => \$em_checks[1])->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $em_ions_label = $em_ions_frame->Label(-text => '                     Neutralize system:')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $em_remw_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $em_remw_fantom_frame = $em_remw_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $em_remw_checkbox = $em_remw_frame->Checkbutton( -selectimage => $check ,-image=> $uncheck,-variable => \$em_checks[2])->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $em_remw_label = $em_remw_frame->Label(-text => 'Remove water/ions from output PDB File:')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $em_en_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $em_en_fantom_frame = $em_en_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $em_en_checkbox = $em_en_frame->Checkbutton(-selectimage => $check ,-image=> $uncheck, -variable => \$em_checks[3])->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $em_en_label = $em_en_frame->Label(-text => 'Show resulting Energy Graph (.xvg file):')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $em_param_frame = $em_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 2);
my $em_param_entry = $em_param_frame->LabEntry(-label => 'MDRun Parameter File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_mdrun_par_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left',-fill=>'x', -padx => 3,  -expand => 1);
my $em_param_create_button = $em_param_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_par_file_gro_callback,'mdrun'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($em_param_create_button, -msg => 'Edit/Create New Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $em_param_browse_button = $em_param_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_par_file_gro_callback,'mdrun'])->pack( -side => 'right');
$main_balloon->attach($em_param_browse_button, -msg => 'Open Parameter File', -initwait => 400, -balloonposition => 'mouse');

# Ligbuilder Buttons

# Ligbuilder V 1.2

my $lig_frame =$buildlig_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $ligbuilder_frame = $lig_frame ->LabFrame(-label => 'Ligbuilder v1.2')->pack(-side => 'left', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 0);
my $pocket_frame = $ligbuilder_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $pocket_button = $pocket_frame->Button(-text => 'Pocket', -width => $button_size_w, -command => [\&ligbuilder_buttons_callback,'pocket'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($pocket_button, -msg => 'Run Pocket', -initwait => 400, -balloonposition => 'mouse');
my $pocket_group_frame = $pocket_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $pocket_parameter_entry = $pocket_group_frame->LabEntry(-label => 'Parameter File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pocket_par_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3,  -expand => 1);
my $pocket_parameter_create_button = $pocket_group_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_par_file_callback,'pocket'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($pocket_parameter_create_button, -msg => 'Edit/Create New Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $pocket_parameter_browse_button = $pocket_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_par_file_callback,'pocket'])->pack( -side => 'right');
$main_balloon->attach($pocket_parameter_browse_button, -msg => 'Open Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $grow_frame = $ligbuilder_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $grow_button = $grow_frame->Button(-text => 'Grow', -width => $button_size_w, -command => [\&ligbuilder_buttons_callback,'grow'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($grow_button, -msg => 'Run Grow', -initwait => 400, -balloonposition => 'mouse');
my $grow_add_spc_frame = $grow_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 0, -fill => 'x', -pady => 0);
my $grow_add_spc_fantom_frame = $grow_add_spc_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $grow_add_spc_checkbox = $grow_add_spc_frame->Checkbutton(-selectimage => $check ,-image=> $uncheck, -variable => \$add_spc)->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $grow_add_spc_label = $grow_add_spc_frame->Label(-text => 'Make all seed hydrogens growing sites:')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $grow_group_frame = $grow_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $grow_parameter_entry = $grow_group_frame->LabEntry(-label => 'Parameter File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_grow_par_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $grow_parameter_create_button = $grow_group_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_par_file_callback,'grow'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($grow_parameter_create_button, -msg => 'Edit/Create New Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $grow_parameter_browse_button = $grow_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_par_file_callback,'grow'])->pack( -side => 'right');
$main_balloon->attach($grow_parameter_browse_button, -msg => 'Open Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $link_frame = $ligbuilder_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $link_button = $link_frame->Button(-text => 'Link', -width => $button_size_w, -command => [\&ligbuilder_buttons_callback,'link'])->pack( -side => 'top',-expand => 1);
$main_balloon->attach($link_button, -msg => 'Run Link', -initwait => 400, -balloonposition => 'mouse');
my $link_add_spc_frame = $link_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 0, -fill => 'x', -pady => 0);
my $link_add_spc_fantom_frame = $link_add_spc_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $link_add_spc_checkbox = $link_add_spc_frame->Checkbutton( -selectimage => $check ,-image=> $uncheck,-variable => \$add_spc)->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
my $link_add_spc_label = $link_add_spc_frame->Label(-text => 'Make all seed hydrogens growing sites:')->pack(-side => 'right', -padx => 0, -fill => 'none', -expand => 0);
my $link_group_frame = $link_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $link_parameter_entry = $link_group_frame->LabEntry(-label => 'Parameter File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_link_par_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $link_parameter_create_button = $link_group_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_par_file_callback,'link'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($link_parameter_create_button, -msg => 'Edit/Create New Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $link_parameter_browse_button = $link_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_par_file_callback,'link'])->pack( -side => 'right');
$main_balloon->attach($link_parameter_browse_button, -msg => 'Open Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $process_frame = $ligbuilder_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $process_button = $process_frame->Button(-text => 'Process', -width => $button_size_w, -command => [\&ligbuilder_buttons_callback,'process'])->pack( -side => 'top',-expand => 1);
$main_balloon->attach($process_button, -msg => 'Run Process', -initwait => 400, -balloonposition => 'mouse');
my $process_group_frame = $process_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $process_parameter_entry = $process_group_frame->LabEntry(-label => 'Parameter File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_process_par_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left',-fill=>'x', -padx => 3,  -expand => 1);
my $process_parameter_create_button = $process_group_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_par_file_callback,'process'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($process_parameter_create_button, -msg => 'Edit/Create New Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $process_parameter_browse_button = $process_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_par_file_callback,'process'])->pack( -side => 'right');
$main_balloon->attach($process_parameter_browse_button, -msg => 'Open Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $interrupt_frame = $ligbuilder_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 4);
my $interrupt_button = $interrupt_frame->Button(-text => 'Abort', -width => $button_size_w, -command => [\&ligbuilder_buttons_callback,'interrupt'], -state => 'disabled')->pack( -side => 'top',-expand => 1);
$main_balloon->attach($interrupt_button, -msg => 'Interrupt Running Command', -initwait => 400, -balloonposition => 'mouse');

#Ligbuilder V 2.0

my $ligbuilder_20_frame = $lig_frame ->LabFrame(-label => 'Ligbuilder v2')->pack(-side => 'right', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 0);
my $cavity_frame = $ligbuilder_20_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $cavity_button = $cavity_frame->Button(-text => 'Cavity', -width => $button_size_w, -command => [\&ligbuilder_20_buttons_callback,'cavity'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($cavity_button, -msg => 'Run Cavity', -initwait => 400, -balloonposition => 'mouse');
my $cavity_group_frame = $cavity_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $cavity_parameter_entry = $cavity_group_frame->LabEntry(-label => '        Parameter File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_cavity_par_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $cavity_parameter_create_button = $cavity_group_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_par_file_20_callback,'cavity'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($cavity_parameter_create_button, -msg => 'Edit/Create New Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $cavity_parameter_browse_button = $cavity_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_par_file_20_callback,'cavity'])->pack( -side => 'right');
$main_balloon->attach($cavity_parameter_browse_button, -msg => 'Open Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $build_frame = $ligbuilder_20_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $build_button = $build_frame->Button(-text => 'Build', -width => $button_size_w, -command => [\&ligbuilder_20_buttons_callback,'build'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($build_button, -msg => 'Run Build', -initwait => 400, -balloonposition => 'mouse');
my $build_group_frame = $build_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $build_parameter_entry = $build_group_frame->LabEntry(-label => '        Parameter File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_build_par_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $build_parameter_create_button = $build_group_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_par_file_20_callback,'build'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($build_parameter_create_button, -msg => 'Edit/Create New Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $build_parameter_browse_button = $build_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_par_file_20_callback,'build'])->pack( -side => 'right');
$main_balloon->attach($build_parameter_browse_button, -msg => 'Open Parameter File', -initwait => 400, -balloonposition => 'mouse');
my $build_group_f_frame = $build_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $build_function_entry = $build_group_f_frame->BrowseEntry( -label => '         Build Function: ', -labelPack => [ -side => "left" ], -textvariable => \$build_function,-arrowimage=>$down_arrow, -insertofftime => 0);
$build_function_entry->insert("end", "-Normal");
$build_function_entry->insert("end", "-Process");
$build_function_entry->insert("end", "-Filter");
$build_function_entry->insert("end", "-YScore");
$build_function_entry->insert("end", "-Recommend");
$build_function_entry->insert("end", "-Cluster");
$build_function_entry->insert("end", "-Extract");
$build_function_entry->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1);
my $build_fantom_frame = $build_group_f_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $makeindex_output_frame = $ligbuilder_20_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both');
my $makeindex_button = $makeindex_output_frame->Button(-text => 'Create INDEX File for MOL2 DB', -width => $button_size_w, -command => [\&ligbuilder_20_buttons_callback,'index'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($makeindex_button, -msg => 'Create Index File for a MOL2 DB', -initwait => 400, -balloonposition => 'mouse');
my $makeindex_group_frame = $makeindex_output_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $makeindex_output_entry = $makeindex_group_frame->LabEntry(-label => 'MOL2 DB Directory:', -labelPack => [ -side => "left" ], -textvariable => \$selected_build_db_folder, -background => 'white', -insertofftime => 0)->pack(-side => 'left',-fill=>'x', -padx => 3, -expand => 1);
my $makeindex_output_phantom_frame = $makeindex_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $makeindex_output_browse_button = $makeindex_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&browse_par_file_20_callback,'db_dir'])->pack( -side => 'right', -padx => 0);
$main_balloon->attach($makeindex_output_browse_button, -msg => 'Browse for Directory', -initwait => 400, -balloonposition => 'mouse');
my $interrupt_20_frame = $ligbuilder_20_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 5);
my $interrupt_20_button = $interrupt_20_frame->Button(-text => 'Abort', -width => $button_size_w, -command => [\&ligbuilder_20_buttons_callback,'interrupt'], -state => 'disabled')->pack( -side => 'top', -expand => 1);
$main_balloon->attach($interrupt_20_button, -msg => 'Interrupt Running Command', -initwait => 400, -balloonposition => 'mouse');




#pharmacophore buttons

my $pha_frame = $pha_tab_frame->Frame()->pack(-side => 'left', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $pha_button = $pha_frame->Button(-text => "Start the Construct", -width => $button_size_w, -command => [\&pharmacophore_export_button_callback,'construct'])->pack( -side => 'top', -expand => 1);
$main_balloon->attach($pha_button, -msg => 'Start the export', -initwait => 400, -balloonposition => 'mouse');
my $pha_pr_frame = $pha_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 2);
my $pha_input_pr_frame = $pha_pr_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $pha_input_pr_entry = $pha_input_pr_frame->LabEntry(-label => '  Input mol2 Protein File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pha_pr_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3,-expand => 1);
my $pha_input_pr_phantom_frame = $pha_input_pr_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $pha_input_pr_browse_button = $pha_input_pr_frame->Button(-text => '...', -image => $browse_button_image,-command => [\&browse_pha_file_callback,'in1'])->pack( -side => 'right');
$main_balloon->attach($pha_input_pr_browse_button, -msg => 'Open Protein File', -initwait => 400, -balloonposition => 'mouse');
my $pha_li_frame = $pha_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 2);
my $pha_input_li_frame = $pha_li_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
my $pha_input_li_entry = $pha_input_li_frame->LabEntry(-label => '  Input mol2 Ligand File: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pha_li_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3,-expand => 1);
my $pha_input_li_phantom_frame = $pha_input_li_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $pha_input_li_browse_button = $pha_input_li_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&browse_pha_file_callback,'in2'])->pack( -side => 'right');
$main_balloon->attach($pha_input_li_browse_button, -msg => 'Open Ligand File', -initwait => 400, -balloonposition => 'mouse');
my $pha_cp_frame = $pha_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $pha_cp_button = $pha_cp_frame->Button(-text => "Input files", -width => $button_size_w, -command => [\&copy_pr_files_callback])->pack( -side => 'top',-expand => 0);
$main_balloon->attach($pha_cp_button, -msg => 'Input files', -initwait => 400, -balloonposition => 'mouse');
my $pha_interrupt_frame = $pha_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 0);
my $pha_interrupt_button = $pha_interrupt_frame->Button(-text => 'Abort', -width => $button_size_w, -command => [\&pharmacophore_export_button_callback,'interrupt'], -state => 'disabled')->pack( -side => 'top', -expand => 1, -pady => 1);
$main_balloon->attach($pha_interrupt_button, -msg => 'Interrupt Running Command', -initwait => 400, -balloonposition => 'mouse');
my $pha_output = $pha_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'x');
my $pha_out = $pha_output->LabEntry(-label => ' directory: ', -labelPack => [ -side => "left" ], -textvariable => \$pha_save_file_dir, -background => 'white', -insertofftime => 0)->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1);
my $pha_output_phantom_frame = $pha_output->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 14, -pady => 0);
my $pha_output_browse_button = $pha_output->Button(-text => '...', -image => $folder_button_image, -command => [\&browse_pha_save_callback],-state =>'disabled')->pack( -side => 'right');
$main_balloon->attach($pha_output_browse_button, -msg => 'Move output File', -initwait => 400, -balloonposition => 'mouse');
my $pha_mv_frame = $pha_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $pha_mv_button = $pha_mv_frame->Button(-text => "MOVE", -width => $button_size_w, -command => [\&mv_files_callback], -state => 'disabled' )->pack( -side => 'bottom', -fill => 'y', -expand => 0);
$main_balloon->attach($pha_mv_button, -msg => 'Move output files', -initwait => 400, -balloonposition => 'mouse');
my $pha_options_frame = $pha_tab_frame->LabFrame(-label => 'Pharmacophore Options')->pack(-side => 'right', -anchor => 'n', -expand => 2, -fill => 'both', -pady => 1);
my $pha_def_options_frame = $pha_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 0, -fill => 'x', -pady => 1);
my $pha_def_options_fantom_frame = $pha_def_options_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 15, -pady => 0);
my $pha_param_create_button = $pha_def_options_frame->Button(-text => 'New', -image => $parameter_button_image, -command => [\&create_conf_file_callback,'pha'],-state => 'normal')->pack( -side => 'right', -padx => 3);
$main_balloon->attach($pha_param_create_button, -msg => 'Edit/Create New Configuration File', -initwait => 400, -balloonposition => 'mouse');
my $pha_input_pr_name_frame = $pha_options_frame->Frame()->pack(-side => 'top', -anchor => 's', -expand => 1, -fill => 'both', -pady => 4);
my $pha_input_pr_name_entry = $pha_input_pr_name_frame->LabEntry(-label => '  Input Protein File name(.mol2): ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pha_input_pr_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3, -fill => 'x', -expand => 1);
$pha_input_pr_name_entry->insert("end", '.mol2');
my $pha_input_li_name_frame = $pha_options_frame->Frame()->pack(-side => 'top', -anchor => 's', -expand => 1, -fill => 'both', -pady => 4);
my $pha_input_li_name_entry = $pha_input_li_name_frame->LabEntry(-label => '  Input Ligand File name(.mol2): ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pha_input_li_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left',-fill=>'x', -padx => 3, -fill => 'x', -expand => 1);
$pha_input_li_name_entry->insert("end", '.mol2');
my $pha_output_frame = $pha_options_frame->Frame()->pack(-side => 'top', -anchor => 's', -expand => 1, -fill => 'both', -pady => 4);
my $pha_output_entry = $pha_output_frame->LabEntry(-label => '  Output Pharmacophore File name: ', -labelPack => [ -side => "left" ], -textvariable => \$selected_pha_save_file, -background => 'white', -insertofftime => 0)->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1);
my $pha_create_frame = $pha_options_frame->Frame()->pack(-side => 'bottom', -anchor => 'n', -expand => 1, -fill => 'both', -pady => 2);
my $pha_ff_frame = $pha_options_frame->Frame()->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x', -pady => 2);
my $pha_ff_entry = $pha_ff_frame->BrowseEntry( -label => 'Configuration file: ', -labelPack => [ -side => "left" ],-arrowimage=>$down_arrow, -textvariable => \$pharmacophore_ff, -insertofftime => 0);
$pha_ff_entry->insert("end", "default");
$pha_ff_entry->insert("end", "custom");
$pha_ff_entry->pack( -side => 'left', -padx => 3, -fill => 'x', -expand => 1, -pady => 2);
my $pha_create_button = $pha_create_frame->Button(-text => "Create file", -width => $button_size_w, -command => [\&config_file_callback])->pack( -side => 'top', -fill => 'none', -expand => 0);
$main_balloon->attach($pha_create_button, -msg => 'Adjusts the configuration file with the input and output parameters', -initwait => 400, -balloonposition => 'mouse');






# Output Window
my $panedwin2 = $panedwin1 -> Panedwindow(-orient => 'vertical') -> pack( -side => 'bottom', -fill=>'both', -expand => 1, -anchor => 'e');
my $output_frame = $panedwin2->LabFrame(-label => "Output", -labelside => "acrosstop");
my $output_text = $output_frame->Scrolled('ROText',-scrollbars => 'osoe', -wrap=>'none',-borderwidth=> 2,-background=>'white',-foreground=>'black',-relief=>'sunken',-height=>25, -insertofftime => 0)->pack(-side=>'bottom',-anchor => 'nw', -fill => 'both', -expand => 1);
$panedwin2->add($output_frame);
$panedwin1->add($button_frame,$panedwin2);

# Tags
$output_text->tagConfigure('red', -font => "Courier -14 bold", -foreground=>'red');
$output_text->tagConfigure('boldy', -font => "Courier -12 bold", -foreground=>'black');


# Initialization - Load Preferences
&load_preferences();
$mw->deiconify;

# Before quiting do
END {
	if (openhandle(\*LOG)) {
		print LOG "Log file closed at ";
		print LOG scalar(localtime)."\n\n";
		close LOG;
	}
	my $dummy_file = File::Spec->catfile($drugOns_directory,'.tmp','dummy_file');
	system "touch $dummy_file" and die "Creating $dummy_file returned $?.";
	my $to_delete = File::Spec->catfile($drugOns_directory,'.tmp','*');
	system "rm $to_delete" and die "deleting $to_delete returned $?\n";
	print "bye bye\n";
	
}
# Main Loop
MainLoop();


# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	General --- Mainwindow Callbacks & Subroutines																			#
# --------------------------------------------------------------------------------------------------------------#




sub ligbuilder_tab_frame_callback { # NO USE NOW
	my ($opt) = @_;
}

sub positionwindow {
	my($window, $width, $height, $offsetx, $offsety) = @_;
	$window->idletasks;
	my $x = int( $mw->x + ($mw->width / 2) - ($width / 2) + $offsetx);
	my $y = int( $mw->y + ($mw->height / 2) - ($height / 2) + $offsety);
	$window->geometry($width . "x" . $height . "+" . $x . "+" . $y);
}


sub load_preferences {
	if (-e File::Spec->catfile($drugOns_directory,'.default_prefs')){
		open IN, File::Spec->catfile($drugOns_directory,'.default_prefs') or die "error reading default options";
		while (my $line = <IN>)	{
			next if (substr($line,0,1) eq '#' or $line =~ /^[\s\n]*$/);
			chomp $line;
			my @ar = split /\t/, $line;
			if (exists $ar[1]) {
				if ($ar[0] eq 'default_working_directory') {
					$default_working_directory = $ar[1];
					if (defined $default_working_directory) {
						if (-d $default_working_directory) {
							$working_directory = $default_working_directory;
							chdir "$working_directory" or die "Can't cd to $working_directory: $!\n";
						}
					}
				} elsif ($ar[0] eq 'ligbuilder_path') {
					$ligbuilder_path = $ar[1];
				} elsif ($ar[0] eq 'ligbuilder_20_path') {
					$ligbuilder_20_path = $ar[1];
				} elsif ($ar[0] eq 'enable_fun') {
					$enable_fun = $ar[1];
				} elsif ($ar[0] eq 'pdb2pqr_path') {
					$pdb2pqr_path = $ar[1];
				} elsif ($ar[0] eq 'gromacs_path') {
					$gromacs_path = $ar[1];
				} elsif ($ar[0] eq 'pharmACOphore_path') {
					$pharmACOphore_path = $ar[1];	
				} elsif ($ar[0] eq 'enable_log') {
					if ($enable_log != $ar[1]) {
						&log_button_callback;
					}
				} elsif ($ar[0] eq 'default_log_file') {
					$default_log_file = $ar[1];
				} elsif ($ar[0] eq 'default_gromacs_param_file') {
					$default_gromacs_param_file = $ar[1];
					if (defined $default_gromacs_param_file) {
						$selected_mdrun_par_file = $default_gromacs_param_file;
					}
				} elsif ($ar[0] eq 'default_pocket_param_file') {
					$default_pocket_param_file = $ar[1];
					if (defined $default_pocket_param_file) {
						$selected_pocket_par_file = $default_pocket_param_file;
					}
				} elsif ($ar[0] eq 'default_grow_param_file') {
					$default_grow_param_file = $ar[1];
					if (defined $default_grow_param_file) {
						$selected_grow_par_file = $default_grow_param_file;
					}
				} elsif ($ar[0] eq 'default_link_param_file') {
					$default_link_param_file = $ar[1];
					if (defined $default_link_param_file) {
						$selected_link_par_file = $default_link_param_file;
					}
				} elsif ($ar[0] eq 'default_process_param_file') {
					$default_process_param_file = $ar[1];
					if (defined $default_process_param_file) {
						$selected_process_par_file = $default_process_param_file;
					}
				} elsif ($ar[0] eq 'default_cavity_param_file') {
					$default_cavity_param_file = $ar[1];
					if (defined $default_cavity_param_file) {
						$selected_cavity_par_file = $default_cavity_param_file;
					}
				} elsif ($ar[0] eq 'default_build_param_file') {
					$default_build_param_file = $ar[1];
					if (defined $default_build_param_file) {
						$selected_build_par_file = $default_build_param_file;
					}
				} elsif ($ar[0] eq 'preferred_terminal') {
					$preferred_terminal = $ar[1];
				} elsif ($ar[0] eq 'preferred_text_editor') {
					$preferred_text_editor = $ar[1];
				} elsif ($ar[0] eq 'preferred_mol_viewer') {
					$preferred_mol_viewer = $ar[1];
				} elsif ($ar[0] eq 'preferred_graph_viewer') {
					$preferred_graph_viewer = $ar[1];
				} else {}
			}
		}
		close IN;
	} else {
		$mw -> MsgBox( -title => 'Load Preferences', -message=>"File \'.default_prefs\' doesn\'t exist.\nPreferences could not be loaded.\n\nGo to Edit->Preferences and fill in the necessary information.",-type=>'ok',-icon=>'warning')->Show;
	}
}

sub error {
    (my $message, my $signif) = @_;
    
    print "\a";
    $mw->messageBox(-icon => 'error',-message => $message,-default => 'Ok',-title => 'Ups!',-type => 'Ok');
}



sub menu_file_callback {
	my ($opt) = @_;
	
	if ($opt eq 'load') {
		my $types=[];
		push @$types, ['TXT files', '*.txt'];
		push @$types, ['All files', '*'];
		my $file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if ((defined $file) and (-e $file)) {
			open IN, "$file" or die "error reading file \'$file\'!";
			$output_text->insert('end', "\n----------\nLoading File: \'$file\'\n----------\n\n",'red'); $output_text->SetCursor('end');
			foreach my $line (<IN>) {
				$output_text->insert('end', $line); $output_text->SetCursor('end');
			}
			$output_text->insert('end', "\n----------\n\n",'red'); $output_text->SetCursor('end');
			close IN;
		}
	} elsif ($opt eq 'save_as') {
		my $types=[];
		push @$types, ['TXT files', '*.txt'];
		push @$types, ['All files', '*'];
		my $file = $mw->getSaveFile(-filetypes => $types, -initialfile => 'output_window.txt',-initialdir => $working_directory, -multiple => 0);
		if (defined $file) {
			open OUT, '>', $file or die "Cannot create file \'$file\'\n";
			print OUT $output_text->get('1.0','end');
			close OUT;
		}
	} elsif ($opt eq 'print') {
		if (open (my $handle, '|-', 'lp -')) {
			print $handle $output_text->get('1.0','end');
			close $handle;
		}
	} elsif ($opt eq 'clear') {
		$output_text->delete('1.0','end');
	} elsif ($opt eq 'log') {
		&log_button_callback;
	} else {}		
}

sub menu_edit_callback {
	my ($opt) = @_;
	
	if ($opt eq 'fragment') {
		&fragment_db_window();
	} elsif ($opt eq 'parameter_mdrun') {
		&parameter_files_gro_window();
	} elsif ($opt eq 'parameter_lig_1') {
		&parameter_files_window();
	} elsif ($opt eq 'parameter_lig_2') {
		&parameter_files_20_window();
	} elsif ($opt eq 'preferences') {
		&preferences_window();
	} else {}		
}

sub menu_open_callback {
	my ($opt) = @_;
	
	if ($opt eq 'terminal') {
		my $to_run=$preferred_terminal;
		$to_run =~ s/\$working_directory/$working_directory/; # **********
		system "$to_run &" and die "running $to_run returned $?\n";
	} elsif ($opt eq 'text_editor') {
		my $to_run=$preferred_text_editor;
		system "$to_run &" and die "running $to_run returned $?\n";
	} elsif ($opt eq 'molecular_viewer') {
		my $to_run=$preferred_mol_viewer;
		system "$to_run &" and die "running $to_run returned $?\n";
	} else {}		
}

sub menu_help_callback {
	my ($opt) = @_;
	
	if ($opt eq 'help') {
		&help(File::Spec->catfile($drugOns_directory,'Program_Manual/DrugOn_quick_guide.pdf'));
	} elsif ($opt eq 'gromacs_website') {
		&help('http://www.gromacs.org/');
	} elsif ($opt eq 'program_website') {
		&help('http://www.bioacademy.gr/bioinformatics/drugon/');
	} elsif ($opt eq 'ligbuilder_2_0_website') {
		&help('http://ligbuilder.org/');
	} elsif ($opt eq 'pdb2pqr_website') {
		&help('http://www.poissonboltzmann.org/pdb2pqr');
	} elsif ($opt eq 'about') {
		$mw -> MsgBox( -title => 'About', -message=>"DrugOn \n\nBioinformatics & Medical Informatics Team\nBiomedical Research Foundation, Academy of Athens\n\nDeveloped by:\nParaskevas Fakourelis, Eng\n\nScientific supervisors:\nSophia Kossida, DPhil\nDimitrios Vlachakis, PhD\n\n\nPlease send comments and bug reports to: dr Dimitrios Vlachakis (dvlachakis\@bioacademy.gr)\n Paraskevas Fakourelis (fakourelis\@ceid.upatras.gr)",-type=>'ok',-icon=>'info')->Show;
	} elsif ($opt eq 'lig_warn') {
		&help(File::Spec->catfile($drugOns_directory,'Program_Manual/index.html'));
	} elsif ($opt eq 'pdb_website') {
		&help('http://www.pdb.org/pdb/home/home.do');
	} else {}		
}

sub help {
	my ($url) = @_;
	
	BROWSER: {
		for my $dir (split /:/, $ENV{PATH}) {
			for my $browser (qw(firefox mozilla opera konqueror)) {
				my $br_path = File::Spec->catfile($dir,$browser);
				if (-x "$br_path") {
					system "$br_path $url &" and die "running $browser returned $?\n";
					last BROWSER;
				}
			}
		}
	&error("No browser found\n");
	}
}

sub browse_directory_callback {
	my ($opt) = @_;
	
	if ($opt eq 'wd') {
		my $dir = $mw->DirSelect(-title => 'Choose Directory')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$working_directory = $dir;
				chdir "$dir" or die "Can't cd to $dir: $!\n";
			}
		}
	} else {}
}

sub quit_callback {
	my $d = $mw -> MsgBox( -title => 'Quit', -message=>"Really quit?",-type=>'yesno',-icon=>'question');
	my $button = $d->Show;
	if( $button eq "yes" ) {
		$mw->destroy;
	}
}

sub system_buttons_callback {
	my ($opt) = @_;
	
	if ($opt eq 'terminal') {
		my $to_run=$preferred_terminal;
		$to_run =~ s/\$working_directory/$working_directory/;
		system "$to_run &" and die "running $to_run returned $?\n";
	} elsif ($opt eq 'text_editor') {
		my $to_run=$preferred_text_editor." ".$selected_text_editor_files;
		system "$to_run &" and die "running $to_run returned $?\n";
	} elsif ($opt eq 'molecular_viewer') {
		my $to_run=$preferred_mol_viewer." ".$selected_molecular_viewer_files;
		system "sh $to_run &" and die "running $to_run returned $?\n";
	} else {}	
}

sub browse_file_callback {
	my ($opt) = @_;
		
	if ($opt eq 'text') {
		my $types=[];
		push @$types, ['Text files', '*.txt'];
		push @$types, ['All files', '*'];
		my $files;
		$files = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 5);
		if (defined $files) {
			$selected_text_editor_files = "@$files";
		}
	} elsif ($opt eq 'mol') {
		my $types=[];
		push @$types, ['PDB files', '*.pdb'];
		push @$types, ['MOL2 files', '*.mol2'];
		push @$types, ['All files', '*'];
		my $files;
		$files = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 5);
		if (defined $files) {
			$selected_molecular_viewer_files = "@$files";
		}
	} else {}	
}

sub PBcolor {
    my @colors = (  0, '#00FF00',  1, '#00FF00',  2, '#00FF00',  3, '#00FF00',  4, '#00FF00',
		    5, '#00FF00',  6, '#00FF00',  7, '#00FF00',  8, '#00FF00',  9, '#00FF00',
		    10, '#00FF00', 11, '#00FF00', 12, '#00FF00', 13, '#00FF00', 14, '#00FF00',
		    15, '#00FF00', 16, '#00FF00', 17, '#00FF00', 18, '#00FF00', 19, '#00FF00',
		    20, '#00FF00', 21, '#00FF00', 22, '#00FF00', 23, '#00FF00', 24, '#00FF00',
		    25, '#00FF00', 26, '#00FF00', 27, '#00FF00', 28, '#00FF00', 29, '#00FF00',
		    30, '#00FF00', 31, '#00FF00', 32, '#00FF00', 33, '#00FF00', 34, '#00FF00',
		    35, '#00FF00', 36, '#00FF00', 37, '#00FF00', 38, '#00FF00', 39, '#00FF00',
		    40, '#00FF00', 41, '#00FF00', 42, '#00FF00', 43, '#00FF00', 44, '#00FF00',
		    45, '#00FF00', 46, '#00FF00', 47, '#00FF00', 48, '#00FF00', 49, '#00FF00',
		    50, '#00FF00', 51, '#00FF00', 52, '#00FF00', 53, '#00FF00', 54, '#00FF00',
		    55, '#00FF00', 56, '#00FF00', 57, '#00FF00', 58, '#00FF00', 59, '#00FF00',
		    60, '#00FF00', 61, '#00FF00', 62, '#00FF00', 63, '#00FF00', 64, '#00FF00',
		    65, '#00FF00', 66, '#00FF00', 67, '#00FF00', 68, '#00FF00', 69, '#00FF00',
		    70, '#00FF00', 71, '#00FF00', 72, '#00FF00', 73, '#00FF00', 74, '#00FF00',
		    75, '#00FF00', 76, '#00FF00', 77, '#00FF00', 78, '#00FF00', 79, '#00FF00',
		    80, '#00FF00', 81, '#00FF00', 82, '#00FF00', 83, '#00FF00', 84, '#00FF00',
		    85, '#00FF00', 86, '#00FF00', 87, '#00FF00', 88, '#00FF00', 89, '#00FF00',
		    90, '#00FF00', 91, '#00FF00', 92, '#00FF00', 93, '#00FF00', 94, '#00FF00',
		    95, '#00FF00', 96, '#00FF00', 97, '#00FF00', 98, '#00FF00', 99, '#00FF00' );
    return \@colors;
}

sub log_button_callback {
	if ($enable_log==0) {
		if ($default_log_file) {
			my $oldfh = select(LOG); $| = 1; select($oldfh);
			if (-e $default_log_file) {
				open LOG, ">>$default_log_file" 
					or ($mw -> MsgBox( -title => 'Error Opening Log File', -message=>"Log File could not be created/opened.\nPlease check for syntax errors at the appropriate entry in the Preferences Window.\nAuto Logging is turned Off.", -type=>'ok',-icon=>'warning')->Show && die "error opening $default_log_file $!");
					print LOG "Log file restarted at ";
					print LOG scalar(localtime)."\n\n";
			} else {
				open LOG, ">$default_log_file"
					or ($mw -> MsgBox( -title => 'Error Opening Log File', -message=>"Log File could not be created/opened.\nPlease check for syntax errors at the appropriate entry in the Preferences Window.\nAuto Logging is turned Off.", -type=>'ok',-icon=>'warning')->Show && die "error creating $default_log_file $!");
				print LOG "DrugOn\n\nLog file started at ";
				print LOG scalar(localtime)."\n\n";
				$mw->update;
			}
			$enable_log=1;
			$oldfh = select(LOG); $| = 0; select($oldfh);
			$log_icon->configure(-image=> $on_image);
		} else {
			$mw -> MsgBox( -title => 'Error Opening Log File', -message=>"Log File $default_log_file could not be created/opened.\nPlease check for syntax errors at the appropriate entry in the Preferences Window.\nAuto Logging is turned Off.", -type=>'ok',-icon=>'warning')->Show;
		}
	} else {
		print LOG "Log file closed at ";
		print LOG scalar(localtime)."\n\n";
		$enable_log=0;
		close LOG;
		$log_icon->configure(-image=> $off_image);
	}
}

sub log_refresh_callback {
	my ($enable_log_checkbox) = @_;
	$enable_log_checkbox->toggle;
	&log_button_callback;
}

sub create_stats {
    my %stats;
    open(my $stat_fh,"< /proc/stat") or die("Unable To Open /proc/stat: $!\n");
    while (my @line = split " ", readline($stat_fh)){
	last unless (substr($line[0],0,3) eq 'cpu');
	$prev_stats{$line[0]}= [@line[1..3]];
    }
    @perc = sort keys %prev_stats;
    $cpus = (keys %prev_stats) - 1;
    seek $stat_fh,0,0;
    my $counter=0;
    my $lxs  = Sys::Statistics::Linux::MemStats->new;
    my $mem_stat = $lxs->get;
    $mem_usage = $mem_usage = 100 - $mem_stat->{realfreeper};;
    $swap_usage = $mem_stat->{swapusedper};
    $mw->repeat($refresh_rate,[\&update_stats,$stat_fh, \%prev_stats]);
}

sub update_stats {
    (my $stat_fh, my $previous) = @_;
    my %stats;
    while (my @line = split " ", readline($stat_fh)) {
	last unless (substr($line[0],0,3) eq 'cpu');
	$stats{$line[0]}= [@line[1..3]];
    }
    seek $stat_fh,0,0;
    my $interval = tv_interval(\@time);
    my $counter = 0;
    foreach (sort keys %stats) {
        my $total = 0;
        my $percent;
	for (my $counter = 0;$counter<3;$counter++) {
	    my $diff = $stats{$_}->[$counter] - $previous->{$_}->[$counter];
	    $total += $diff;
	}
        $percent = sprintf("%3.2f%%",min 100,$total/($interval*$cpus)) if $counter == 0;
        $percent = sprintf("%3.2f%%",min 100,$total/$interval) if $counter != 0;
        $perc[$counter]=$percent;
	$counter++;
    }
    $cpu_label->configure(-text=>$perc[0]);
    @time = gettimeofday;
    %prev_stats = %stats;
    my $lxs  = Sys::Statistics::Linux::MemStats->new;
    my $mem_stat = $lxs->get;
    $mem_usage = 100 - $mem_stat->{realfreeper};
    $swap_usage = $mem_stat->{swapusedper};

}

sub toggle_running_icon {
	if ($running_bulb == 0) {
		$pb__bulb_label->configure(-image => $bulb_on_image);
		$running_bulb = 1;
	} else {
		$pb__bulb_label->configure(-image => $bulb_off_image);
		$running_bulb = 0;
	}
}




# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Preferences Window																			#
# --------------------------------------------------------------------------------------------------------------#


sub preferences_window {
	my $pw = $mw->Toplevel(-title => 'Preferences');
	$pw->withdraw;
	$pw->grab();
	$pw->transient($mw);
	$pw->iconimage($program_image);
	my $pw_options_frame = $pw->Frame()->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $pw_buttons_frame = $pw->Frame()->pack(-side => 'bottom', -fill=>'none',-expand=>0);
	my $pw_options_left_frame = $pw_options_frame->Frame()->pack(-side => 'left', -fill=>'both',-expand=>1);
	my $pw_options_right_frame = $pw_options_frame->Frame()->pack(-side => 'right', -fill=>'both',-expand=>1);
	my $pw_general_options_frame = $pw_options_left_frame->LabFrame(-label => "General Settings", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both', -expand => 1, -anchor => 'nw');
	my $pw_path_frame = $pw_options_right_frame->LabFrame(-label => "Path Settings", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both', -expand => 1, -anchor => 'nw');
	my $pw_param_frame = $pw_options_right_frame->LabFrame(-label => "Default Parameter Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both', -expand => 1, -anchor => 'nw');
	my $pw_sys_options_frame = $pw_options_left_frame->LabFrame(-label => "Preferred Applications", -labelside => "acrosstop")->pack(-side => 'bottom', -fill=>'both', -expand => 1, -anchor => 'nw');
	my $enable_log_frame = $pw_general_options_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $enable_log_label = $enable_log_frame->Label(-text => '      Enable Auto Logging:', -font => ['Courier', '10', 'bold'])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0);
	my $enable_log_checkbox = $enable_log_frame->Checkbutton( -variable => \$enable_log)->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
	$enable_log_checkbox->configure(-command => [\&log_refresh_callback, $enable_log_checkbox]);
	my $pw_log_file_group_frame = $pw_general_options_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both',-padx => 5);
	my $pw_log_file_entry = $pw_log_file_group_frame->LabEntry(-label => '            Log File Name:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_log_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_log_file_browse_button = $pw_log_file_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_file_callback,'log',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_log_file_browse_button, -msg => 'Open/Create Log File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_pdb2pqr_group_frame = $pw_path_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_pdb2pqr_entry = $pw_pdb2pqr_group_frame->LabEntry(-label => '      PDB2PQR Folder Path:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pdb2pqr_path, -state => 'readonly', -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_pdb2pqr_browse_button = $pw_pdb2pqr_group_frame->Button(-text => '...', -image => $folder_button_image, -command => [\&pw_browse_directory_callback,'pdb2pqr_path',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_pdb2pqr_browse_button, -msg => 'Browse for PDB2PQR Folder Path', -initwait => 800, -balloonposition => 'mouse');
	my $pw_ligbuilder_group_frame = $pw_path_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_ligbuilder_entry = $pw_ligbuilder_group_frame->LabEntry(-label => 'LigBuilder v1 Folder Path:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$ligbuilder_path, -state => 'readonly', -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_ligbuilder_browse_button = $pw_ligbuilder_group_frame->Button(-text => '...', -image => $folder_button_image, -command => [\&pw_browse_directory_callback,'ligb_path',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_ligbuilder_browse_button, -msg => 'Browse for LigBuilder v1 Folder Path', -initwait => 800, -balloonposition => 'mouse');
	my $pw_ligbuilder_20_group_frame = $pw_path_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_ligbuilder_20_entry = $pw_ligbuilder_20_group_frame->LabEntry(-label => 'LigBuilder v2 Folder Path:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$ligbuilder_20_path, -state => 'readonly', -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_ligbuilder_20_browse_button = $pw_ligbuilder_20_group_frame->Button(-text => '...', -image => $folder_button_image, -command => [\&pw_browse_directory_callback,'ligb_20_path',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_ligbuilder_20_browse_button, -msg => 'Browse for LigBuilder v2 Folder Path', -initwait => 800, -balloonposition => 'mouse');
	my $pw_pharmACOphore_group_frame = $pw_path_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_pharmACOphore_entry = $pw_pharmACOphore_group_frame->LabEntry(-label => 'PharmACOphore Folder Path:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pharmACOphore_path, -state => 'readonly', -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_pharmACOphore_browse_button = $pw_pharmACOphore_group_frame->Button(-text => '...', -image => $folder_button_image, -command => [\&pw_browse_directory_callback,'pharm_path',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_pharmACOphore_browse_button, -msg => 'Browse for PharmACOphore Folder Path', -initwait => 800, -balloonposition => 'mouse');
	my $pw_gromacs_param_group_frame = $pw_param_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_gromacs_param_entry = $pw_gromacs_param_group_frame->LabEntry(-label => '                    MDRun:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_gromacs_param_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_gromacs_param_browse_button = $pw_gromacs_param_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_par_file_callback,'gromacs',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_gromacs_param_browse_button, -msg => 'Open Parameter File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_pocket_param_group_frame = $pw_param_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_pocket_param_entry = $pw_pocket_param_group_frame->LabEntry(-label => '                   Pocket:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_pocket_param_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_pocket_param_browse_button = $pw_pocket_param_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_par_file_callback,'pocket',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_pocket_param_browse_button, -msg => 'Open Parameter File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_grow_param_group_frame = $pw_param_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_grow_param_entry = $pw_grow_param_group_frame->LabEntry(-label => '                     Grow:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_grow_param_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_grow_param_browse_button = $pw_grow_param_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_par_file_callback,'grow',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_grow_param_browse_button, -msg => 'Open Parameter File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_link_param_group_frame = $pw_param_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_link_param_entry = $pw_link_param_group_frame->LabEntry(-label => '                     Link:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_link_param_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_link_param_browse_button = $pw_link_param_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_par_file_callback,'link',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_link_param_browse_button, -msg => 'Open Parameter File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_process_param_group_frame = $pw_param_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_process_param_entry = $pw_process_param_group_frame->LabEntry(-label => '                  Process:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_process_param_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_process_param_browse_button = $pw_process_param_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_par_file_callback,'process',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_process_param_browse_button, -msg => 'Open Parameter File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_cavity_param_group_frame = $pw_param_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_cavity_param_entry = $pw_cavity_param_group_frame->LabEntry(-label => '                   Cavity:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_cavity_param_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_cavity_param_browse_button = $pw_cavity_param_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_par_file_callback,'cavity',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_cavity_param_browse_button, -msg => 'Open Parameter File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_build_param_group_frame = $pw_param_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_build_param_entry = $pw_build_param_group_frame->LabEntry(-label => '                    Build:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$default_build_param_file, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_build_param_browse_button = $pw_build_param_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_par_file_callback,'build',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_build_param_browse_button, -msg => 'Open Parameter File', -initwait => 800, -balloonposition => 'mouse');
	my $pw_terminal_group_frame = $pw_sys_options_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_terminal_entry = $pw_terminal_group_frame->LabEntry(-label => '                 Terminal:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$preferred_terminal, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_terminal_browse_button = $pw_terminal_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_file_callback,'term',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_terminal_browse_button, -msg => 'Browse for Terminal Executable', -initwait => 800, -balloonposition => 'mouse');
	my $pw_text_editor_group_frame = $pw_sys_options_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_text_editor_entry = $pw_text_editor_group_frame->LabEntry(-label => '              Text Editor:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$preferred_text_editor, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_text_editor_browse_button = $pw_text_editor_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_file_callback,'text',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_text_editor_browse_button, -msg => 'Browse for Text Editor Executable', -initwait => 800, -balloonposition => 'mouse');
	my $pw_mol_viewer_group_frame = $pw_sys_options_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_mol_viewer_entry = $pw_mol_viewer_group_frame->LabEntry(-label => '         Molecular Viewer:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$preferred_mol_viewer, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_mol_viewer_browse_button = $pw_mol_viewer_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_file_callback,'mol',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_mol_viewer_browse_button, -msg => 'Browse for Molecular Viewer Executable', -initwait => 800, -balloonposition => 'mouse');
	my $pw_graph_viewer_group_frame = $pw_sys_options_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $pw_graph_viewer_entry = $pw_graph_viewer_group_frame->LabEntry(-label => '         XVG Graph Viewer:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$preferred_graph_viewer, -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $pw_graph_viewer_browse_button = $pw_graph_viewer_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&pw_browse_file_callback,'graph',$pw])->pack( -side => 'right');
	$main_balloon->attach($pw_graph_viewer_browse_button, -msg => 'Browse for Graph Viewer Executable', -initwait => 800, -balloonposition => 'mouse');
	my $pw_save_button = $pw_buttons_frame->Button(-text => 'Save', -command => [\&pw_button_callback,'save',$pw])->pack( -side => 'left', -padx => 1);
	my $pw_restore_button = $pw_buttons_frame->Button(-text => 'Load Defaults', -command => [\&pw_button_callback,'restore',$pw])->pack( -side => 'left', -padx => 1);
	my $pw_close_button = $pw_buttons_frame->Button(-text => 'Close', -command => [\&pw_button_callback,'close',$pw])->pack( -side => 'left', -padx => 1);
	$pw->deiconify;
}

sub pw_browse_directory_callback {
	my ($opt,$pw) = @_;
	
	if ($opt eq 'd_working_dir') {
		my $dir = $pw->DirSelect(-title => 'Choose Default Directory')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$default_working_directory = $dir;
			}
		}
	} elsif ($opt eq 'ligb_path') {
		my $dir = $pw->DirSelect(-title => 'Path to Ligbuilder v1')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$ligbuilder_path = $dir;
			}
		}
	} elsif ($opt eq 'ligb_20_path') {
		my $dir = $pw->DirSelect(-title => 'Path to Ligbuilder v2')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$ligbuilder_20_path = $dir;
			}
		}
	} elsif ($opt eq 'pdb2pqr_path') {
		my $dir = $pw->DirSelect(-title => 'Path to PDB2PQR')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$pdb2pqr_path = $dir;
			}
		}
	} elsif ($opt eq 'gromacs_path') {
		my $dir = $pw->DirSelect(-title => 'Path to Gromacs')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$gromacs_path = $dir;
			}
		}
	} elsif ($opt eq 'pharm_path') {
		my $dir = $pw->DirSelect(-title => 'Path to PharmACOphore')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$pharmACOphore_path = $dir;
			}
		}
		
	
	} else {}
}

sub pw_browse_par_file_callback {
	my ($opt,$pw) = @_;
	
	my $types=[];
	push @$types, ['Parameter files', '*.index'];
	push @$types, ['All files', '*'];
	my $types_20=[];
	push @$types_20, ['Parameter files', '*.input'];
	push @$types_20, ['All files', '*'];
	my $types_mdp=[];
	push @$types_mdp, ['Parameter files', '*.mdp'];
	push @$types_mdp, ['All files', '*'];
	my $file;
	if ($opt eq 'pocket') {
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$default_pocket_param_file = $file;
		}
	} elsif ($opt eq 'grow') {
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$default_grow_param_file = $file;
		}
	} elsif ($opt eq 'link') {
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$default_link_param_file = $file;
		}
	} elsif ($opt eq 'process') {
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$default_process_param_file = $file;
		}
	} elsif ($opt eq 'cavity') {
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types_20, -multiple => 0);
		if (defined $file) {
			$default_cavity_param_file = $file;
		}
	} elsif ($opt eq 'build') {
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types_20, -multiple => 0);
		if (defined $file) {
			$default_build_param_file = $file;
		}
	} elsif ($opt eq 'gromacs') {
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types_mdp, -multiple => 0);
		if (defined $file) {
			$default_gromacs_param_file = $file;
		}
	} else {}
}

sub pw_browse_file_callback {
	my ($opt,$pw) = @_;
		
	if ($opt eq 'term') {
		my $types=[];
		push @$types, ['All files', '*'];
		my $file;
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types);
		if (defined $file) {
			$preferred_terminal = $file;
		}
	} elsif ($opt eq 'text') {
		my $types=[];
		push @$types, ['All files', '*'];
		my $file;
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types);
		if (defined $file) {
			$preferred_text_editor = $file;
		} 
	} elsif ($opt eq 'mol') {
		my $types=[];
		push @$types, ['All files', '*'];
		my $file;
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$preferred_mol_viewer = $file;
		} 
	} elsif ($opt eq 'graph') {
		my $types=[];
		push @$types, ['All files', '*'];
		my $file;
		$file = $pw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$preferred_graph_viewer = $file;
		} 
	} elsif ($opt eq 'log') {
		my $types=[];
		push @$types, ['LOG files', '*.log'];
		push @$types, ['All files', '*'];
		my $file = $mw->getSaveFile(-filetypes => $types, -initialfile => 'drugOn_log.log',-initialdir => $working_directory, -multiple => 0);
		if (defined $file) {
			$default_log_file=$file;
		}
	}
	else {}
}

sub pw_button_callback {
	my ($opt,$pw) = @_;
	
	if ($opt eq 'save') {
		open OUT, '>', File::Spec->catfile($drugOns_directory,'.default_prefs') or die "error writing default options";
		print OUT "#\n# Program Preferences\n#\n\n";
		print OUT "default_working_directory\t$default_working_directory\n";
		print OUT "enable_fun\t$enable_fun\n";
		print OUT "default_log_file\t$default_log_file\n";
		print OUT "enable_log\t$enable_log\n";
		print OUT "pdb2pqr_path\t$pdb2pqr_path\n";
		print OUT "gromacs_path\t$gromacs_path\n";
		print OUT "ligbuilder_path\t$ligbuilder_path\n";
		print OUT "ligbuilder_20_path\t$ligbuilder_20_path\n";
		print OUT "pharmACOphore_path\t$pharmACOphore_path\n";
		print OUT "default_gromacs_param_file\t$default_gromacs_param_file\n";
		print OUT "default_pocket_param_file\t$default_pocket_param_file\n";
		print OUT "default_grow_param_file\t$default_grow_param_file\n";
		print OUT "default_link_param_file\t$default_link_param_file\n";
		print OUT "default_process_param_file\t$default_process_param_file\n";
		print OUT "default_cavity_param_file\t$default_cavity_param_file\n";
		print OUT "default_build_param_file\t$default_build_param_file\n";
		print OUT "preferred_terminal\t$preferred_terminal\n";
		print OUT "preferred_text_editor\t$preferred_text_editor\n";
		print OUT "preferred_mol_viewer\t$preferred_mol_viewer\n";
		print OUT "preferred_graph_viewer\t$preferred_graph_viewer\n";
		close OUT;
		$pw -> MsgBox( -title => 'Save Preferences', -message=>"Preferences were successfully saved.",-type=>'ok',-icon=>'info')->Show;
	} elsif ($opt eq 'restore') {
		if (-e File::Spec->catfile($drugOns_directory,'.default_prefs.r')) {
			open IN, File::Spec->catfile($drugOns_directory,'.default_prefs.r') or die "error reading default options";
			while (my $line = <IN>)	{
				next if (substr($line,0,1) eq '#' or $line =~ /^[\s\n]*$/);
				chomp $line;
				my @ar = split /\t/, $line;
				if (exists $ar[1]) {
					if ($ar[0] eq 'default_working_directory') {
						$default_working_directory = $ar[1];
					} elsif ($ar[0] eq 'ligbuilder_path') {
						$ligbuilder_path = $ar[1];
					} elsif ($ar[0] eq 'ligbuilder_20_path') {
						$ligbuilder_20_path = $ar[1];
					} elsif ($ar[0] eq 'pharmACOphore_path') {
						$pharmACOphore_path = $ar[1];							
					} elsif ($ar[0] eq 'pdb2pqr_path') {
						$pdb2pqr_path = $ar[1];
					} elsif ($ar[0] eq 'gromacs_path') {
						$gromacs_path = $ar[1];
					} elsif ($ar[0] eq 'enable_log') {
						if ($enable_log != $ar[1]) {
							&log_button_callback;
						}
					} elsif ($ar[0] eq 'default_log_file') {
						$default_log_file = $ar[1];
					} elsif ($ar[0] eq 'enable_fun') {
						$enable_fun = $ar[1];
					} elsif ($ar[0] eq 'default_gromacs_param_file') {
						$default_gromacs_param_file = $ar[1];
					} elsif ($ar[0] eq 'default_pocket_param_file') {
						$default_pocket_param_file = $ar[1];
					} elsif ($ar[0] eq 'default_grow_param_file') {
						$default_grow_param_file = $ar[1];
					} elsif ($ar[0] eq 'default_link_param_file') {
						$default_link_param_file = $ar[1];
					} elsif ($ar[0] eq 'default_process_param_file') {
						$default_process_param_file = $ar[1];
					} elsif ($ar[0] eq 'default_cavity_param_file') {
						$default_cavity_param_file = $ar[1];
					} elsif ($ar[0] eq 'default_build_param_file') {
						$default_build_param_file = $ar[1];
					} elsif ($ar[0] eq 'preferred_terminal') {
						$preferred_terminal = $ar[1];
					} elsif ($ar[0] eq 'preferred_text_editor') {
						$preferred_text_editor = $ar[1];
					} elsif ($ar[0] eq 'preferred_mol_viewer') {
						$preferred_mol_viewer = $ar[1];
					} elsif ($ar[0] eq 'preferred_graph_viewer') {
						$preferred_graph_viewer = $ar[1];
					} else {}
				}
			}
			close IN;
		} else {
			$pw -> MsgBox( -title => 'Restore Default Preferences', -message=>"File \'.default_prefs.r\' doesn\'t exist.\nDefault Preferences could not be loaded.",-type=>'ok',-icon=>'warning')->Show;
		}
	} elsif ($opt eq 'close') {
		$pw->destroy;
	} else {}
}


# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Handle Databases																				#
# --------------------------------------------------------------------------------------------------------------#


sub fragment_db_window {
	$fragment_db_path = "";
	
	my $dw = $mw->Toplevel(-title => 'Handle Databases');
	$dw->withdraw;
	$dw->grab();
	$dw->transient($mw);
	$dw->iconimage($program_image);
	&positionwindow($dw,'780','450','-5','-35');
	my $dw_options_frame = $dw->Frame()->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $dw_buttons_frame = $dw->Frame()->pack(-side => 'bottom', -fill=>'none',-expand=>0);
	my $dw_db_options_frame = $dw_options_frame->LabFrame(-label => "Database", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both', -expand => 1, -anchor => 'nw');
	my $dw_db_group_frame = $dw_db_options_frame->Frame()->pack( -side => 'top', -anchor => 'nw',  -fill => 'x', -expand => 0, -padx => 5, -pady => 5);
	my $dw_db_entry = $dw_db_group_frame->LabEntry(-label => 'Database Folder Path:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$fragment_db_path, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 0, -fill => 'x', -expand => 1);
	my $dw_db_browse_button = $dw_db_group_frame->Button(-text => '...', -image => $folder_button_image)->pack( -side => 'right');
	$main_balloon->attach($dw_db_browse_button, -msg => 'Browse for Database Folder Path', -initwait => 800, -balloonposition => 'mouse');
	my $dw_info_group_frame = $dw_db_options_frame->Frame()->pack( -side => 'top', -anchor => 'nw', -fill => 'x', -expand => 0, -padx => 5, -pady => 5);
	my $dw_info_label = $dw_info_group_frame->Label(-text => '  Database Information:', -font => ['Courier', '10', 'bold'])->pack(-side => "left", -anchor => 'nw');
	my $dw_info_text = $dw_info_group_frame->Scrolled('Text', -scrollbars => 'osoe', '-wrap','none','-background', 'white','-foreground', 'black','-relief', 'sunken','-height', 4, '-width', 10, -insertofftime => 0)->pack( -side => 'left',-anchor => 'nw', -fill => 'x', -expand => 1);
	my $dw_info_group_phantom_frame = $dw_info_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $dw_mol_group_frame = $dw_db_options_frame->Frame()->pack( -side => 'top', -anchor => 'nw', -fill => 'both', -expand => 1, -padx => 5, -pady => 3);
	my $dw_mol_label = $dw_mol_group_frame->Label(-text => '             Molecules:', -font => ['Courier', '10', 'bold'])->pack(-side => "left", -anchor => 'nw');

	my $arrayVar = {};
	my $dw_table = $dw_mol_group_frame->Scrolled('Spreadsheet', -rows => 1, -cols => 4,-variable => $arrayVar, -titlerows => 1, -titlecols => 0, -selectmode => 'extended', -selecttype => 'row', -rowstretchmode => 'none');
        $main_balloon->attach($dw_table, -msg => 'Database Entries', -initwait => 800, -balloonposition => 'mouse');
        $dw_table->pack(-side => "left", -expand => 1, -fill => 'both'); 
        $dw_table->colWidth( 0 => 10, 1 => 20, 2 => 12, 3 => 30);
	$dw_table->set('0,0' => "Fragment ID");
	$dw_table->set('0,1' => "Fragment File Name");
	$dw_table->set('0,2' => "Possibility");
	$dw_table->set('0,3' => "Description");
	$dw_table->tagConfigure('dis_col', -state => 'disabled');
	$dw_table->tagCol('dis_col', 0,1);
	my $dw_buttons_mol_frame = $dw_mol_group_frame->Frame()->pack(-side => 'right', -fill=>'none',-expand=>0);
	my $dw_mol_add_button = $dw_buttons_mol_frame->Button(-text => 'Add', -image => $plus_button_image, -command => [\&dw_button_callback,'add',$dw,$dw_table], -state => 'disabled')->pack( -side => 'top', -padx => 1);
        $main_balloon->attach($dw_mol_add_button, -msg => 'Add New Entry', -initwait => 800, -balloonposition => 'mouse');
	my $dw_mol_rem_button = $dw_buttons_mol_frame->Button(-text => 'Remove', -image => $minus_button_image, -command => [\&dw_button_callback,'remove',$dw,$dw_table], -state => 'disabled')->pack( -side => 'top', -padx => 1);
        $main_balloon->attach($dw_mol_rem_button, -msg => 'Remove Selected Entry(ies)', -initwait => 800, -balloonposition => 'mouse');
	my $dw_mol_view_button = $dw_buttons_mol_frame->Button(-text => 'View', -image => $molecular_viewer_button_image, -command => [\&dw_button_callback,'view',$dw,$dw_table], -state => 'disabled')->pack( -side => 'top', -padx => 1);
        $main_balloon->attach($dw_mol_view_button, -msg => 'View Molecule(s) with Molecular Viewer', -initwait => 800, -balloonposition => 'mouse');
	my $dw_mol_view_plus_button = $dw_buttons_mol_frame->Button(-text => 'View', -image => $molecular_viewer_plus_button_image, -command => [\&dw_button_callback,'view_plus',$dw,$dw_table], -state => 'disabled')->pack( -side => 'top', -padx => 1);
        $main_balloon->attach($dw_mol_view_plus_button, -msg => 'View Molecule(s) plus other with Molecular Viewer', -initwait => 800, -balloonposition => 'mouse');
	my $dw_concat_button = $dw_buttons_mol_frame->Button(-text => 'Concat', -image => $concat_image, -command => [\&dw_button_callback,'concat',$dw,$dw_table], -state => 'disabled')->pack( -side => 'top', -padx => 1);
        $main_balloon->attach($dw_concat_button, -msg => 'Create Multiple-MOL2 file from selected molecules', -initwait => 800, -balloonposition => 'mouse');
	my $dw_new_button = $dw_buttons_frame->Button(-text => 'New')->pack( -side => 'left', -padx => 1);
	my $dw_save_button = $dw_buttons_frame->Button(-text => 'Save', -command => [\&dw_button_callback,'save',$dw,$dw_table,$dw_info_text], -state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $dw_save_as_button = $dw_buttons_frame->Button(-text => 'Save As', -command => [\&dw_button_callback,'save_as',$dw,$dw_table,$dw_info_text], -state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $dw_delete_button = $dw_buttons_frame->Button(-text => 'Delete', -state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $dw_close_button = $dw_buttons_frame->Button(-text => 'Close', -command => [\&dw_button_callback,'close',$dw,$dw_table,$dw_info_text])->pack( -side => 'left', -padx => 1);
	$dw_db_browse_button->configure(-command => [\&dw_browse_directory_callback,'fragment_db_path',$dw,$dw_info_text,$dw_table,$dw_mol_add_button,$dw_mol_rem_button,$dw_mol_view_button,$dw_mol_view_plus_button,$dw_concat_button,$dw_save_button,$dw_save_as_button,$dw_delete_button]);
	$dw_new_button->configure( -command => [\&dw_button_callback,'new',$dw,$dw_table,$dw_info_text,$dw_mol_add_button,$dw_mol_rem_button,$dw_mol_view_button,$dw_mol_view_plus_button,$dw_concat_button,$dw_save_button,$dw_save_as_button,$dw_delete_button]);
	$dw_delete_button->configure( -command => [\&dw_button_callback,'delete',$dw,$dw_table,$dw_info_text,$dw_mol_add_button,$dw_mol_rem_button,$dw_mol_view_button,$dw_mol_view_plus_button,$dw_concat_button,$dw_save_button,$dw_save_as_button,$dw_delete_button]);
	$dw->deiconify;
}

sub dw_browse_directory_callback {
	my ($opt,$dw,$dw_info_text,$dw_table,$dw_mol_add_button,$dw_mol_rem_button,$dw_mol_view_button,$dw_mol_view_plus_button,$dw_concat_button,$dw_save_button,$dw_save_as_button,$dw_delete_button) = @_;
	
	if ($opt eq 'fragment_db_path') {
		my $dir = $dw->DirSelect(-title => 'Choose Database Directory')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				unless (-e File::Spec->catfile($dir,"INDEX")) {
					$dw -> MsgBox( -title => 'Error loading Database', -message=>"Directory \'$dir\' doesn\'t appear to be a valid database folder.\nINDEX File could not be located.", -type=>'ok',-icon=>'warning')->Show;
				} else {
					$fragment_db_path = $dir;
					open IN, File::Spec->catfile($dir,"INDEX") or die "Error trying to open database INDEX file!";
					my $enter=0;
					my $after_col=0;
					my $temp = "";
					$dw_info_text->delete('1.0','end');
					my $fragment_count = 0;
					$dw_table->insertRows(0);
					$dw_table->deleteCols(0,$dw_table->cget(-cols));
					foreach my $line (<IN>) {
						if ($line =~ /^#/ && $enter == 0) {
							if ((my $col_name) = ($line =~ /column: (.*)/)) {
								$temp .= $line;
								$after_col = 1;
								$dw_table->insertCols($dw_table->cget(-cols),1);
								my $col_num = $dw_table->cget(-cols) - 2;
								$dw_table->set("0,$col_num" => "$col_name");
							} else {}
							$temp .= $line if ($after_col == 0);
						} elsif ($line =~ /^#/ && $enter == 1) {
							next;
						} else {
							chomp($line);
							if ($line =~ /^\s*$/) {
								next;
							}
							my @ar;
							@ar = split(/\s+/, $line,($dw_table->cget(-cols) - 1));
							$enter =1;
							my $count=$dw_table->cget(-rows);
							$dw_table->insertRows($count,1);
							$fragment_count++;
							my $count2 = 0;
							foreach (@ar) {
								$dw_table->set("$fragment_count,$count2" => $_);
								$count2++;
							}
						}
					}
					chomp($temp);
					$dw_info_text->insert('end', "$temp"); 
					$dw_table->deleteRows($fragment_count+1,$dw_table->cget(-rows)-$fragment_count);
					$dw_table->deleteCols($dw_table->cget(-cols),1);
					$dw_table->tagCol('dis_col', 0,1);
					$dw_table->clearSizes();
					$dw_mol_add_button->configure(-state => 'normal');
					$dw_mol_rem_button->configure(-state => 'normal');
					$dw_mol_view_button->configure(-state => 'normal');
					$dw_mol_view_plus_button->configure(-state => 'normal');
					$dw_concat_button->configure(-state => 'normal');
					$dw_save_button->configure(-state => 'normal');
					$dw_save_as_button->configure(-state => 'normal');
					$dw_delete_button->configure(-state => 'normal');
					close IN;
				}
			}
		}
	} else {}
}


sub dw_button_callback {
	my ($opt,$dw,$dw_table,$dw_info_text,$dw_mol_add_button,$dw_mol_rem_button,$dw_mol_view_button,$dw_mol_view_plus_button,$dw_concat_button,$dw_save_button,$dw_save_as_button,$dw_delete_button) = @_;
	
	if ($opt eq 'new') {
		my $dir = $dw->DirSelect(-title => 'Create New Database Directory')->Show($working_directory);
		if (defined $dir) {
			my $d_box = $dw->Toplevel(-title => 'Edit Header/Create Columns');
			$d_box->grab();
			$d_box->transient($dw);
			$d_box->iconimage($program_image);
			&positionwindow($d_box,'350','250','-5','-35');
			my $d_box__text_frame = $d_box->Frame()->pack(-side => 'top', -fill=>'both',-expand=>1);
			my $d_box_buttons_frame = $d_box->Frame()->pack(-side => 'bottom', -fill=>'none',-expand=>0);
			my $d_box_label = $d_box__text_frame->Label(-text => '  Enter text here:', -font => ['Courier', '10', 'bold'])->pack(-side => "top", -anchor => 'n');
			my $d_box_text = $d_box__text_frame->Scrolled('Text', -scrollbars => 'osoe', '-wrap','none','-background', 'white','-foreground', 'black','-relief', 'sunken','-height', 4, '-width', 10, -insertofftime => 0)->pack( -side => 'left',-anchor => 'nw', -fill => 'both', -expand => 1);
			$d_box_text->insert('end',$dw_info_text->get('1.0','end'));
			my $text = '';
			my $d_box_create_button = $d_box_buttons_frame->Button(-text => 'Create', -command =>sub{$text = $d_box_text->get('1.0','end');$d_box->destroy})->pack( -side => 'left', -padx => 1);
			my $d_box_cancel_button = $d_box_buttons_frame->Button(-text => 'Cancel', -command =>sub{$text = 'cancel';$d_box->destroy})->pack( -side => 'left', -padx => 1);
			$dw->waitVariable(\$text);
			unless ($text eq 'cancel') {
				chomp($text);
				$dw_mol_add_button->configure(-state => 'normal');
				$dw_mol_rem_button->configure(-state => 'normal');
				$dw_mol_view_button->configure(-state => 'normal');
				$dw_mol_view_plus_button->configure(-state => 'normal');
				$dw_concat_button->configure(-state => 'normal');
				$dw_save_button->configure(-state => 'normal');
				$dw_save_as_button->configure(-state => 'normal');
				$dw_delete_button->configure(-state => 'normal');
				unless (-d $dir) {
					system "mkdir $dir" and die "creating directory $dir returned $?\n";
				}
				$fragment_db_path = $dir;
				open OUT, '>', File::Spec->catfile($fragment_db_path,'INDEX') or die "Error trying to open database INDEX file $!";
				print OUT $text;
				print OUT "#\n";
				close OUT;
				open IN, File::Spec->catfile($fragment_db_path,'INDEX') or die "Error trying to open database INDEX file!";
				my $temp = "";
				$dw_info_text->delete('1.0','end');
				my $fragment_count = 0;
				$dw_table->insertRows(0);
				$dw_table->deleteCols(0,$dw_table->cget(-cols));
				foreach my $line (<IN>) {
					if ($line =~ /^#/) {
						if ((my $col_name) = ($line =~ /column: (.*)/)) {
							$dw_table->insertCols($dw_table->cget(-cols),1);
							my $col_num = $dw_table->cget(-cols) - 2;
							$dw_table->set("0,$col_num" => "$col_name");
						} else {}
						$temp .= $line;										
					}
				}
				chomp($temp);
				$dw_info_text->insert('end', "$temp"); 
				$dw_table->deleteRows($fragment_count+1,$dw_table->cget(-rows)-$fragment_count);
				$dw_table->deleteCols($dw_table->cget(-cols),1);
				$dw_table->tagCol('dis_col', 0,1);
				$dw_table->clearSizes();
			}
		}
	} elsif ($opt eq 'save') {
		open OUT, '>', File::Spec->catfile($fragment_db_path,'INDEX') or die "Error trying to open database INDEX file $!";
		print OUT $dw_info_text->get('1.0','end');
		print OUT "#\n";
		for (my $i=1;$i < $dw_table->cget(-rows);$i++) {
				my $tempstr = "";
				for (my $j=0;$j < $dw_table->cget(-cols);$j++) {
					$tempstr .= $dw_table->get("$i,$j")."\t";
				}
				$tempstr =~ s/\t$/\n/;
				print OUT $tempstr;
			}
		close OUT;
		$dw -> MsgBox( -title => 'Save Database', -message=>"Database INDEX file was successfully saved.",-type=>'ok',-icon=>'info')->Show;
	} elsif ($opt eq 'save_as') {
		my $dir = $dw->DirSelect(-title => 'Create New Database Directory')->Show($working_directory);
		if (defined $dir) {
			unless (-d $dir) {
				system "mkdir $dir" and die "creating directory $dir returned $?\n";
			}
			system "cp $fragment_db_path/* $dir" and die "copying files from $fragment_db_path to $dir returned $?\n";
			open OUT, '>', File::Spec->catfile($dir,'INDEX') or die "Error trying to open database INDEX file $!";
			print OUT $dw_info_text->get('1.0','end');
			print OUT "#\n";
			for (my $i=1;$i < $dw_table->cget(-rows);$i++) {
				my $tempstr = "";
				for (my $j=0;$j < $dw_table->cget(-cols);$j++) {
					$tempstr .= $dw_table->get("$i,$j")."\t";
				}
				$tempstr =~ s/\t$/\n/;
				print OUT $tempstr;
			}
			close OUT;
			$fragment_db_path = $dir;
			$dw -> MsgBox( -title => 'Save Database', -message=>"Database was successfully saved in the new directory.",-type=>'ok',-icon=>'info')->Show;
		}
	} elsif ($opt eq 'delete') {
		my $d = $dw -> MsgBox( -title => 'Delete Database', -message=>"This function will permanently delete ALL the files AND the directory of the database, including any files that recide in that folder but do not belong to the database.\nAre you sure you want to proceed?",-type=>'yesno',-icon=>'warning');
		my $button = $d->Show;
		if( $button eq "yes" ) {
			system "rm $fragment_db_path/*" and die "deleting files from directory $fragment_db_path returned $?\n";
			system "rmdir $fragment_db_path" and die "deleting directory $fragment_db_path returned $?\n";
			$fragment_db_path="";
			$dw_info_text->delete('1.0','end');
			$dw_table->deleteRows(1,$dw_table->cget(-rows)) if $dw_table->cget(-rows) >= 2;;
			$dw_table->configure(-cols => 4);
			$dw_table->colWidth( 0 => 10, 1 => 20, 2 => 12, 3 => 30);
			$dw_table->set('0,0' => "Fragment ID");
			$dw_table->set('0,1' => "Fragment File Name");
			$dw_table->set('0,2' => "Possibility");
			$dw_table->set('0,3' => "Description");
			$dw_table->tagConfigure('dis_col', -state => 'disabled');
			$dw_table->tagCol('dis_col', 0,1);
			$dw_mol_add_button->configure(-state => 'disabled');
			$dw_mol_rem_button->configure(-state => 'disabled');
			$dw_mol_view_button->configure(-state => 'disabled');
			$dw_mol_view_plus_button->configure(-state => 'disabled');
			$dw_concat_button->configure(-state => 'disabled');
			$dw_save_button->configure(-state => 'disabled');
			$dw_save_as_button->configure(-state => 'disabled');
			$dw_delete_button->configure(-state => 'disabled');
		}
	} elsif ($opt eq 'close') {
		$fragment_db_path = "";
		$dw->destroy;
	}  elsif ($opt eq 'add') {
		my $types=[];
		push @$types, ['MOL2 files', '*.mol2'];
		push @$types, ['All files', '*'];
		my $file;
		$file = $dw->getOpenFile( -title => 'Select MOL2 File to Add',-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			system "cp $file $fragment_db_path &" and die "copying $file to $fragment_db_path returned $?\n";
			(undef,undef,my $filename) = File::Spec->splitpath( $file );
			my $fragment_count=$dw_table->cget(-rows);
			$dw_table->insertRows($fragment_count,1);
			$dw_table->set("$fragment_count,0" => $fragment_count);
			$dw_table->set("$fragment_count,1" => $filename);
		}
	} elsif ($opt eq 'remove') {
		my @sel = $dw_table->curselection();
		if (@sel) {
			my @selected_array=map { my @temp=split(/,/,$_,2); $temp[0] } @sel;
			my @uniqu = uniq(@selected_array);
			my @unique = sort {$b <=> $a} @uniqu;
			my $que = $dw -> MsgBox( -title => 'Delete Entry(s)', -message=>"Do you want to permanently delete the corresponding MOL2 file(s)?",-type=>'yesnocancel',-icon=>'question');
			my $ans = $que->Show;
			if ( $ans eq "yes" ) {
				foreach my $selected (@unique) {
					my $file = $dw_table->get("$selected,1");
					system "rm $fragment_db_path/$file &" and die "deleting $file from $fragment_db_path returned $?\n";
					$dw_table->deleteRows($selected,1);
				}
				for (my $i=1;$i < $dw_table->cget(-rows);$i++) {
					$dw_table->set("$i,0" => $i);
				}
			} elsif ( $ans eq "no" ) {
				foreach my $selected (@unique) {
					$dw_table->deleteRows($selected,1);
				}
				for (my $i=1;$i < $dw_table->cget(-rows);$i++) {
					$dw_table->set("$i,0" => $i);
				}
			} else {}
		}
	} elsif ($opt eq 'view') {
		my @sel = $dw_table->curselection();
		if (@sel) {
			my @selected_array=map { my @temp=split(/,/,$_,2); $temp[0] } @sel;
			my @unique = uniq(@selected_array);
			my @files;
			foreach my $selected (@unique) {
				push @files, File::Spec->catfile($fragment_db_path,$dw_table->get("$selected,1"));
			}
			my $to_run=$preferred_mol_viewer." "."@files";
			print $to_run;
			system "$to_run &" and die "running $to_run returned $?\n";
		} else {
			$dw -> MsgBox( -title => 'Select Molecule', -message=>"You must select at least one line in the table before proceeding.",-type=>'ok',-icon=>'info')->Show;
		}
	} elsif ($opt eq 'view_plus') {
		my $types=[];
		push @$types, ['PDB files', '*.pdb'];
		push @$types, ['MOL2 files', '*.mol2'];
		push @$types, ['All files', '*'];
		my $files_plus;
		$files_plus = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 5);
		if (defined $files_plus) {
			my @sel = $dw_table->curselection();
			if (@sel) {
				my @selected_array=map { my @temp=split(/,/,$_,2); $temp[0] } @sel;
				my @unique = uniq(@selected_array);
				my @files;
				foreach my $selected (@unique) {
					push @files, File::Spec->catfile($fragment_db_path,$dw_table->get("$selected,1"));
				}
				my $to_run=$preferred_mol_viewer." "."@files"." "."@$files_plus";
				print $to_run;
				system "$to_run &" and die "running $to_run returned $?\n";
			} else {
				$dw -> MsgBox( -title => 'Select Molecule', -message=>"You must select at least one line in the table before proceeding.",-type=>'ok',-icon=>'info')->Show;
			}
		} 
	} elsif ($opt eq 'concat') {
		my $types=[];
		push @$types, ['MOL2 files', '*.mol2'];
		push @$types, ['All files', '*'];
		my $files_plus;
		$files_plus = $mw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $files_plus) {
			my @sel = $dw_table->curselection();
			if (@sel) {
				my @selected_array=map { my @temp=split(/,/,$_,2); $temp[0] } @sel;
				my @unique = uniq(@selected_array);
				my @files;
				foreach my $selected (@unique) {
					push @files, File::Spec->catfile($fragment_db_path,$dw_table->get("$selected,1"));
				}
				system "cat @files > $files_plus" and die "Concatenating files returned $?.";
				$dw -> MsgBox( -title => 'Concatenate files', -message=>"$files_plus was successfully created.",-type=>'ok',-icon=>'info')->Show;
			} else {
				$dw -> MsgBox( -title => 'Select Molecule', -message=>"You must select at least one line in the table before proceeding.",-type=>'ok',-icon=>'info')->Show;
			}
		}
	} else {}	
}


# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Parameter Windows																				#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Receptor Optimization																				#
# --------------------------------------------------------------------------------------------------------------#


sub parameter_files_gro_window {
	my ($default_tab) = @_;
	
	my ($edit_mdrun_file);
	my $fw_gro = $mw->Toplevel(-title => 'Edit Parameter Files');
	$fw_gro->withdraw;
	$fw_gro->grab();
	$fw_gro->transient($mw);
	$fw_gro->iconimage($program_image);
	&positionwindow($fw_gro,'500','620','-5','-35');
	my $fw_gro_options_frame = $fw_gro->LabFrame(-label => "Parameter Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_gro_buttons_frame = $fw_gro->Frame()->pack(-side => 'bottom', -fill=>'none',-expand=>0);
	my $fw_gro_tabbed_frame = $fw_gro_options_frame->NoteBook(-dynamicgeometry => 0)->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_gro_mdrun_frame = $fw_gro_tabbed_frame->add('Gromacs MDRun', -label => 'mdrun');
	my $fw_gro_filename_mdrun_frame = $fw_gro_mdrun_frame->Frame()->pack(-side => 'top', -fill=>'x',-expand=>0, -padx => 10);
	my $fw_gro_filename_mdrun_entry = $fw_gro_filename_mdrun_frame->LabEntry(-label => 'Parameter File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_mdrun_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_gro_filename_mdrun_browse_button = $fw_gro_filename_mdrun_frame->Button(-text => '...', -image => $browse_button_image)->pack( -side => 'right');
	my $fw_gro_text_mdrun_frame = $fw_gro_mdrun_frame->LabFrame(-label => "Text Editor", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_gro_text_mdrun = $fw_gro_text_mdrun_frame->Scrolled('Text', -scrollbars => 'osoe', '-wrap','none','-background', 'white','-foreground', 'black','-relief', 'sunken', -insertofftime => 0)->pack(-fill => 'both', -expand => 1);
	my $fw_gro_new_button = $fw_gro_buttons_frame->Button(-text => 'New')->pack( -side => 'left', -padx => 1);
	my $fw_gro_save_button = $fw_gro_buttons_frame->Button(-text => 'Save',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_gro_save_as_button = $fw_gro_buttons_frame->Button(-text => 'Save As',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_gro_delete_button = $fw_gro_buttons_frame->Button(-text => 'Delete',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_gro_close_button = $fw_gro_buttons_frame->Button(-text => 'Close', -command => [\&fw_gro_button_callback,'close',$fw_gro])->pack( -side => 'left', -padx => 1);
	$fw_gro_filename_mdrun_browse_button->configure(-command => [\&fw_gro_browse_file_callback,'mdrun',$fw_gro,\$edit_mdrun_file,$fw_gro_save_button,$fw_gro_save_as_button,$fw_gro_delete_button,$fw_gro_text_mdrun]);
	$fw_gro_new_button->configure(-command => [\&fw_gro_button_callback,'new',$fw_gro,$fw_gro_tabbed_frame,$fw_gro_save_button,$fw_gro_save_as_button,$fw_gro_delete_button,\$edit_mdrun_file,$fw_gro_text_mdrun]);
	$fw_gro_save_button->configure(-command => [\&fw_gro_button_callback,'save',$fw_gro,$fw_gro_tabbed_frame,$fw_gro_save_button,$fw_gro_save_as_button,$fw_gro_delete_button,\$edit_mdrun_file,$fw_gro_text_mdrun]);
	$fw_gro_save_as_button->configure(-command => [\&fw_gro_button_callback,'save_as',$fw_gro,$fw_gro_tabbed_frame,$fw_gro_save_button,$fw_gro_save_as_button,$fw_gro_delete_button,\$edit_mdrun_file,$fw_gro_text_mdrun]);
	$fw_gro_delete_button->configure(-command => [\&fw_gro_button_callback,'delete',$fw_gro,$fw_gro_tabbed_frame,$fw_gro_save_button,$fw_gro_save_as_button,$fw_gro_delete_button,\$edit_mdrun_file,$fw_gro_text_mdrun]);
										
	if (defined $default_tab) {
		if ($selected_mdrun_par_file && -e $selected_mdrun_par_file) {			
			$edit_mdrun_file = $selected_mdrun_par_file;
			&populate_mdrun(\$edit_mdrun_file,$fw_gro_text_mdrun);
			$fw_gro_save_button->configure(-state=>'normal');
			$fw_gro_save_as_button->configure(-state=>'normal');
			$fw_gro_delete_button->configure(-state=>'normal');
		}	
	}
	$fw_gro->deiconify;
}

sub fw_gro_browse_file_callback {
	my ($opt,$fw,$edit_file_ref,$fw_save_button,$fw_save_as_button,$fw_delete_button,$fw_gro_text_mdrun) = @_;

	my $types=[];
	push @$types, ['Parameter files', '*.mdp'];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'mdrun') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_mdrun(\$file,$fw_gro_text_mdrun);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}		
	} else {}
}

sub populate_mdrun {
	my ($file_ref,$fw_gro_text_mdrun) = @_;
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	$fw_gro_text_mdrun->delete('1.0','end');
	foreach my $line (<IN>) {
		$fw_gro_text_mdrun->insert('end',$line);
	}
	close IN;
}

sub fw_gro_button_callback {
	my ($opt,$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,$edit_mdrun_file_ref,$fw_gro_text_mdrun) = @_;
	
	if ($opt eq 'new') {
		my $types=[];
		push @$types, ['Parameter files', '*.mdp'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Create New Parameter File', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
			# my $temp_var = $fw_tabbed_frame->raised();
			# if ($temp_var eq 'mdrun') {
				$fw_gro_text_mdrun->delete('1.0','end');
				$$edit_mdrun_file_ref = $file;	
			# } else {print "what?"}
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} elsif ($opt eq 'save') {
		# my $temp_var = $fw_tabbed_frame->raised();
		# if ($temp_var eq 'mdrun') {
			open OUT, '>', "$$edit_mdrun_file_ref" or die "Error trying to open $$edit_mdrun_file_ref. $!";
			print OUT $fw_gro_text_mdrun->Contents();
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_mdrun_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		# } else {}
	} elsif ($opt eq 'save_as') {
		my $types=[];
		push @$types, ['Parameter files', '*.input'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Save Parameter File As', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
			# my $temp_var = $fw_tabbed_frame->raised();
			# if ($temp_var eq 'mdrun') {
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT $fw_gro_text_mdrun->Contents();
				close OUT;
				$$edit_mdrun_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
			# } else {}
		}
	} elsif ($opt eq 'delete') {
		# my $temp_var = $fw_tabbed_frame->raised();
		# if ($temp_var eq 'mdrun') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_mdrun_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_mdrun_file_ref" and die "deleting $$edit_mdrun_file_ref returned $?\n";
				$$edit_mdrun_file_ref = '';
				$fw_gro_text_mdrun->delete('1.0','end');	
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		# } else {}
	} elsif ($opt eq 'close') {
		$fw->destroy;
	} else {}
}


# --------------------------------------------------------------------------------------------------------------#
#	Build Ligands - LigBuilder 1.2																				#
# --------------------------------------------------------------------------------------------------------------#


sub parameter_files_window {
	my ($default_tab) = @_;
	
	my ($edit_pocket_file, $edit_grow_file, $edit_link_file, $edit_process_file);
	my @pocket_param_values; my @grow_param_values; my @link_param_values; my @process_param_values;
	my $fw = $mw->Toplevel(-title => 'Edit Parameter Files');
	$fw->withdraw;
	$fw->grab();
	$fw->transient($mw);
	$fw->iconimage($program_image);
	&positionwindow($fw,'500','620','-5','-35');
	my $fw_options_frame = $fw->LabFrame(-label => "Parameter Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_buttons_frame = $fw->Frame()->pack(-side => 'bottom', -fill=>'none',-expand=>0);
	my $fw_tabbed_frame = $fw_options_frame->NoteBook(-dynamicgeometry => 0)->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_pocket_frame = $fw_tabbed_frame->add('Pocket', -label => 'Pocket');
	my $fw_grow_frame = $fw_tabbed_frame->add('Grow', -label => 'Grow');
	my $fw_link_frame = $fw_tabbed_frame->add('Link', -label => 'Link');
	my $fw_process_frame = $fw_tabbed_frame->add('Process', -label => 'Process');
	# pocket frame
	my $fw_filename_pocket_frame = $fw_pocket_frame->Frame()->pack(-side => 'top', -fill=>'both',-expand=>1, -padx => 10);
	my $fw_filename_pocket_entry = $fw_filename_pocket_frame->LabEntry(-label => 'Parameter File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_pocket_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_filename_pocket_browse_button = $fw_filename_pocket_frame->Button(-text => '...', -image => $browse_button_image)->pack( -side => 'right');
	my $fw_if_pocket_frame = $fw_pocket_frame->LabFrame(-label => "Input Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_of_pocket_frame = $fw_pocket_frame->LabFrame(-label => "Output Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_kisap_pocket_frame = $fw_pocket_frame->LabFrame(-label => "Key Interaction Sites and Pharmacophore", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_pocket_1_group_frame = $fw_if_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_1_entry = $fw_pocket_1_group_frame->LabEntry(-label => '           Receptor File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[1], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_1_browse_button = $fw_pocket_1_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_pdb',\$pocket_param_values[1],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_1_browse_button, -msg => 'Open - PDB File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_2_group_frame = $fw_if_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_2_entry = $fw_pocket_2_group_frame->LabEntry(-label => '             Ligand File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[2], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_2_browse_button = $fw_pocket_2_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_mol2',\$pocket_param_values[2],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_2_browse_button, -msg => 'Open - MOL2 File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_3_group_frame = $fw_if_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_3_entry = $fw_pocket_3_group_frame->LabEntry(-label => '     Parameter Directory:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[3], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_3_browse_button = $fw_pocket_3_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$pocket_param_values[3],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_3_browse_button, -msg => 'Browse for Parameter Directory', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_4_group_frame = $fw_of_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_4_entry = $fw_pocket_4_group_frame->LabEntry(-label => '        Pocket Atom File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[4], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_4_browse_button = $fw_pocket_4_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_txt',\$pocket_param_values[4],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_4_browse_button, -msg => 'Save as - TXT File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_5_group_frame = $fw_of_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_5_entry = $fw_pocket_5_group_frame->LabEntry(-label => '        Pocket Grid File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[5], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_5_browse_button = $fw_pocket_5_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_txt',\$pocket_param_values[5],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_5_browse_button, -msg => 'Save as - TXT File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_6_group_frame = $fw_kisap_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_6_entry = $fw_pocket_6_group_frame->LabEntry(-label => '           Key Site File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[6], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_6_browse_button = $fw_pocket_6_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_pdb',\$pocket_param_values[6],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_6_browse_button, -msg => 'Save as - PDB File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_7_group_frame = $fw_kisap_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_7_entry = $fw_pocket_7_group_frame->LabEntry(-label => '  Pharmacophore TXT File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[7], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_7_browse_button = $fw_pocket_7_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_txt',\$pocket_param_values[7],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_7_browse_button, -msg => 'Save as - TXT File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_8_group_frame = $fw_kisap_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_8_entry = $fw_pocket_8_group_frame->LabEntry(-label => '  Pharmacophore PDB File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[8], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_8_browse_button = $fw_pocket_8_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_pdb',\$pocket_param_values[8],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_pocket_8_browse_button, -msg => 'Save as - PDB File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_pocket_9_group_frame = $fw_kisap_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_9_entry = $fw_pocket_9_group_frame->LabEntry(-label => 'Minimal Feature Distance:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[9], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_9_phantom_frame = $fw_pocket_9_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_pocket_10_group_frame = $fw_kisap_pocket_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_pocket_10_entry = $fw_pocket_10_group_frame->LabEntry(-label => '  Maximal Feature Number:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$pocket_param_values[10], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_pocket_10_phantom_frame = $fw_pocket_10_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	# grow frame
	my $fw_left_grow_frame = $fw_grow_frame->Frame()->pack(-side => 'left', -fill=>'both',-expand=>1);
	my $fw_right_grow_frame = $fw_grow_frame->Frame();
	my $fw_grow_more_button = $fw_grow_frame->Button()->pack(-side => 'right',-fill=>'none',-expand=>0, -padx => 1);
	$fw_grow_more_button->configure(-text => 'more', -image => $arrow_button_image, -command => [\&fw_more_less_callback,'grow',$fw_right_grow_frame,$fw_grow_more_button,$fw]);
	my $fw_filename_grow_frame = $fw_left_grow_frame->Frame()->pack(-side => 'top', -fill=>'both',-expand=>1, -padx => 10);
	my $fw_filename_grow_entry = $fw_filename_grow_frame->LabEntry(-label => 'Parameter File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_grow_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_filename_grow_browse_button = $fw_filename_grow_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_browse_file_callback,'grow'])->pack( -side => 'right');
	my $fw_if_grow_frame = $fw_left_grow_frame->LabFrame(-label => "Input Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_ffd_grow_frame = $fw_left_grow_frame->LabFrame(-label => "Force Field Directory", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_fl_grow_frame = $fw_left_grow_frame->LabFrame(-label => "Fragment Libraries", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_scp_grow_frame = $fw_right_grow_frame->LabFrame(-label => "Structural Construction Parameters", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_cvr_grow_frame = $fw_right_grow_frame->LabFrame(-label => "Chemical Viability Rules", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_gap_grow_frame = $fw_left_grow_frame->LabFrame(-label => "Genetic Algorithm Parameters", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_of_grow_frame = $fw_left_grow_frame->LabFrame(-label => "Output Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_grow_1_group_frame = $fw_if_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_1_entry = $fw_grow_1_group_frame->LabEntry(-label => '           Seed Ligand File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[1], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_1_browse_button = $fw_grow_1_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_mol2',\$grow_param_values[1],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_1_browse_button, -msg => 'Open - MOL2 File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_2_group_frame = $fw_if_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_2_entry = $fw_grow_2_group_frame->LabEntry(-label => '           Pocket Atom File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[2], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_2_browse_button = $fw_grow_2_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_txt',\$grow_param_values[2],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_2_browse_button, -msg => 'Open - TXT File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_3_group_frame = $fw_if_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_3_entry = $fw_grow_3_group_frame->LabEntry(-label => '           Pocket Grid File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[3], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_3_browse_button = $fw_grow_3_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_txt',\$grow_param_values[3],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_3_browse_button, -msg => 'Open - TXT File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_4_group_frame = $fw_ffd_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_4_entry = $fw_grow_4_group_frame->LabEntry(-label => '        Parameter Directory:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[4], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_4_browse_button = $fw_grow_4_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$grow_param_values[4],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_4_browse_button, -msg => 'Browse for Parameter Directory', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_5_group_frame = $fw_fl_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_5_entry = $fw_grow_5_group_frame->LabEntry(-label => '     Building Block Library:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[5], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_5_browse_button = $fw_grow_5_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$grow_param_values[5],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_5_browse_button, -msg => 'Browse for Building Block Library', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_6_group_frame = $fw_fl_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_6_entry = $fw_grow_6_group_frame->LabEntry(-label => 'Forbidden Structure Library:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[6], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_6_browse_button = $fw_grow_6_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$grow_param_values[6],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_6_browse_button, -msg => 'Browse for Forbidden Structure Library', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_7_group_frame = $fw_fl_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_7_entry = $fw_grow_7_group_frame->LabEntry(-label => '    Toxic Structure Library:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[7], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_7_browse_button = $fw_grow_7_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$grow_param_values[7],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_7_browse_button, -msg => 'Browse for Toxic Structure Library', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_8_group_frame = $fw_scp_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_8_entry = $fw_grow_8_group_frame->LabEntry(-label => '            Growing Probability:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[8], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_grow_8_phantom_frame = $fw_grow_8_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_8_scale = $fw_grow_8_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$grow_param_values[8],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	my $fw_grow_9_group_frame = $fw_scp_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_9_entry = $fw_grow_9_group_frame->LabEntry(-label => '            Linking Probability:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[9], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_grow_9_phantom_frame = $fw_grow_9_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_9_scale = $fw_grow_9_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$grow_param_values[9],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	my $fw_grow_10_group_frame = $fw_scp_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_10_entry = $fw_grow_10_group_frame->LabEntry(-label => '           Mutation Probability:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[10], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_grow_10_phantom_frame = $fw_grow_10_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_10_scale = $fw_grow_10_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$grow_param_values[10],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	$grow_param_values[11] = 1;
	my $fw_grow_11_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 8);
	my $fw_grow_11_label = $fw_grow_11_group_frame->Label(-text => '           Apply Chemical Rules:', -font => ['Courier', '10', 'bold'])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0);
	my $fw_grow_11_checkbox = $fw_grow_11_group_frame->Checkbutton( -variable => \$grow_param_values[11])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
	$grow_param_values[12] = 1;
	my $fw_grow_12_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 8);
	my $fw_grow_12_label = $fw_grow_12_group_frame->Label(-text => 'Apply Forbidden Structure Check:', -font => ['Courier', '10', 'bold'])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0);
	my $fw_grow_12_checkbox = $fw_grow_12_group_frame->Checkbutton( -variable => \$grow_param_values[12])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
	$grow_param_values[13] = 1;
	my $fw_grow_13_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 8);
	my $fw_grow_13_label = $fw_grow_13_group_frame->Label(-text => '    Apply Toxic Structure Check:', -font => ['Courier', '10', 'bold'])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0);
	my $fw_grow_13_checkbox = $fw_grow_13_group_frame->Checkbutton( -variable => \$grow_param_values[13])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
	my $fw_grow_14_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_14_entry = $fw_grow_14_group_frame->LabEntry(-label => '       Maximal Molecular Weight:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[14], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_14_phantom_frame = $fw_grow_14_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_15_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_15_entry = $fw_grow_15_group_frame->LabEntry(-label => '       Minimal Molecular Weight:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[15], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_15_phantom_frame = $fw_grow_15_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_16_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_16_entry = $fw_grow_16_group_frame->LabEntry(-label => '                   Maximal LogP:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[16], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_16_phantom_frame = $fw_grow_16_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_17_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_17_entry = $fw_grow_17_group_frame->LabEntry(-label => '                   Minimal LogP:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[17], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_17_phantom_frame = $fw_grow_17_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_18_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_18_entry = $fw_grow_18_group_frame->LabEntry(-label => '         Maximal HB Donor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[18], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_18_phantom_frame = $fw_grow_18_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_19_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_19_entry = $fw_grow_19_group_frame->LabEntry(-label => '         Minimal HB Donor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[19], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_19_phantom_frame = $fw_grow_19_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_20_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_20_entry = $fw_grow_20_group_frame->LabEntry(-label => '      Maximal HB Acceptor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[20], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_20_phantom_frame = $fw_grow_20_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_21_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_21_entry = $fw_grow_21_group_frame->LabEntry(-label => '      Minimal HB Acceptor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[21], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_21_phantom_frame = $fw_grow_21_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_22_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_22_entry = $fw_grow_22_group_frame->LabEntry(-label => '                    Maximal PKD:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[22], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_22_phantom_frame = $fw_grow_22_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_23_group_frame = $fw_cvr_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_23_entry = $fw_grow_23_group_frame->LabEntry(-label => '                    Minimal PKD:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[23], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_23_phantom_frame = $fw_grow_23_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_24_group_frame = $fw_gap_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_24_entry = $fw_grow_24_group_frame->LabEntry(-label => '      Number of Generations:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[24], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_24_phantom_frame = $fw_grow_24_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_25_group_frame = $fw_gap_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_25_entry = $fw_grow_25_group_frame->LabEntry(-label => '       Number of Population:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[25], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_25_phantom_frame = $fw_grow_25_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_26_group_frame = $fw_gap_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_26_entry = $fw_grow_26_group_frame->LabEntry(-label => '          Number of Parents:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[26], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_26_phantom_frame = $fw_grow_26_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_27_group_frame = $fw_gap_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_27_entry = $fw_grow_27_group_frame->LabEntry(-label => '              Elitism Ratio:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[27], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_grow_27_phantom_frame = $fw_grow_27_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_27_scale = $fw_grow_27_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$grow_param_values[27],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	my $fw_grow_28_group_frame = $fw_gap_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_28_entry = $fw_grow_28_group_frame->LabEntry(-label => '          Similarity Cutoff:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[28], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_grow_28_phantom_frame = $fw_grow_28_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_grow_28_scale = $fw_grow_28_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$grow_param_values[28],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	my $fw_grow_29_group_frame = $fw_of_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_29_entry = $fw_grow_29_group_frame->LabEntry(-label => '     Population Record File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[29], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_29_browse_button = $fw_grow_29_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_lig',\$grow_param_values[29],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_29_browse_button, -msg => 'Save as - LIG File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_grow_30_group_frame = $fw_of_grow_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_grow_30_entry = $fw_grow_30_group_frame->LabEntry(-label => '     Ligand Collection File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$grow_param_values[30], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_grow_30_browse_button = $fw_grow_30_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_lig',\$grow_param_values[30],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_grow_30_browse_button, -msg => 'Save as - LIG File', -initwait => 800, -balloonposition => 'mouse');
	# link frame
	my $fw_left_link_frame = $fw_link_frame->Frame()->pack(-side => 'left', -fill=>'both',-expand=>1);
	my $fw_right_link_frame = $fw_link_frame->Frame();
	my $fw_link_more_button = $fw_link_frame->Button()->pack(-side => 'right',-fill=>'none',-expand=>0, -padx => 1);
	$fw_link_more_button->configure(-text => 'more', -image => $arrow_button_image, -command => [\&fw_more_less_callback,'link',$fw_right_link_frame,$fw_link_more_button,$fw]);
	my $fw_filename_link_frame = $fw_left_link_frame->Frame()->pack(-side => 'top', -fill=>'both',-expand=>1, -padx => 10);
	my $fw_filename_link_entry = $fw_filename_link_frame->LabEntry(-label => 'Parameter File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_link_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_filename_link_browse_button = $fw_filename_link_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_browse_file_callback,'link'])->pack( -side => 'right');
	my $fw_if_link_frame = $fw_left_link_frame->LabFrame(-label => "Input Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_ffd_link_frame = $fw_left_link_frame->LabFrame(-label => "Force Field Directory", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_fl_link_frame = $fw_left_link_frame->LabFrame(-label => "Fragment Libraries", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_sbup_link_frame = $fw_right_link_frame->LabFrame(-label => "Structural Building Up Parameters", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_cvr_link_frame = $fw_right_link_frame->LabFrame(-label => "Chemical Viability Rules", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_gap_link_frame = $fw_left_link_frame->LabFrame(-label => "Genetic Algorithm Parameters", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_of_link_frame = $fw_left_link_frame->LabFrame(-label => "Output Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_link_1_group_frame = $fw_if_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_1_entry = $fw_link_1_group_frame->LabEntry(-label => '           Seed Ligand File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[1], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_1_browse_button = $fw_link_1_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_mol2',\$link_param_values[1],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_1_browse_button, -msg => 'Open - MOL2 File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_2_group_frame = $fw_if_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_2_entry = $fw_link_2_group_frame->LabEntry(-label => '           Pocket Atom File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[2], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_2_browse_button = $fw_link_2_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_txt',\$link_param_values[2],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_2_browse_button, -msg => 'Open - TXT File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_3_group_frame = $fw_if_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_3_entry = $fw_link_3_group_frame->LabEntry(-label => '           Pocket Grid File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[3], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_3_browse_button = $fw_link_3_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_txt',\$link_param_values[3],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_3_browse_button, -msg => 'Open - TXT File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_4_group_frame = $fw_ffd_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_4_entry = $fw_link_4_group_frame->LabEntry(-label => '        Parameter Directory:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[4], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_4_browse_button = $fw_link_4_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$link_param_values[4],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_4_browse_button, -msg => 'Browse for Parameter Directory', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_5_group_frame = $fw_fl_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_5_entry = $fw_link_5_group_frame->LabEntry(-label => '     Building Block Library:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[5], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_5_browse_button = $fw_link_5_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$link_param_values[5],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_5_browse_button, -msg => 'Browse for Building Block Library', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_6_group_frame = $fw_fl_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_6_entry = $fw_link_6_group_frame->LabEntry(-label => 'Forbidden Structure Library:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[6], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_6_browse_button = $fw_link_6_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$link_param_values[6],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_6_browse_button, -msg => 'Browse for Forbidden Structure Library', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_7_group_frame = $fw_fl_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_7_entry = $fw_link_7_group_frame->LabEntry(-label => '    Toxic Structure Library:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[7], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_7_browse_button = $fw_link_7_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'open_dir',\$link_param_values[7],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_7_browse_button, -msg => 'Browse for Toxic Structure Library', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_8_group_frame = $fw_sbup_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_8_entry = $fw_link_8_group_frame->LabEntry(-label => '            Growing Probability:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[8], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_link_8_phantom_frame = $fw_link_8_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_8_scale = $fw_link_8_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$link_param_values[8],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	my $fw_link_9_group_frame = $fw_sbup_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_9_entry = $fw_link_9_group_frame->LabEntry(-label => '            Linking Probability:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[9], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_link_9_phantom_frame = $fw_link_9_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_9_scale = $fw_link_9_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$link_param_values[9],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	my $fw_link_10_group_frame = $fw_sbup_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_10_entry = $fw_link_10_group_frame->LabEntry(-label => '           Mutation Probability:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[10], -background => 'white', -insertofftime => 0, -justify => 'right', -width => 5, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_link_10_phantom_frame = $fw_link_10_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_10_scale = $fw_link_10_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$link_param_values[10],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	$link_param_values[11] = 1;
	my $fw_link_11_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 8);
	my $fw_link_11_label = $fw_link_11_group_frame->Label(-text => '           Apply Chemical Rules:', -font => ['Courier', '10', 'bold'])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0);
	my $fw_link_11_checkbox = $fw_link_11_group_frame->Checkbutton( -variable => \$link_param_values[11])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
	$link_param_values[12] = 1;
	my $fw_link_12_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 8);
	my $fw_link_12_label = $fw_link_12_group_frame->Label(-text => 'Apply Forbidden Structure Check:', -font => ['Courier', '10', 'bold'])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0);
	my $fw_link_12_checkbox = $fw_link_12_group_frame->Checkbutton( -variable => \$link_param_values[12])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
	$link_param_values[13] = 1;
	my $fw_link_13_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 8);
	my $fw_link_13_label = $fw_link_13_group_frame->Label(-text => '    Apply Toxic Structure Check:', -font => ['Courier', '10', 'bold'])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0);
	my $fw_link_13_checkbox = $fw_link_13_group_frame->Checkbutton( -variable => \$link_param_values[13])->pack(-side => 'left', -padx => 0, -fill => 'none', -expand => 0, -anchor => 'w');
	my $fw_link_14_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_14_entry = $fw_link_14_group_frame->LabEntry(-label => '       Maximal Molecular Weight:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[14], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_14_phantom_frame = $fw_link_14_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_15_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_15_entry = $fw_link_15_group_frame->LabEntry(-label => '       Minimal Molecular Weight:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[15], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_15_phantom_frame = $fw_link_15_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_16_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_16_entry = $fw_link_16_group_frame->LabEntry(-label => '                   Maximal LogP:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[16], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_16_phantom_frame = $fw_link_16_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_17_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_17_entry = $fw_link_17_group_frame->LabEntry(-label => '                   Minimal LogP:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[17], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_17_phantom_frame = $fw_link_17_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_18_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_18_entry = $fw_link_18_group_frame->LabEntry(-label => '         Maximal HB Donor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[18], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_18_phantom_frame = $fw_link_18_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_19_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_19_entry = $fw_link_19_group_frame->LabEntry(-label => '         Minimal HB Donor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[19], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_19_phantom_frame = $fw_link_19_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_20_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_20_entry = $fw_link_20_group_frame->LabEntry(-label => '      Maximal HB Acceptor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[20], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_20_phantom_frame = $fw_link_20_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_21_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_21_entry = $fw_link_21_group_frame->LabEntry(-label => '      Minimal HB Acceptor Atoms:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[21], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_21_phantom_frame = $fw_link_21_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_22_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_22_entry = $fw_link_22_group_frame->LabEntry(-label => '                    Maximal PKD:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[22], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_22_phantom_frame = $fw_link_22_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_23_group_frame = $fw_cvr_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_23_entry = $fw_link_23_group_frame->LabEntry(-label => '                    Minimal PKD:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[23], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_23_phantom_frame = $fw_link_23_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_24_group_frame = $fw_gap_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_24_entry = $fw_link_24_group_frame->LabEntry(-label => '      Number of Generations:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[24], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_24_phantom_frame = $fw_link_24_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_25_group_frame = $fw_gap_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_25_entry = $fw_link_25_group_frame->LabEntry(-label => '       Number of Population:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[25], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_25_phantom_frame = $fw_link_25_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_link_26_group_frame = $fw_of_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_26_entry = $fw_link_26_group_frame->LabEntry(-label => '     Population Record File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[26], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_26_browse_button = $fw_link_26_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_lig',\$link_param_values[26],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_26_browse_button, -msg => 'Save as - LIG File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_link_27_group_frame = $fw_of_link_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_link_27_entry = $fw_link_27_group_frame->LabEntry(-label => '     Ligand Collection File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$link_param_values[27], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_link_27_browse_button = $fw_link_27_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'save_lig',\$link_param_values[27],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_link_27_browse_button, -msg => 'Save as - LIG File', -initwait => 800, -balloonposition => 'mouse');
	# process frame
	my $fw_filename_process_frame = $fw_process_frame->Frame()->pack(-side => 'top', -fill=>'both',-expand=>1, -padx => 10);
	my $fw_filename_process_entry = $fw_filename_process_frame->LabEntry(-label => 'Parameter File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_process_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_filename_process_browse_button = $fw_filename_process_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_browse_file_callback,'process'])->pack( -side => 'right');
	my $fw_if_process_frame = $fw_process_frame->LabFrame(-label => "Input Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_cr_process_frame = $fw_process_frame->LabFrame(-label => "Chemical Rules", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_st_process_frame = $fw_process_frame->LabFrame(-label => "Similarity Threshhold", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_of_process_frame = $fw_process_frame->LabFrame(-label => "Output Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_process_1_group_frame = $fw_if_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_1_entry = $fw_process_1_group_frame->LabEntry(-label => '    Ligand Collection File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[1], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_1_browse_button = $fw_process_1_group_frame->Button(-text => '...', -image => $browse_button_image, -command => [\&fw_button_left_callback,'open_lig',\$process_param_values[1],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_process_1_browse_button, -msg => 'Open - LIG File', -initwait => 800, -balloonposition => 'mouse');
	my $fw_process_2_group_frame = $fw_cr_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_2_entry = $fw_process_2_group_frame->LabEntry(-label => '  Maximal Molecular Weight:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[2], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_2_phantom_frame = $fw_process_2_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_3_group_frame = $fw_cr_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_3_entry = $fw_process_3_group_frame->LabEntry(-label => '  Minimal Molecular Weight:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[3], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_3_phantom_frame = $fw_process_3_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_4_group_frame = $fw_cr_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_4_entry = $fw_process_4_group_frame->LabEntry(-label => '              Maximal LogP:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[4], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_4_phantom_frame = $fw_process_4_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_5_group_frame = $fw_cr_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_5_entry = $fw_process_5_group_frame->LabEntry(-label => '              Minimal LogP:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[5], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_5_phantom_frame = $fw_process_5_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_6_group_frame = $fw_cr_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_6_entry = $fw_process_6_group_frame->LabEntry(-label => '               Maximal PKD:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[6], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_6_phantom_frame = $fw_process_6_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_7_group_frame = $fw_cr_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_7_entry = $fw_process_7_group_frame->LabEntry(-label => '               Minimal PKD:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[7], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_7_phantom_frame = $fw_process_7_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_8_group_frame = $fw_st_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_8_entry = $fw_process_8_group_frame->LabEntry(-label => '         Similarity Cutoff:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[8], -background => 'white', -insertofftime => 0, -justify => 'right', -state => 'readonly', -width => 5)->pack(-side => 'left', -padx => 3, -fill => 'none', -expand => 0);
	my $fw_process_8_phantom_frame = $fw_process_8_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_8_scale = $fw_process_8_group_frame->Scale(-label => '', -showvalue => 0, -orient=>'h',-digits=>0, -resolution => 0.01,-from=>0,-to=>1,-variable=>\$process_param_values[8],-tickinterval=>0, -sliderlength => 30)->pack( -fill => 'x', -expand => 1);
	my $fw_process_9_group_frame = $fw_of_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_9_entry = $fw_process_9_group_frame->LabEntry(-label => 'Number of Output Molecules:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[9], -background => 'white', -insertofftime => 0, -justify => 'right')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_9_phantom_frame = $fw_process_9_group_frame->Frame()->pack( -side => 'right', -anchor => 'w', -fill => 'none', -expand => 0, -padx => 12, -pady => 0);
	my $fw_process_10_group_frame = $fw_of_process_frame->Frame()->pack( -anchor => 'n', -expand => 1, -fill => 'both', -padx => 5);
	my $fw_process_10_entry = $fw_process_10_group_frame->LabEntry(-label => '          Output Directory:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$process_param_values[10], -background => 'white', -insertofftime => 0)->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_process_10_browse_button = $fw_process_10_group_frame->Button(-text => '...',-image => $folder_button_image, -command => [\&fw_button_left_callback,'save_dir',\$process_param_values[10],$fw])->pack( -side => 'right');
        $main_balloon->attach($fw_process_10_browse_button, -msg => 'Browse for Output Directory', -initwait => 800, -balloonposition => 'mouse');
	
	my $fw_new_button = $fw_buttons_frame->Button(-text => 'New')->pack( -side => 'left', -padx => 1);
	my $fw_save_button = $fw_buttons_frame->Button(-text => 'Save',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_save_as_button = $fw_buttons_frame->Button(-text => 'Save As',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_delete_button = $fw_buttons_frame->Button(-text => 'Delete',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_close_button = $fw_buttons_frame->Button(-text => 'Close', -command => [\&fw_button_callback,'close',$fw])->pack( -side => 'left', -padx => 1);
	
	$fw_filename_pocket_browse_button->configure(-command => [\&fw_browse_file_callback,'pocket',$fw,\$edit_pocket_file,$fw_save_button,$fw_save_as_button,$fw_delete_button,\@pocket_param_values]);
	$fw_filename_grow_browse_button->configure(-command => [\&fw_browse_file_callback,'grow',$fw,\$edit_grow_file,$fw_save_button,$fw_save_as_button,$fw_delete_button,\@grow_param_values]);
	$fw_filename_link_browse_button->configure(-command => [\&fw_browse_file_callback,'link',$fw,\$edit_link_file,$fw_save_button,$fw_save_as_button,$fw_delete_button,\@link_param_values]);
	$fw_filename_process_browse_button->configure(-command => [\&fw_browse_file_callback,'process',$fw,\$edit_process_file,$fw_save_button,$fw_save_as_button,$fw_delete_button,\@process_param_values]);
	$fw_new_button->configure(-command => [\&fw_button_callback,'new',$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,\$edit_pocket_file,\$edit_grow_file,\$edit_link_file,\$edit_process_file,\@pocket_param_values,\@grow_param_values,\@link_param_values,\@process_param_values]);
	$fw_save_button->configure(-command => [\&fw_button_callback,'save',$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,\$edit_pocket_file,\$edit_grow_file,\$edit_link_file,\$edit_process_file,\@pocket_param_values,\@grow_param_values,\@link_param_values,\@process_param_values]);
	$fw_save_as_button->configure(-command => [\&fw_button_callback,'save_as',$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,\$edit_pocket_file,\$edit_grow_file,\$edit_link_file,\$edit_process_file,\@pocket_param_values,\@grow_param_values,\@link_param_values,\@process_param_values]);
	$fw_delete_button->configure(-command => [\&fw_button_callback,'delete',$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,\$edit_pocket_file,\$edit_grow_file,\$edit_link_file,\$edit_process_file,\@pocket_param_values,\@grow_param_values,\@link_param_values,\@process_param_values]);
	$fw_tabbed_frame->pageconfigure('Pocket', -raisecmd => [\&fw_raised_tab_callback,'pocket',$fw_tabbed_frame,\$edit_pocket_file,$fw_save_button,$fw_save_as_button,$fw_delete_button]);
	$fw_tabbed_frame->pageconfigure('Grow',-raisecmd => [\&fw_raised_tab_callback,'grow',$fw_tabbed_frame,\$edit_grow_file,$fw_save_button,$fw_save_as_button,$fw_delete_button]);
	$fw_tabbed_frame->pageconfigure('Link',-raisecmd => [\&fw_raised_tab_callback,'link',$fw_tabbed_frame,\$edit_link_file,$fw_save_button,$fw_save_as_button,$fw_delete_button]);
	$fw_tabbed_frame->pageconfigure('Process',-raisecmd => [\&fw_raised_tab_callback,'process',$fw_tabbed_frame,\$edit_process_file,$fw_save_button,$fw_save_as_button,$fw_delete_button]);

	if (defined $default_tab) {
		if ($selected_pocket_par_file && -e $selected_pocket_par_file) {			
			$edit_pocket_file = $selected_pocket_par_file;
			&populate_pocket(\$edit_pocket_file,\@pocket_param_values);
		}
		if ($selected_grow_par_file && -e $selected_grow_par_file) {
			$edit_grow_file = $selected_grow_par_file;
			&populate_grow(\$edit_grow_file,\@grow_param_values);
		}
		if ($selected_link_par_file && -e $selected_link_par_file) {
			$edit_link_file = $selected_link_par_file;
			&populate_link(\$edit_link_file,\@link_param_values);
		}
		if ($selected_process_par_file && -e $selected_process_par_file) {
			$edit_process_file = $selected_process_par_file;
			&populate_process(\$edit_process_file,\@process_param_values);
		}
		if ($default_tab eq 'pocket') {
			$fw_tabbed_frame->raise('Pocket');
		} elsif ($default_tab eq 'grow') {
			$fw_tabbed_frame->raise('Grow');
		} elsif ($default_tab eq 'link') {
			$fw_tabbed_frame->raise('Link');
		} elsif ($default_tab eq 'process') {
			$fw_tabbed_frame->raise('Process');
		} else {}	
	}
	$fw->deiconify;
}

sub fw_browse_file_callback {
	my ($opt,$fw,$edit_file_ref,$fw_save_button,$fw_save_as_button,$fw_delete_button,$param_values_ref) = @_;

	my $types=[];
	push @$types, ['Parameter files', '*.index'];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'pocket') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_pocket(\$file,$param_values_ref);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}		
	} elsif ($opt eq 'grow') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_grow(\$file,$param_values_ref);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} elsif ($opt eq 'link') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_link(\$file,$param_values_ref);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} elsif ($opt eq 'process') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_process(\$file,$param_values_ref);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} else {}
}

sub populate_pocket {
	my ($file_ref,$param_values_ref) = @_;
	
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	foreach my $line (<IN>) {
		next if $line =~ /^#/;
		next if $line =~ /^\s*$/;
		chomp($line);
		$line =~ s/\s*$//;
		(my $par_name,my $par_value) = $line =~ /(\S*)\s*(.*)/;
		if ($par_name eq 'RECEPTOR_FILE') {
			$$param_values_ref[1] = $par_value;
		} elsif ($par_name eq 'LIGAND_FILE') {
			$$param_values_ref[2] = $par_value;
		} elsif ($par_name eq 'PARAMETER_DIRECTORY') {
			$$param_values_ref[3] = $par_value;
		} elsif ($par_name eq 'POCKET_ATOM_FILE') {
			$$param_values_ref[4] = $par_value;
		} elsif ($par_name eq 'POCKET_GRID_FILE') {
			$$param_values_ref[5] = $par_value;
		} elsif ($par_name eq 'KEY_SITE_FILE') {
			$$param_values_ref[6] = $par_value;
		} elsif ($par_name eq 'PHARMACOPHORE_TXT_FILE') {
			$$param_values_ref[7] = $par_value;
		} elsif ($par_name eq 'PHARMACOPHORE_PDB_FILE') {
			$$param_values_ref[8] = $par_value;
		} elsif ($par_name eq 'MINIMAL_FEATURE_DISTANCE') {
			$$param_values_ref[9] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_FEATURE_NUMBER') {
			$$param_values_ref[10] = $par_value;
		} else {}	
	}
	close IN;
}

sub populate_grow {
	my ($file_ref,$param_values_ref) = @_;
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	foreach my $line (<IN>) {
		next if $line =~ /^#/;
		next if $line =~ /^\s*$/;
		chomp($line);
		$line =~ s/\s*$//;
		(my $par_name,my $par_value) = $line =~ /(\S*)\s*(.*)/;
		$par_value = 1 if ($par_value eq 'YES');
		$par_value = 0 if ($par_value eq 'NO');
		if ($par_name eq 'SEED_LIGAND_FILE') {
			$$param_values_ref[1] = $par_value;
		} elsif ($par_name eq 'POCKET_ATOM_FILE') {
			$$param_values_ref[2] = $par_value;
		} elsif ($par_name eq 'POCKET_GRID_FILE') {
			$$param_values_ref[3] = $par_value;
		} elsif ($par_name eq 'PARAMETER_DIRECTORY') {
			$$param_values_ref[4] = $par_value;
		} elsif ($par_name eq 'BUILDING_BLOCK_LIBRARY') {
			$$param_values_ref[5] = $par_value;
		} elsif ($par_name eq 'FORBIDDEN_STRUCTURE_LIBRARY') {
			$$param_values_ref[6] = $par_value;
		} elsif ($par_name eq 'TOXIC_STRUCTURE_LIBRARY') {
			$$param_values_ref[7] = $par_value;
		} elsif ($par_name eq 'GROWING_PROBABILITY') {
			$$param_values_ref[8] = $par_value;
		} elsif ($par_name eq 'LINKING_PROBABILITY') {
			$$param_values_ref[9] = $par_value;
		} elsif ($par_name eq 'MUTATION_PROBABILITY') {
			$$param_values_ref[10] = $par_value;
		} elsif ($par_name eq 'APPLY_CHEMICAL_RULES') {
			$$param_values_ref[11] = $par_value;
		} elsif ($par_name eq 'APPLY_FORBIDDEN_STRUCTURE_CHECK') {
			$$param_values_ref[12] = $par_value;
		} elsif ($par_name eq 'APPLY_TOXIC_STRUCTURE_CHECK') {
			$$param_values_ref[13] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_MOLECULAR_WEIGHT') {
			$$param_values_ref[14] = $par_value;
		} elsif ($par_name eq 'MINIMAL_MOLECULAR_WEIGHT') {
			$$param_values_ref[15] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_LOGP') {
			$$param_values_ref[16] = $par_value;
		} elsif ($par_name eq 'MINIMAL_LOGP') {
			$$param_values_ref[17] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_HB_DONOR_ATOM') {
			$$param_values_ref[18] = $par_value;
		} elsif ($par_name eq 'MINIMAL_HB_DONOR_ATOM') {
			$$param_values_ref[19] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_HB_ACCEPTOR_ATOM') {
			$$param_values_ref[20] = $par_value;
		} elsif ($par_name eq 'MINIMAL_HB_ACCEPTOR_ATOM') {
			$$param_values_ref[21] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_PKD') {
			$$param_values_ref[22] = $par_value;
		} elsif ($par_name eq 'MINIMAL_PKD') {
			$$param_values_ref[23] = $par_value;
		} elsif ($par_name eq 'NUMBER_OF_GENERATION') {
			$$param_values_ref[24] = $par_value;
		} elsif ($par_name eq 'NUMBER_OF_POPULATION') {
			$$param_values_ref[25] = $par_value;
		} elsif ($par_name eq 'NUMBER_OF_PARENTS') {
			$$param_values_ref[26] = $par_value;
		} elsif ($par_name eq 'ELITISM_RATIO') {
			$$param_values_ref[27] = $par_value;
		} elsif ($par_name eq 'SIMILARITY_CUTOFF') {
			$$param_values_ref[28] = $par_value;
		} elsif ($par_name eq 'POPULATION_RECORD_FILE') {
			$$param_values_ref[29] = $par_value;
		} elsif ($par_name eq 'LIGAND_COLLECTION_FILE') {
			$$param_values_ref[30] = $par_value;
		} else {}	
	}
	close IN;
}

sub populate_link {
	my ($file_ref,$param_values_ref) = @_;
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	foreach my $line (<IN>) {
		next if $line =~ /^#/;
		next if $line =~ /^\s*$/;
		chomp($line);
		$line =~ s/\s*$//;
		(my $par_name,my $par_value) = $line =~ /(\S*)\s*(.*)/;
		$par_value = 1 if ($par_value eq 'YES');
		$par_value = 0 if ($par_value eq 'NO');
		if ($par_name eq 'SEED_LIGAND_FILE') {
			$$param_values_ref[1] = $par_value;
		} elsif ($par_name eq 'POCKET_ATOM_FILE') {
			$$param_values_ref[2] = $par_value;
		} elsif ($par_name eq 'POCKET_GRID_FILE') {
			$$param_values_ref[3] = $par_value;
		} elsif ($par_name eq 'PARAMETER_DIRECTORY') {
			$$param_values_ref[4] = $par_value;
		} elsif ($par_name eq 'BUILDING_BLOCK_LIBRARY') {
			$$param_values_ref[5] = $par_value;
		} elsif ($par_name eq 'FORBIDDEN_STRUCTURE_LIBRARY') {
			$$param_values_ref[6] = $par_value;
		} elsif ($par_name eq 'TOXIC_STRUCTURE_LIBRARY') {
			$$param_values_ref[7] = $par_value;
		} elsif ($par_name eq 'GROWING_PROBABILITY') {
			$$param_values_ref[8] = $par_value;
		} elsif ($par_name eq 'LINKING_PROBABILITY') {
			$$param_values_ref[9] = $par_value;
		} elsif ($par_name eq 'MUTATION_PROBABILITY') {
			$$param_values_ref[10] = $par_value;
		} elsif ($par_name eq 'APPLY_CHEMICAL_RULES') {
			$$param_values_ref[11] = $par_value;
		} elsif ($par_name eq 'APPLY_FORBIDDEN_STRUCTURE_CHECK') {
			$$param_values_ref[12] = $par_value;
		} elsif ($par_name eq 'APPLY_TOXIC_STRUCTURE_CHECK') {
			$$param_values_ref[13] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_MOLECULAR_WEIGHT') {
			$$param_values_ref[14] = $par_value;
		} elsif ($par_name eq 'MINIMAL_MOLECULAR_WEIGHT') {
			$$param_values_ref[15] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_LOGP') {
			$$param_values_ref[16] = $par_value;
		} elsif ($par_name eq 'MINIMAL_LOGP') {
			$$param_values_ref[17] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_HB_DONOR_ATOM') {
			$$param_values_ref[18] = $par_value;
		} elsif ($par_name eq 'MINIMAL_HB_DONOR_ATOM') {
			$$param_values_ref[19] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_HB_ACCEPTOR_ATOM') {
			$$param_values_ref[20] = $par_value;
		} elsif ($par_name eq 'MINIMAL_HB_ACCEPTOR_ATOM') {
			$$param_values_ref[21] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_PKD') {
			$$param_values_ref[22] = $par_value;
		} elsif ($par_name eq 'MINIMAL_PKD') {
			$$param_values_ref[23] = $par_value;
		} elsif ($par_name eq 'NUMBER_OF_GENERATION') {
			$$param_values_ref[24] = $par_value;
		} elsif ($par_name eq 'NUMBER_OF_POPULATION') {
			$$param_values_ref[25] = $par_value;
		} elsif ($par_name eq 'POPULATION_RECORD_FILE') {
			$$param_values_ref[26] = $par_value;
		} elsif ($par_name eq 'LIGAND_COLLECTION_FILE') {
			$$param_values_ref[27] = $par_value;
		} else {}	
	}
	close IN;
}

sub populate_process {
	my ($file_ref,$param_values_ref) = @_;
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	foreach my $line (<IN>) {
		next if $line =~ /^#/;
		next if $line =~ /^\s*$/;
		chomp($line);
		$line =~ s/\s*$//;
		(my $par_name,my $par_value) = $line =~ /(\S*)\s*(.*)/;
		if ($par_name eq 'LIGAND_COLLECTION_FILE') {
			$$param_values_ref[1] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_MOLECULAR_WEIGHT') {
			$$param_values_ref[2] = $par_value;
		} elsif ($par_name eq 'MINIMAL_MOLECULAR_WEIGHT') {
			$$param_values_ref[3] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_LOGP') {
			$$param_values_ref[4] = $par_value;
		} elsif ($par_name eq 'MINIMAL_LOGP') {
			$$param_values_ref[5] = $par_value;
		} elsif ($par_name eq 'MAXIMAL_PKD') {
			$$param_values_ref[6] = $par_value;
		} elsif ($par_name eq 'MINIMAL_PKD') {
			$$param_values_ref[7] = $par_value;
		} elsif ($par_name eq 'SIMILARITY_CUTOFF') {
			$$param_values_ref[8] = $par_value;
		} elsif ($par_name eq 'NUMBER_OF_OUTPUT_MOLECULES') {
			$$param_values_ref[9] = $par_value;
		} elsif ($par_name eq 'OUTPUT_DIRECTORY') {
			$$param_values_ref[10] = $par_value;
		} else {}	
	}
	close IN;
}

sub fw_button_callback {
	my ($opt,$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,$edit_pocket_file_ref,$edit_grow_file_ref,$edit_link_file_ref,$edit_process_file_ref,$pocket_param_values_ref,$grow_param_values_ref,$link_param_values_ref,$process_param_values_ref) = @_;
	
	if ($opt eq 'new') {
		my $types=[];
		push @$types, ['Parameter files', '*.index'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Create New Parameter File', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
			my $temp_var = $fw_tabbed_frame->raised();
			if ($temp_var eq 'Pocket') {
				$$edit_pocket_file_ref = $file;
				for (my $i=1;$i<=10;$i++) {
					$$pocket_param_values_ref[$i] = '';
				}
			} elsif ($temp_var eq 'Grow') {
				$$edit_grow_file_ref = $file;
				for (my $i=1;$i<=7;$i++) {
					$$grow_param_values_ref[$i] = '';
				}
				$$grow_param_values_ref[8] = '0.00';
				$$grow_param_values_ref[9] = '0.00';
				$$grow_param_values_ref[10] = '0.00';
				for (my $i=11;$i<=26;$i++) {
					$$grow_param_values_ref[$i] = '';
				}
				$$grow_param_values_ref[27] = '0.00';
				$$grow_param_values_ref[28] = '0.00';
				for (my $i=29;$i<=30;$i++) {
					$$grow_param_values_ref[$i] = '';
				}
			} elsif ($temp_var eq 'Link') {
				$$edit_link_file_ref = $file;
				for (my $i=1;$i<=7;$i++) {
					$$link_param_values_ref[$i] = '';
				}
				$$link_param_values_ref[8] = '0.00';
				$$link_param_values_ref[9] = '0.00';
				$$link_param_values_ref[10] = '0.00';
				for (my $i=11;$i<=27;$i++) {
					$$link_param_values_ref[$i] = '';
				}
			} elsif ($temp_var eq 'Process') {
				$$edit_process_file_ref = $file;
				for (my $i=1;$i<=7;$i++) {
					$$process_param_values_ref[$i] = '';
				}
				$$process_param_values_ref[8] = '0.00';
				for (my $i=9;$i<=10;$i++) {
					$$process_param_values_ref[$i] = '';
				}
			} else {print "what?"}
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} elsif ($opt eq 'save') {
		my $temp_var = $fw_tabbed_frame->raised();
		if ($temp_var eq 'Pocket') {
			open OUT, '>', "$$edit_pocket_file_ref" or die "Error trying to open $$edit_pocket_file_ref. $!";
			print OUT "#\n# Input Files\n#\n";
			print OUT "RECEPTOR_FILE\t$$pocket_param_values_ref[1]\n";
			print OUT "LIGAND_FILE\t$$pocket_param_values_ref[2]\n";
			print OUT "PARAMETER_DIRECTORY\t$$pocket_param_values_ref[3]\n";
			print OUT "#\n# Output Files\n#\n";
			print OUT "POCKET_ATOM_FILE\t$$pocket_param_values_ref[4]\n";
			print OUT "POCKET_GRID_FILE\t$$pocket_param_values_ref[5]\n";
			print OUT "#\n# Key Interaction Sites and Pharmacophore\n#\n";
			print OUT "KEY_SITE_FILE\t$$pocket_param_values_ref[6]\n";
			print OUT "PHARMACOPHORE_TXT_FILE\t$$pocket_param_values_ref[7]\n";
			print OUT "PHARMACOPHORE_PDB_FILE\t$$pocket_param_values_ref[8]\n";
			print OUT "MINIMAL_FEATURE_DISTANCE\t$$pocket_param_values_ref[9]\n";
			print OUT "MAXIMAL_FEATURE_NUMBER\t$$pocket_param_values_ref[10]\n";
			print OUT "#\n#\n#\n";
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_pocket_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		} elsif ($temp_var eq 'Grow') {
			open OUT, '>', "$$edit_grow_file_ref" or die "Error trying to open $$edit_grow_file_ref. $!";
			print OUT "#\n# Input Files\n#\n";
			print OUT "SEED_LIGAND_FILE\t$$grow_param_values_ref[1]\n";
			print OUT "POCKET_ATOM_FILE\t$$grow_param_values_ref[2]\n";
			print OUT "POCKET_GRID_FILE\t$$grow_param_values_ref[3]\n";
			print OUT "#\n# Force Field Directory\n#\n";
			print OUT "PARAMETER_DIRECTORY\t$$grow_param_values_ref[4]\n";
			print OUT "#\n# Fragment Libraries\n#\n";
			print OUT "BUILDING_BLOCK_LIBRARY\t$$grow_param_values_ref[5]\n";
			print OUT "FORBIDDEN_STRUCTURE_LIBRARY\t$$grow_param_values_ref[6]\n";
			print OUT "TOXIC_STRUCTURE_LIBRARY\t$$grow_param_values_ref[7]\n";
			print OUT "#\n# Structural Construction Parameters\n#\n";
			print OUT "GROWING_PROBABILITY\t$$grow_param_values_ref[8]\n";
			print OUT "LINKING_PROBABILITY\t$$grow_param_values_ref[9]\n";
			print OUT "MUTATION_PROBABILITY\t$$grow_param_values_ref[10]\n";
			print OUT "#\n# Chemical Viability Rules\n#\n";
			if ($$grow_param_values_ref[11]) {
				print OUT "APPLY_CHEMICAL_RULES\tYES\n";
			} else {
				print OUT "APPLY_CHEMICAL_RULES\tNO\n";
			}
			if ($$grow_param_values_ref[12]) {
				print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tYES\n";
			} else {
				print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tNO\n";
			}
			if ($$grow_param_values_ref[13]) {
				print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tYES\n";
			} else {
				print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tNO\n";
			}
			print OUT "MAXIMAL_MOLECULAR_WEIGHT\t$$grow_param_values_ref[14]\n";
			print OUT "MINIMAL_MOLECULAR_WEIGHT\t$$grow_param_values_ref[15]\n";
			print OUT "MAXIMAL_LOGP\t$$grow_param_values_ref[16]\n";
			print OUT "MINIMAL_LOGP\t$$grow_param_values_ref[17]\n";
			print OUT "MAXIMAL_HB_DONOR_ATOM\t$$grow_param_values_ref[18]\n";
			print OUT "MINIMAL_HB_DONOR_ATOM\t$$grow_param_values_ref[19]\n";
			print OUT "MAXIMAL_HB_ACCEPTOR_ATOM\t$$grow_param_values_ref[20]\n";
			print OUT "MINIMAL_HB_ACCEPTOR_ATOM\t$$grow_param_values_ref[21]\n";
			print OUT "MAXIMAL_PKD\t$$grow_param_values_ref[22]\n";
			print OUT "MINIMAL_PKD\t$$grow_param_values_ref[23]\n";
			print OUT "#\n# Genetic Algorithm Parameters\n#\n";
			print OUT "NUMBER_OF_GENERATION\t$$grow_param_values_ref[24]\n";
			print OUT "NUMBER_OF_POPULATION\t$$grow_param_values_ref[25]\n";
			print OUT "NUMBER_OF_PARENTS\t$$grow_param_values_ref[26]\n";
			print OUT "ELITISM_RATIO\t$$grow_param_values_ref[27]\n";
			print OUT "SIMILARITY_CUTOFF\t$$grow_param_values_ref[28]\n";
			print OUT "#\n# Output Files\n#\n";
			print OUT "POPULATION_RECORD_FILE\t$$grow_param_values_ref[29]\n";
			print OUT "LIGAND_COLLECTION_FILE\t$$grow_param_values_ref[30]\n";
			print OUT "#\n#\n#\n";
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_grow_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		} elsif ($temp_var eq 'Link') {
			open OUT, '>', "$$edit_link_file_ref" or die "Error trying to open $$edit_link_file_ref. $!";
			print OUT "#\n# Input Files\n#\n";
			print OUT "SEED_LIGAND_FILE\t$$link_param_values_ref[1]\n";
			print OUT "POCKET_ATOM_FILE\t$$link_param_values_ref[2]\n";
			print OUT "POCKET_GRID_FILE\t$$link_param_values_ref[3]\n";
			print OUT "#\n# Force Field Directory\n#\n";
			print OUT "PARAMETER_DIRECTORY\t$$link_param_values_ref[4]\n";
			print OUT "#\n# Fragment Libraries\n#\n";
			print OUT "BUILDING_BLOCK_LIBRARY\t$$link_param_values_ref[5]\n";
			print OUT "FORBIDDEN_STRUCTURE_LIBRARY\t$$link_param_values_ref[6]\n";
			print OUT "TOXIC_STRUCTURE_LIBRARY\t$$link_param_values_ref[7]\n";
			print OUT "#\n# Structural Building Up Parameters\n#\n";
			print OUT "GROWING_PROBABILITY\t$$link_param_values_ref[8]\n";
			print OUT "LINKING_PROBABILITY\t$$link_param_values_ref[9]\n";
			print OUT "MUTATION_PROBABILITY\t$$link_param_values_ref[10]\n";
			print OUT "#\n# Chemical Viability Rules\n#\n";
			if ($$link_param_values_ref[11]) {
				print OUT "APPLY_CHEMICAL_RULES\tYES\n";
			} else {
				print OUT "APPLY_CHEMICAL_RULES\tNO\n";
			}
			if ($$link_param_values_ref[12]) {
				print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tYES\n";
			} else {
				print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tNO\n";
			}
			if ($$link_param_values_ref[13]) {
				print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tYES\n";
			} else {
				print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tNO\n";
			}
			print OUT "MAXIMAL_MOLECULAR_WEIGHT\t$$link_param_values_ref[14]\n";
			print OUT "MINIMAL_MOLECULAR_WEIGHT\t$$link_param_values_ref[15]\n";
			print OUT "MAXIMAL_LOGP\t$$link_param_values_ref[16]\n";
			print OUT "MINIMAL_LOGP\t$$link_param_values_ref[17]\n";
			print OUT "MAXIMAL_HB_DONOR_ATOM\t$$link_param_values_ref[18]\n";
			print OUT "MINIMAL_HB_DONOR_ATOM\t$$link_param_values_ref[19]\n";
			print OUT "MAXIMAL_HB_ACCEPTOR_ATOM\t$$link_param_values_ref[20]\n";
			print OUT "MINIMAL_HB_ACCEPTOR_ATOM\t$$link_param_values_ref[21]\n";
			print OUT "MAXIMAL_PKD\t$$link_param_values_ref[22]\n";
			print OUT "MINIMAL_PKD\t$$link_param_values_ref[23]\n";
			print OUT "#\n# Genetic Algorithm Parameters\n#\n";
			print OUT "NUMBER_OF_GENERATION\t$$link_param_values_ref[24]\n";
			print OUT "NUMBER_OF_POPULATION\t$$link_param_values_ref[25]\n";
			print OUT "#\n# Output Files\n#\n";
			print OUT "POPULATION_RECORD_FILE\t$$link_param_values_ref[26]\n";
			print OUT "LIGAND_COLLECTION_FILE\t$$link_param_values_ref[27]\n";
			print OUT "#\n#\n#\n";
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_link_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		} elsif ($temp_var eq 'Process') {
			open OUT, '>', "$$edit_process_file_ref" or die "Error trying to open $$edit_process_file_ref. $!";
			print OUT "#\n# Input Files\n#\n";
			print OUT "LIGAND_COLLECTION_FILE\t$$process_param_values_ref[1]\n";
			print OUT "#\n# Chemical Rules\n#\n";
			print OUT "MAXIMAL_MOLECULAR_WEIGHT\t$$process_param_values_ref[2]\n";
			print OUT "MINIMAL_MOLECULAR_WEIGHT\t$$process_param_values_ref[3]\n";
			print OUT "MAXIMAL_LOGP\t$$process_param_values_ref[4]\n";
			print OUT "MINIMAL_LOGP\t$$process_param_values_ref[5]\n";
			print OUT "MAXIMAL_PKD\t$$process_param_values_ref[6]\n";
			print OUT "MINIMAL_PKD\t$$process_param_values_ref[7]\n";
			print OUT "#\n# Similarity Threshold\n#\n";
			print OUT "SIMILARITY_CUTOFF\t$$process_param_values_ref[8]\n";
			print OUT "#\n# Output Files\n#\n";
			print OUT "NUMBER_OF_OUTPUT_MOLECULES\t$$process_param_values_ref[9]\n";
			print OUT "OUTPUT_DIRECTORY\t$$process_param_values_ref[10]\n";
			print OUT "#\n#\n#\n";
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_process_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		} else {print "what?"}
	} elsif ($opt eq 'save_as') {
		my $types=[];
		push @$types, ['Parameter files', '*.index'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Save Parameter File As', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
			my $temp_var = $fw_tabbed_frame->raised();
			if ($temp_var eq 'Pocket') {
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT "#\n# Input Files\n#\n";
				print OUT "RECEPTOR_FILE\t$$pocket_param_values_ref[1]\n";
				print OUT "LIGAND_FILE\t$$pocket_param_values_ref[2]\n";
				print OUT "PARAMETER_DIRECTORY\t$$pocket_param_values_ref[3]\n";
				print OUT "#\n# Output Files\n#\n";
				print OUT "POCKET_ATOM_FILE\t$$pocket_param_values_ref[4]\n";
				print OUT "POCKET_GRID_FILE\t$$pocket_param_values_ref[5]\n";
				print OUT "#\n# Key Interaction Sites and Pharmacophore\n#\n";
				print OUT "KEY_SITE_FILE\t$$pocket_param_values_ref[6]\n";
				print OUT "PHARMACOPHORE_TXT_FILE\t$$pocket_param_values_ref[7]\n";
				print OUT "PHARMACOPHORE_PDB_FILE\t$$pocket_param_values_ref[8]\n";
				print OUT "MINIMAL_FEATURE_DISTANCE\t$$pocket_param_values_ref[9]\n";
				print OUT "MAXIMAL_FEATURE_NUMBER\t$$pocket_param_values_ref[10]\n";
				print OUT "#\n#\n#\n";
				close OUT;
				$$edit_pocket_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
			} elsif ($temp_var eq 'Grow') {
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT "#\n# Input Files\n#\n";
				print OUT "SEED_LIGAND_FILE\t$$grow_param_values_ref[1]\n";
				print OUT "POCKET_ATOM_FILE\t$$grow_param_values_ref[2]\n";
				print OUT "POCKET_GRID_FILE\t$$grow_param_values_ref[3]\n";
				print OUT "#\n# Force Field Directory\n#\n";
				print OUT "PARAMETER_DIRECTORY\t$$grow_param_values_ref[4]\n";
				print OUT "#\n# Fragment Libraries\n#\n";
				print OUT "BUILDING_BLOCK_LIBRARY\t$$grow_param_values_ref[5]\n";
				print OUT "FORBIDDEN_STRUCTURE_LIBRARY\t$$grow_param_values_ref[6]\n";
				print OUT "TOXIC_STRUCTURE_LIBRARY\t$$grow_param_values_ref[7]\n";
				print OUT "#\n# Structural Construction Parameters\n#\n";
				print OUT "GROWING_PROBABILITY\t$$grow_param_values_ref[8]\n";
				print OUT "LINKING_PROBABILITY\t$$grow_param_values_ref[9]\n";
				print OUT "MUTATION_PROBABILITY\t$$grow_param_values_ref[10]\n";
				print OUT "#\n# Chemical Viability Rules\n#\n";
				if ($$grow_param_values_ref[11]) {
					print OUT "APPLY_CHEMICAL_RULES\tYES\n";
				} else {
					print OUT "APPLY_CHEMICAL_RULES\tNO\n";
				}
				if ($$grow_param_values_ref[12]) {
					print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tYES\n";
				} else {
					print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tNO\n";
				}
				if ($$grow_param_values_ref[13]) {
					print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tYES\n";
				} else {
					print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tNO\n";
				}
				print OUT "MAXIMAL_MOLECULAR_WEIGHT\t$$grow_param_values_ref[14]\n";
				print OUT "MINIMAL_MOLECULAR_WEIGHT\t$$grow_param_values_ref[15]\n";
				print OUT "MAXIMAL_LOGP\t$$grow_param_values_ref[16]\n";
				print OUT "MINIMAL_LOGP\t$$grow_param_values_ref[17]\n";
				print OUT "MAXIMAL_HB_DONOR_ATOM\t$$grow_param_values_ref[18]\n";
				print OUT "MINIMAL_HB_DONOR_ATOM\t$$grow_param_values_ref[19]\n";
				print OUT "MAXIMAL_HB_ACCEPTOR_ATOM\t$$grow_param_values_ref[20]\n";
				print OUT "MINIMAL_HB_ACCEPTOR_ATOM\t$$grow_param_values_ref[21]\n";
				print OUT "MAXIMAL_PKD\t$$grow_param_values_ref[22]\n";
				print OUT "MINIMAL_PKD\t$$grow_param_values_ref[23]\n";
				print OUT "#\n# Genetic Algorithm Parameters\n#\n";
				print OUT "NUMBER_OF_GENERATION\t$$grow_param_values_ref[24]\n";
				print OUT "NUMBER_OF_POPULATION\t$$grow_param_values_ref[25]\n";
				print OUT "NUMBER_OF_PARENTS\t$$grow_param_values_ref[26]\n";
				print OUT "ELITISM_RATIO\t$$grow_param_values_ref[27]\n";
				print OUT "SIMILARITY_CUTOFF\t$$grow_param_values_ref[28]\n";
				print OUT "#\n# Output Files\n#\n";
				print OUT "POPULATION_RECORD_FILE\t$$grow_param_values_ref[29]\n";
				print OUT "LIGAND_COLLECTION_FILE\t$$grow_param_values_ref[30]\n";
				print OUT "#\n#\n#\n";
				close OUT;
				$$edit_grow_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
			} elsif ($temp_var eq 'Link') {
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT "#\n# Input Files\n#\n";
				print OUT "SEED_LIGAND_FILE\t$$link_param_values_ref[1]\n";
				print OUT "POCKET_ATOM_FILE\t$$link_param_values_ref[2]\n";
				print OUT "POCKET_GRID_FILE\t$$link_param_values_ref[3]\n";
				print OUT "#\n# Force Field Directory\n#\n";
				print OUT "PARAMETER_DIRECTORY\t$$link_param_values_ref[4]\n";
				print OUT "#\n# Fragment Libraries\n#\n";
				print OUT "BUILDING_BLOCK_LIBRARY\t$$link_param_values_ref[5]\n";
				print OUT "FORBIDDEN_STRUCTURE_LIBRARY\t$$link_param_values_ref[6]\n";
				print OUT "TOXIC_STRUCTURE_LIBRARY\t$$link_param_values_ref[7]\n";
				print OUT "#\n# Structural Building Up Parameters\n#\n";
				print OUT "GROWING_PROBABILITY\t$$link_param_values_ref[8]\n";
				print OUT "LINKING_PROBABILITY\t$$link_param_values_ref[9]\n";
				print OUT "MUTATION_PROBABILITY\t$$link_param_values_ref[10]\n";
				print OUT "#\n# Chemical Viability Rules\n#\n";
				if ($$link_param_values_ref[11]) {
					print OUT "APPLY_CHEMICAL_RULES\tYES\n";
				} else {
					print OUT "APPLY_CHEMICAL_RULES\tNO\n";
				}
				if ($$link_param_values_ref[12]) {
					print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tYES\n";
				} else {
					print OUT "APPLY_FORBIDDEN_STRUCTURE_CHECK\tNO\n";
				}
				if ($$link_param_values_ref[13]) {
					print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tYES\n";
				} else {
					print OUT "APPLY_TOXIC_STRUCTURE_CHECK\tNO\n";
				}
				print OUT "MAXIMAL_MOLECULAR_WEIGHT\t$$link_param_values_ref[14]\n";
				print OUT "MINIMAL_MOLECULAR_WEIGHT\t$$link_param_values_ref[15]\n";
				print OUT "MAXIMAL_LOGP\t$$link_param_values_ref[16]\n";
				print OUT "MINIMAL_LOGP\t$$link_param_values_ref[17]\n";
				print OUT "MAXIMAL_HB_DONOR_ATOM\t$$link_param_values_ref[18]\n";
				print OUT "MINIMAL_HB_DONOR_ATOM\t$$link_param_values_ref[19]\n";
				print OUT "MAXIMAL_HB_ACCEPTOR_ATOM\t$$link_param_values_ref[20]\n";
				print OUT "MINIMAL_HB_ACCEPTOR_ATOM\t$$link_param_values_ref[21]\n";
				print OUT "MAXIMAL_PKD\t$$link_param_values_ref[22]\n";
				print OUT "MINIMAL_PKD\t$$link_param_values_ref[23]\n";
				print OUT "#\n# Genetic Algorithm Parameters\n#\n";
				print OUT "NUMBER_OF_GENERATION\t$$link_param_values_ref[24]\n";
				print OUT "NUMBER_OF_POPULATION\t$$link_param_values_ref[25]\n";
				print OUT "#\n# Output Files\n#\n";
				print OUT "POPULATION_RECORD_FILE\t$$link_param_values_ref[26]\n";
				print OUT "LIGAND_COLLECTION_FILE\t$$link_param_values_ref[27]\n";
				print OUT "#\n#\n#\n";
				close OUT;
				$$edit_link_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
			} elsif ($temp_var eq 'Process') {
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT "#\n# Input Files\n#\n";
				print OUT "LIGAND_COLLECTION_FILE\t$$process_param_values_ref[1]\n";
				print OUT "#\n# Chemical Rules\n#\n";
				print OUT "MAXIMAL_MOLECULAR_WEIGHT\t$$process_param_values_ref[2]\n";
				print OUT "MINIMAL_MOLECULAR_WEIGHT\t$$process_param_values_ref[3]\n";
				print OUT "MAXIMAL_LOGP\t$$process_param_values_ref[4]\n";
				print OUT "MINIMAL_LOGP\t$$process_param_values_ref[5]\n";
				print OUT "MAXIMAL_PKD\t$$process_param_values_ref[6]\n";
				print OUT "MINIMAL_PKD\t$$process_param_values_ref[7]\n";
				print OUT "#\n# Similarity Threshold\n#\n";
				print OUT "SIMILARITY_CUTOFF\t$$process_param_values_ref[8]\n";
				print OUT "#\n# Output Files\n#\n";
				print OUT "NUMBER_OF_OUTPUT_MOLECULES\t$$process_param_values_ref[9]\n";
				print OUT "OUTPUT_DIRECTORY\t$$process_param_values_ref[10]\n";
				print OUT "#\n#\n#\n";
				close OUT;
				$$edit_process_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
			} else {print "what?"}
		}
	} elsif ($opt eq 'delete') {
		my $temp_var = $fw_tabbed_frame->raised();
		if ($temp_var eq 'Pocket') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_pocket_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_pocket_file_ref" and die "deleting $$edit_pocket_file_ref returned $?\n";
				$$edit_pocket_file_ref = '';
				for (my $i=1;$i<=10;$i++) {
					$$pocket_param_values_ref[$i] = '';
				}
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		} elsif ($temp_var eq 'Grow') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_grow_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_grow_file_ref" and die "deleting $$edit_grow_file_ref returned $?\n";
				$$edit_grow_file_ref = '';
				for (my $i=1;$i<=7;$i++) {
					$$grow_param_values_ref[$i] = '';
				}
				$$grow_param_values_ref[8] = '0.00';
				$$grow_param_values_ref[9] = '0.00';
				$$grow_param_values_ref[10] = '0.00';
				for (my $i=11;$i<=26;$i++) {
					$$grow_param_values_ref[$i] = '';
				}
				$$grow_param_values_ref[27] = '0.00';
				$$grow_param_values_ref[28] = '0.00';
				for (my $i=29;$i<=30;$i++) {
					$$grow_param_values_ref[$i] = '';
				}
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		} elsif ($temp_var eq 'Link') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_link_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_link_file_ref" and die "deleting $$edit_link_file_ref returned $?\n";
				$$edit_link_file_ref = '';
				for (my $i=1;$i<=7;$i++) {
					$$link_param_values_ref[$i] = '';
				}
				$$link_param_values_ref[8] = '0.00';
				$$link_param_values_ref[9] = '0.00';
				$$link_param_values_ref[10] = '0.00';
				for (my $i=11;$i<=27;$i++) {
					$$link_param_values_ref[$i] = '';
				}
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		} elsif ($temp_var eq 'Process') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_process_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_process_file_ref" and die "deleting $$edit_process_file_ref returned $?\n";
				$$edit_process_file_ref = '';
				for (my $i=1;$i<=7;$i++) {
					$$process_param_values_ref[$i] = '';
				}
				$$process_param_values_ref[8] = '0.00';
				for (my $i=9;$i<=10;$i++) {
					$$process_param_values_ref[$i] = '';
				}
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		} else {print "what?"}
	} elsif ($opt eq 'close') {
		$fw->destroy;
	} else {}
}

sub fw_raised_tab_callback {
	my ($opt,$fw_tabbed_frame,$edit_file_ref,$fw_save_button,$fw_save_as_button,$fw_delete_button) = @_;
	
	if ($$edit_file_ref) {
		$fw_save_button->configure(-state=>'normal');
		$fw_save_as_button->configure(-state=>'normal');
		$fw_delete_button->configure(-state=>'normal');
	} else {
		$fw_save_button->configure(-state=>'disabled');
		$fw_save_as_button->configure(-state=>'disabled');
		$fw_delete_button->configure(-state=>'disabled');
	}
}

sub fw_button_left_callback {
	my ($opt,$fw_entry_ref,$fw) = @_;
	
	if ($opt eq 'open_pdb') {
		my $types=[];
		push @$types, ['PDB files', '*.pdb'];
		push @$types, ['All files', '*'];
		my $file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open PDB File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'open_mol2') {
		my $types=[];
		push @$types, ['MOL2 files', '*.mol2'];
		push @$types, ['All files', '*'];
		my $file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open MOL2 File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'open_txt') {
		my $types=[];
		push @$types, ['TXT files', '*.txt'];
		push @$types, ['All files', '*'];
		my $file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open TXT File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'open_lig') {
		my $types=[];
		push @$types, ['LIG files', '*.lig'];
		push @$types, ['All files', '*'];
		my $file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open LIG File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'open_dir') {
		my $dir = $fw->DirSelect(-title => 'Open Directory')->Show($working_directory);
		if (defined $dir) {
			$$fw_entry_ref = $dir;
		}
	} elsif ($opt eq 'save_pdb') {
		my $types=[];
		push @$types, ['PDB files', '*.pdb'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Write to PDB File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'save_mol2') {
		my $types=[];
		push @$types, ['MOL2 files', '*.mol2'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Write to MOL2 File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'save_txt') {
		my $types=[];
		push @$types, ['TXT files', '*.txt'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Write to TXT File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'save_lig') {
		my $types=[];
		push @$types, ['LIG files', '*.lig'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Write to LIG File', -multiple => 0);
		if (defined $file) {
			$$fw_entry_ref = $file;
		}
	} elsif ($opt eq 'save_dir') {
		my $dir = $fw->DirSelect(-title => 'Write to Directory')->Show($working_directory);
		if (defined $dir) {
			$$fw_entry_ref = $dir;
		}
	} else {}
}

sub fw_more_less_callback {
	my ($opt,$fw_options_frame,$fw_button,$fw) = @_;
		
	if ($opt eq 'grow') {
		if ($grow_param_expand) {
			$fw_button->configure(-image => $arrow_button_image);
			$fw_options_frame->packForget();
			$fw->geometry('');
			$grow_param_expand = 0;
		} else {
			$fw_button->configure(-image => $arrow_180_button_image);
			$fw_options_frame->pack(-side => 'right', -fill=>'both',-expand=>1);
			$fw->geometry('');
			$grow_param_expand = 1;
		}
	} elsif ($opt eq 'link') {
		if ($link_param_expand) {
			$fw_button->configure(-image => $arrow_button_image);
			$fw_options_frame->packForget();
			$fw->geometry('');
			$link_param_expand = 0;
		} else {
			$fw_button->configure(-image => $arrow_180_button_image);
			$fw_options_frame->pack(-side => 'right', -fill=>'both',-expand=>1);
			$fw->geometry('');
			$link_param_expand = 1;
		}
	} else {}
}


# --------------------------------------------------------------------------------------------------------------#
#	Build Ligands - LigBuilder 2.0																				#
# --------------------------------------------------------------------------------------------------------------#


sub parameter_files_20_window {
	my ($default_tab) = @_;

	my ($edit_cavity_file, $edit_build_file);
	my $fw_20 = $mw->Toplevel(-title => 'Edit Parameter Files');
	$fw_20->withdraw;
	$fw_20->grab();
	$fw_20->transient($mw);
	$fw_20->iconimage($program_image);
	&positionwindow($fw_20,'500','620','-5','-35');
	my $fw_20_options_frame = $fw_20->LabFrame(-label => "Parameter Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_20_buttons_frame = $fw_20->Frame()->pack(-side => 'bottom', -fill=>'none',-expand=>0);
	my $fw_20_tabbed_frame = $fw_20_options_frame->NoteBook(-dynamicgeometry => 0)->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_20_cavity_frame = $fw_20_tabbed_frame->add('Cavity', -label => 'Cavity');
	my $fw_20_build_frame = $fw_20_tabbed_frame->add('Build', -label => 'Build');
	my $fw_20_filename_cavity_frame = $fw_20_cavity_frame->Frame()->pack(-side => 'top', -fill=>'x',-expand=>0, -padx => 10);
	my $fw_20_filename_cavity_entry = $fw_20_filename_cavity_frame->LabEntry(-label => 'Parameter File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_cavity_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_20_filename_cavity_browse_button = $fw_20_filename_cavity_frame->Button(-text => '...', -image => $browse_button_image)->pack( -side => 'right');
	my $fw_20_text_cavity_frame = $fw_20_cavity_frame->LabFrame(-label => "Text Editor", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_20_text_cavity = $fw_20_text_cavity_frame->Scrolled('Text', -scrollbars => 'osoe', '-wrap','none','-background', 'white','-foreground', 'black','-relief', 'sunken', -insertofftime => 0)->pack(-fill => 'both', -expand => 1);
	my $fw_20_filename_build_frame = $fw_20_build_frame->Frame()->pack(-side => 'top', -fill=>'both',-expand=>0, -padx => 10);
	my $fw_20_filename_build_entry = $fw_20_filename_build_frame->LabEntry(-label => 'Parameter File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_build_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_20_filename_build_browse_button = $fw_20_filename_build_frame->Button(-text => '...', -image => $browse_button_image)->pack( -side => 'right');
	my $fw_20_text_build_frame = $fw_20_build_frame->LabFrame(-label => "Text Editor", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_20_text_build = $fw_20_text_build_frame->Scrolled('Text', -scrollbars => 'osoe', '-wrap','none','-background', 'white','-foreground', 'black','-relief', 'sunken', -insertofftime => 0)->pack(-fill => 'both', -expand => 1);
	my $fw_20_new_button = $fw_20_buttons_frame->Button(-text => 'New')->pack( -side => 'left', -padx => 1);
	my $fw_20_save_button = $fw_20_buttons_frame->Button(-text => 'Save',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_20_save_as_button = $fw_20_buttons_frame->Button(-text => 'Save As',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_20_delete_button = $fw_20_buttons_frame->Button(-text => 'Delete',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_20_close_button = $fw_20_buttons_frame->Button(-text => 'Close', -command => [\&fw_20_button_callback,'close',$fw_20])->pack( -side => 'left', -padx => 1);
	
	$fw_20_filename_cavity_browse_button->configure(-command => [\&fw_20_browse_file_callback,'cavity',$fw_20,\$edit_cavity_file,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button,$fw_20_text_cavity,$fw_20_text_build]);
	$fw_20_filename_build_browse_button->configure(-command => [\&fw_20_browse_file_callback,'build',$fw_20,\$edit_build_file,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button,$fw_20_text_cavity,$fw_20_text_build]);
	$fw_20_new_button->configure(-command => [\&fw_20_button_callback,'new',$fw_20,$fw_20_tabbed_frame,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button,\$edit_cavity_file,\$edit_build_file,$fw_20_text_cavity,$fw_20_text_build]);
	$fw_20_save_button->configure(-command => [\&fw_20_button_callback,'save',$fw_20,$fw_20_tabbed_frame,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button,\$edit_cavity_file,\$edit_build_file,$fw_20_text_cavity,$fw_20_text_build]);
	$fw_20_save_as_button->configure(-command => [\&fw_20_button_callback,'save_as',$fw_20,$fw_20_tabbed_frame,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button,\$edit_cavity_file,\$edit_build_file,$fw_20_text_cavity,$fw_20_text_build]);
	$fw_20_delete_button->configure(-command => [\&fw_20_button_callback,'delete',$fw_20,$fw_20_tabbed_frame,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button,\$edit_cavity_file,\$edit_build_file,$fw_20_text_cavity,$fw_20_text_build]);
	$fw_20_tabbed_frame->pageconfigure('Cavity', -raisecmd => [\&fw_20_raised_tab_callback,'cavity',$fw_20_tabbed_frame,\$edit_cavity_file,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button]);
	$fw_20_tabbed_frame->pageconfigure('Build',-raisecmd => [\&fw_20_raised_tab_callback,'build',$fw_20_tabbed_frame,\$edit_build_file,$fw_20_save_button,$fw_20_save_as_button,$fw_20_delete_button]);

	if (defined $default_tab) {
		if ($selected_cavity_par_file && -e $selected_cavity_par_file) {			
			$edit_cavity_file = $selected_cavity_par_file;
			&populate_cavity(\$edit_cavity_file,$fw_20_text_cavity);
		}
		if ($selected_build_par_file && -e $selected_build_par_file) {
			$edit_build_file = $selected_build_par_file;
			&populate_build(\$edit_build_file,$fw_20_text_build);
		}		
		if ($default_tab eq 'cavity') {
			$fw_20_tabbed_frame->raise('Cavity');
		} elsif ($default_tab eq 'build') {
			$fw_20_tabbed_frame->raise('Build');
		} else {}	
	}
	$fw_20->deiconify;
}

sub fw_20_browse_file_callback {
	my ($opt,$fw,$edit_file_ref,$fw_save_button,$fw_save_as_button,$fw_delete_button,$fw_20_text_cavity,$fw_20_text_build) = @_;

	my $types=[];
	push @$types, ['Parameter files', '*.input'];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'cavity') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_cavity(\$file,$fw_20_text_cavity);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}		
	} elsif ($opt eq 'build') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_build(\$file,$fw_20_text_build);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} else {}
}

sub fw_20_raised_tab_callback {
	my ($opt,$fw_tabbed_frame,$edit_file_ref,$fw_save_button,$fw_save_as_button,$fw_delete_button) = @_;
	
	if ($$edit_file_ref) {
		$fw_save_button->configure(-state=>'normal');
		$fw_save_as_button->configure(-state=>'normal');
		$fw_delete_button->configure(-state=>'normal');
	} else {
		$fw_save_button->configure(-state=>'disabled');
		$fw_save_as_button->configure(-state=>'disabled');
		$fw_delete_button->configure(-state=>'disabled');
	}
}

sub populate_cavity {
	my ($file_ref,$fw_20_text_cavity) = @_;
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	$fw_20_text_cavity->delete('1.0','end');
	foreach my $line (<IN>) {
		$fw_20_text_cavity->insert('end',$line);
	}
	close IN;
}

sub populate_build {
	my ($file_ref,$fw_20_text_build) = @_;
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	$fw_20_text_build->delete('1.0','end');
	foreach my $line (<IN>) {
		$fw_20_text_build->insert('end',$line);
	}
	close IN;
}

sub fw_20_button_callback {
	my ($opt,$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,$edit_cavity_file_ref,$edit_build_file_ref,$fw_20_text_cavity,$fw_20_text_build) = @_;
	
	if ($opt eq 'new') {
		my $types=[];
		push @$types, ['Parameter files', '*.input'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Create New Parameter File', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
			my $temp_var = $fw_tabbed_frame->raised();
			if ($temp_var eq 'Cavity') {
				$fw_20_text_cavity->delete('1.0','end');
				$$edit_cavity_file_ref = $file;
				
			} elsif ($temp_var eq 'Build') {
				$$edit_build_file_ref = $file;
				$fw_20_text_build->delete('1.0','end');				
			} else {print "what?"}
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} elsif ($opt eq 'save') {
		my $temp_var = $fw_tabbed_frame->raised();
		if ($temp_var eq 'Cavity') {
			open OUT, '>', "$$edit_cavity_file_ref" or die "Error trying to open $$edit_cavity_file_ref. $!";
			print OUT $fw_20_text_cavity->Contents();
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_cavity_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		} elsif ($temp_var eq 'Build') {
			open OUT, '>', "$$edit_build_file_ref" or die "Error trying to open $$edit_build_file_ref. $!";
			print OUT $fw_20_text_build->Contents();
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_build_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		} else {}
	} elsif ($opt eq 'save_as') {
		my $types=[];
		push @$types, ['Parameter files', '*.input'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Save Parameter File As', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
			my $temp_var = $fw_tabbed_frame->raised();
			if ($temp_var eq 'Cavity') {
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT $fw_20_text_cavity->Contents();
				close OUT;
				$$edit_cavity_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
			} elsif ($temp_var eq 'Build') {
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT $fw_20_text_build->Contents();
				close OUT;
				$$edit_build_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
			} else {}
		}
	} elsif ($opt eq 'delete') {
		my $temp_var = $fw_tabbed_frame->raised();
		if ($temp_var eq 'Cavity') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_cavity_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_cavity_file_ref" and die "deleting $$edit_cavity_file_ref returned $?\n";
				$$edit_cavity_file_ref = '';
				$fw_20_text_cavity->delete('1.0','end');	
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		} elsif ($temp_var eq 'Build') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_build_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_build_file_ref" and die "deleting $$edit_build_file_ref returned $?\n";
				$$edit_build_file_ref = '';
				$fw_20_text_build->delete('1.0','end');	
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		} else {}
	} elsif ($opt eq 'close') {
		$fw->destroy;
	} else {}
}


# --------------------------------------------------------------------------------------------------------------#
#	Pharmacophore					    							#
# --------------------------------------------------------------------------------------------------------------#


sub configure_pha_files_window {
	my ($default_tab) = @_;
	
	my ($edit_pha_file);
	my $fw_conf = $mw->Toplevel(-title => 'Edit Parameter Files');
	$fw_conf->withdraw;
	$fw_conf->grab();
	$fw_conf->transient($mw);
	$fw_conf->iconimage($program_image);
	&positionwindow($fw_conf,'500','620','-5','-35');
	my $fw_conf_options_frame = $fw_conf->LabFrame(-label => "Configuration Files", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_conf_buttons_frame = $fw_conf->Frame()->pack(-side => 'bottom', -fill=>'none',-expand=>0);
	my $fw_conf_tabbed_frame = $fw_conf_options_frame->NoteBook(-dynamicgeometry => 0)->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_conf_pha_frame = $fw_conf_tabbed_frame->add('pharmACOphore configuration', -label => 'pharmACOphore');
	my $fw_conf_filename_pha_frame = $fw_conf_pha_frame->Frame()->pack(-side => 'top', -fill=>'x',-expand=>0, -padx => 10);
	my $fw_conf_filename_pha_entry = $fw_conf_filename_pha_frame->LabEntry(-label => 'Configuration File:', -labelFont => ['Courier', '10', 'bold'], -labelPack => [ -side => "left" ], -textvariable => \$edit_pha_file, -background => 'white', -insertofftime => 0, -state => 'readonly')->pack(-side => 'left', -padx => 3, -fill => 'x', -expand => 1);
	my $fw_conf_filename_pha_browse_button = $fw_conf_filename_pha_frame->Button(-text => '...', -image => $browse_button_image)->pack( -side => 'right');
	my $fw_conf_text_pha_frame = $fw_conf_pha_frame->LabFrame(-label => "Text Editor", -labelside => "acrosstop")->pack(-side => 'top', -fill=>'both',-expand=>1);
	my $fw_conf_text_pha = $fw_conf_text_pha_frame->Scrolled('Text', -scrollbars => 'osoe', '-wrap','none','-background', 'white','-foreground', 'black','-relief', 'sunken', -insertofftime => 0)->pack(-fill => 'both', -expand => 1);
	my $fw_conf_new_button = $fw_conf_buttons_frame->Button(-text => 'New')->pack( -side => 'left', -padx => 1);
	my $fw_conf_save_button = $fw_conf_buttons_frame->Button(-text => 'Save',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_conf_save_as_button = $fw_conf_buttons_frame->Button(-text => 'Save As',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_conf_delete_button = $fw_conf_buttons_frame->Button(-text => 'Delete',-state => 'disabled')->pack( -side => 'left', -padx => 1);
	my $fw_conf_close_button = $fw_conf_buttons_frame->Button(-text => 'Close', -command => [\&fw_conf_button_callback,'close',$fw_conf])->pack( -side => 'left', -padx => 1);
	$fw_conf_filename_pha_browse_button->configure(-command => [\&fw_conf_browse_file_callback,'pha',$fw_conf,\$edit_pha_file,$fw_conf_save_button,$fw_conf_save_as_button,$fw_conf_delete_button,$fw_conf_text_pha]);
	$fw_conf_new_button->configure(-command => [\&fw_conf_button_callback,'new',$fw_conf,$fw_conf_tabbed_frame,$fw_conf_save_button,$fw_conf_save_as_button,$fw_conf_delete_button,\$edit_pha_file,$fw_conf_text_pha]);
	$fw_conf_save_button->configure(-command => [\&fw_conf_button_callback,'save',$fw_conf,$fw_conf_tabbed_frame,$fw_conf_save_button,$fw_conf_save_as_button,$fw_conf_delete_button,\$edit_pha_file,$fw_conf_text_pha]);
	$fw_conf_save_as_button->configure(-command => [\&fw_conf_button_callback,'save_as',$fw_conf,$fw_conf_tabbed_frame,$fw_conf_save_button,$fw_conf_save_as_button,$fw_conf_delete_button,\$edit_pha_file,$fw_conf_text_pha]);
	$fw_conf_delete_button->configure(-command => [\&fw_conf_button_callback,'delete',$fw_conf,$fw_conf_tabbed_frame,$fw_conf_save_button,$fw_conf_save_as_button,$fw_conf_delete_button,\$edit_pha_file,$fw_conf_text_pha]);
										
	if (defined $default_tab) {
		if ($selected_pha_conf_file && -e $selected_pha_conf_file) {			
			$edit_pha_file = $selected_pha_conf_file;
			&populate_pha(\$edit_pha_file,$fw_conf_text_pha);
			$fw_conf_save_button->configure(-state=>'normal');
			$fw_conf_save_as_button->configure(-state=>'normal');
			$fw_conf_delete_button->configure(-state=>'normal');
		}	
	}
	$fw_conf->deiconify;
}

sub fw_conf_browse_file_callback {
	my ($opt,$fw,$edit_file_ref,$fw_save_button,$fw_save_as_button,$fw_delete_button,$fw_conf_text_pha) = @_;

	my $types=[];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'pha') {
		$file = $fw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$$edit_file_ref = $file;
			&populate_pha(\$file,$fw_conf_text_pha);
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}		
	} else {}
}

sub populate_pha {
	my ($file_ref,$fw_conf_text_pha) = @_;
	open IN, "$$file_ref" or die "Error trying to open $$file_ref!";
	$fw_conf_text_pha->delete('1.0','end');
	foreach my $line (<IN>) {
		$fw_conf_text_pha->insert('end',$line);
	}
	close IN;
}

sub fw_conf_button_callback {
	my ($opt,$fw,$fw_tabbed_frame,$fw_save_button,$fw_save_as_button,$fw_delete_button,$edit_pha_file_ref,$fw_conf_text_pha) = @_;
	
	if ($opt eq 'new') {
		my $types=[];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Create New Parameter File', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
				$fw_conf_text_pha->delete('1.0','end');
				$$edit_pha_file_ref = $file;	
			$fw_save_button->configure(-state=>'normal');
			$fw_save_as_button->configure(-state=>'normal');
			$fw_delete_button->configure(-state=>'normal');
		}
	} elsif ($opt eq 'save') {
			open OUT, '>', "$$edit_pha_file_ref" or die "Error trying to open $$edit_pha_file_ref. $!";
			print OUT $fw_conf_text_pha->Contents();
			close OUT;
			$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$$edit_pha_file_ref\' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
	} elsif ($opt eq 'save_as') {
		my $types=[];
		push @$types, ['Parameter files', '*.input'];
		push @$types, ['All files', '*'];
		my $file = $fw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Save Parameter File As', -multiple => 0);
		if (defined $file) {
			system "touch $file" and die "Creating $file returned $?.";
				open OUT, '>', "$file" or die "Error trying to open $file. $!";
				print OUT $fw_conf_text_pha->Contents();
				close OUT;
				$$edit_pha_file_ref = $file;
				$fw -> MsgBox( -title => 'Save Parameter File', -message=>"File \'$file' was successfully saved.",-type=>'ok',-icon=>'info')->Show;
		}
	} elsif ($opt eq 'delete') {
			my $d = $fw -> MsgBox( -title => 'Delete File', -message=>"Are you sure you want to permanently delete file \'$$edit_pha_file_ref\'?",-type=>'yesno',-icon=>'question');
			my $button = $d->Show;
			if( $button eq "yes" ) {
				system "rm $$edit_pha_file_ref" and die "deleting $$edit_pha_file_ref returned $?\n";
				$$edit_pha_file_ref = '';
				$fw_conf_text_pha->delete('1.0','end');	
				$fw_save_button->configure(-state=>'disabled');
				$fw_save_as_button->configure(-state=>'disabled');
				$fw_delete_button->configure(-state=>'disabled');
			}
		# } else {}
	} elsif ($opt eq 'close') {
		$fw->destroy;
	} else {}
}

# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Input Preparation																		#
# --------------------------------------------------------------------------------------------------------------#


sub preprocess_callback {
	my ($opt) = @_;

	if ($opt eq 'pdb2pqr') {
		# unless (($selected_pre_file && -e $selected_pre_file) && $selected_pre_save_file) {
		unless ($selected_pre_file && $selected_pre_save_file) {
			$mw -> MsgBox( -title => "Run $opt", -message=>"You must provide input and output PDB files for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			$Tab_frame->pageconfigure('EM',-state => 'disable');
			$Tab_frame->pageconfigure('Build',-state => 'disable');
			$Tab_frame->pageconfigure('pha',-state => 'disable');
			$output_text->insert('end', "----------\nPreparing Receptor Protein PDB File...\n----------\n\n",'red'); $output_text->SetCursor('end');
			syswrite LOG, "----------\nPreparing Receptor Protein PDB File...\n----------\n\n" if $enable_log;

			unless (-e $selected_pre_file) {
				$output_text->insert('end', "Downloading PDB File with ID $selected_pre_file from www.rcsb.org...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Downloading PDB File with ID $selected_pre_file from www.rcsb.org...\n\n" if $enable_log;
				my $temp_pdb = File::Spec->catfile($drugOns_directory,'.tmp',"$selected_pre_file.pdb");
				eval {
					my $url = 'http://www.rcsb.org/pdb/files/'."$selected_pre_file".'.pdb';
					# print $url;
					my $m = WWW::Mechanize->new();
					$m->get($url);
					die("Downloading failed probably due to an unknown PDB id.\n  Please make sure that you have typed the PDB id correctly.") if ($m->text =~ /We're sorry, but/);
					open FH, ">", $temp_pdb or die("Error creating file $temp_pdb");
					print FH $m->content;
					close FH;
					$selected_pre_file = $temp_pdb;
					$output_text->insert('end', "  Created temporary file $temp_pdb successfully.\n\n"); $output_text->SetCursor('end');
					syswrite LOG, "  Created temporary file $temp_pdb successfully.\n\n" if $enable_log;
					$output_text->insert('end', "Done!\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Done.\n\n" if $enable_log;
				};	
				if ($@) {
					$preprocess_current_step = 0;
					$fast_switches[0] = 2;
					if (-e $temp_pdb) { system "rm $temp_pdb" and die "deleting $temp_pdb returned $?\n"};
					$output_text->insert('end', "ERROR:\n  $@");$output_text->SetCursor('end');
					$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
					$output_text->insert('end', "\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
					syswrite LOG, "ERROR:\n  $@Failure!\n\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n" if $enable_log;
					$pre_button->configure(-state => 'normal');
					$pre_interrupt_button->configure(-state => 'disable');
					$Tab_frame->pageconfigure('EM',-state => 'normal');
					$Tab_frame->pageconfigure('Build',-state => 'normal');
					$Tab_frame->pageconfigure('pha',-state => 'normal');
					
					return;
				}
			}
			my $temp_file;
			my $temp_file_2;
			my $pre_final_file;
			my $abs_selected_pre_save_file = File::Spec->rel2abs($selected_pre_save_file);
			my ($volume,$directories,$out_file) = File::Spec->splitpath($abs_selected_pre_save_file);
			$temp_file = File::Spec->catfile($drugOns_directory,'.tmp',$out_file.'.'.$$);
			$temp_file_2 = $temp_file.'_2';
			$pre_button->configure(-state => 'disable');
			$pre_interrupt_button->configure(-state => 'normal');
			if ($pre_rem_hetatm == 1) {
				$preprocess_current_step = 1;
				$output_text->insert('end', "Removing HETATM Records from the PDB File...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Removing HETATM Records from the PDB File...\n\n" if $enable_log;
				eval {
					open IN, $selected_pre_file  or die "error reading input PDB: $! -";
					open OUT, '>', $temp_file or die "error writing output temp PDB file: $! -";
					while (my $line = <IN>)	{
						next if ($line =~ /^HETATM/);
						print OUT $line;
						}
					close OUT;
					close IN;
					$output_text->insert('end', "  Created temporary file $temp_file successfully.\n\n"); $output_text->SetCursor('end');
					syswrite LOG, "  Created temporary file $temp_file successfully.\n\n" if $enable_log;
					$output_text->insert('end', "Done!\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Done.\n\n" if $enable_log;
				};	
				if ($@) {
					$preprocess_current_step = 0;
					$fast_switches[0] = 2;
					if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
					if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
					$output_text->insert('end', "ERROR:\n  $@");$output_text->SetCursor('end');
					$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
					$output_text->insert('end', "\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
					syswrite LOG, "ERROR:\n  $@Failure!\n\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n" if $enable_log;
					$pre_button->configure(-state => 'normal');
					$pre_interrupt_button->configure(-state => 'disable');
					$Tab_frame->pageconfigure('EM',-state => 'normal');
					$Tab_frame->pageconfigure('Build',-state => 'normal');
					$Tab_frame->pageconfigure('pha',-state => 'normal');
					return;
				}
			} else {}
			my $temp = 10;
			$progress = $temp;
			$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
			if ($pre_pdb2pqr == 1) { # not necessary. $pre_pdb2pqr must always be one
				$preprocess_current_step = 2;
				my $pdb2pqr_real_input;
				if ($pre_rem_hetatm == 1) {
					$pdb2pqr_real_input = $temp_file;
				} else {
					$pdb2pqr_real_input = $selected_pre_file;
				}
				
				if ($pre_remote[1]) {
					$output_text->insert('end', "Running $opt on Server...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Running $opt On Server...\n\n" if $enable_log;
				} else {
					$output_text->insert('end', "Running $opt locally...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Running $opt locally...\n\n" if $enable_log;
				}
				
				my $cmd_params = "--verbose --ff $pdb2pqr_ff";
				if ($neutralize[0] == 1) {
					$cmd_params = '--neutralc '.$cmd_params;
				}
				if ($neutralize[1] == 1) {
					$cmd_params = '--neutraln '.$cmd_params;
				}
				my $cmd;
				my $env;
				if ($pre_remote[1]) {
					my ($v,$d,$in_file_neat) = File::Spec->splitpath($pdb2pqr_real_input);
					my ($v2,$d2,$out_file_neat) = File::Spec->splitpath($temp_file_2);
					$env = 'PYTHONPATH='.File::Spec->catfile($drugOns_directory,'Supplementary_Files','opal-py-2.4.1','prereqs','ZSI-2.1_a2-py2.6.egg').':$PYTHONPATH ';
					$cmd = 'python '.File::Spec->catfile($drugOns_directory,'Supplementary_Files','opal-py-2.4.1','GenericServiceClient.py').' -l http://ws.nbcr.net/opal2/services/Pdb2pqrOpalService -r launchJobBlocking -a "'.$cmd_params.' '.$in_file_neat.' '.$out_file_neat.'" -f '.$pdb2pqr_real_input;
					
				} else {
					$env = '';
					if ($pdb2pqr_path) {
						$cmd = 'python '.File::Spec->catfile($pdb2pqr_path,'pdb2pqr.py').' '.$cmd_params.' '.$pdb2pqr_real_input.' '.$temp_file_2;
					} else {
						$cmd = 'python '.'pdb2pqr.py'.' '.$cmd_params.' '.$pdb2pqr_real_input.' '.$temp_file_2;
					}
				}
								
				my $flag;
				my $cmd_handle;
				$pid = open ($cmd_handle, "$env$stdbuf_str $cmd 2>&1 |") or $flag = $!;
				# print $pid;
				#$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
				unless ($flag){
					syswrite LOG, "\$ $cmd\n" if $enable_log;
					#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
					$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
					if ($pre_remote[1]) {
						$mw->fileevent($cmd_handle,'readable', [\&working_pre_remote,$cmd_handle, $opt,$temp_file,$temp_file_2,$abs_selected_pre_save_file]);
					} else {
						$mw->fileevent($cmd_handle,'readable', [\&working_pre,$cmd_handle, $opt,$temp_file,$temp_file_2,$abs_selected_pre_save_file]);
					}
				} else {
					$preprocess_current_step = 0;
					$fast_switches[0] = 2;
					$mw -> MsgBox( -title => 'Run $opt', -message=>"An error occured trying to run $opt!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
				}
			} else {}
		}
	} elsif ($opt eq 'interrupt') {
		if (my $ans = $mw->messageBox( -icon => 'question',-message => "Are you sure you want to interrupt the running process?", -title => 'Interrupt process',-type => 'YesNo') =~ /Yes/i) {
			syswrite LOG, "\nInterrupted by User.\n" if $enable_log;
			#system("kill -9 $pid") and die("Could not send kill signal to $pid");
			#print $pid;
			my $temp_cmd_handle;
			my $temp_pid = open ($temp_cmd_handle, "ps -o pid --no-headers --ppid $pid |") or die('error in ps');
			my $child_pid = <$temp_cmd_handle>;
			#print $child_pid;
			if ($child_pid) {
				kill 1, $child_pid;
			} else {
				kill 1, $pid;
			}
			$preprocess_current_step = 0;
			my $abs_selected_pre_save_file = File::Spec->rel2abs($selected_pre_save_file);
			my ($volume,$directories,$out_file) = File::Spec->splitpath($abs_selected_pre_save_file);
			my $temp_file = File::Spec->catfile($drugOns_directory,'.tmp',$out_file.'.'.$$);
			my $temp_file_2 = $temp_file.'_2';
			if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
			if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
			$output_text->insert('end', "Interrupted by User.\n",'boldy'); $output_text->SetCursor('end');
			$pre_button->configure(-state => 'normal');
			$pre_interrupt_button->configure(-state => 'disable');
			$Tab_frame->pageconfigure('EM',-state => 'normal');
			$Tab_frame->pageconfigure('Build',-state => 'normal');
			$Tab_frame->pageconfigure('pha',-state => 'normal');
			sleep(1);
			$fast_switches[0] = -1;
		}
	} elsif ($opt eq 'remote_clicked') {
		if ($pre_remote[1]) {
			$pre_remote[0] = 0;
		} else {
			$pre_remote[0] = 1;
		}
	} elsif ($opt eq 'local_clicked') {
		if ($pre_remote[0]) {
			$pre_remote[1] = 0;
		} else {
			$pre_remote[1] = 1;
		}
	} else {}
}

sub working_pre {
	my ($cmd_handle,$opt,$temp_file,$temp_file_2,$abs_selected_pre_save_file) = @_;
	
	my $redline;
	my $stat = sysread $cmd_handle, $redline, 4096;
	unless ($stat == 0) {
		$output_text->insert('end', "$redline");
		$output_text->yview('end');
		syswrite LOG, $redline if $enable_log;
		&update_remaining_pre($redline,$opt);
	} else {
		$preprocess_current_step++;
		$mw->fileevent($cmd_handle ,'readable','');
		&tag_output_pre();
		if (-e $temp_file_2) {
			$output_text->insert('end', "  Created temporary file $temp_file successfully.\n\n"); $output_text->SetCursor('end');
			syswrite LOG, "  Created temporary file $temp_file successfully.\n\n" if $enable_log;
			$output_text->insert('end', "Execution of $opt finished successfully.\n",'boldy'); $output_text->SetCursor('end');
			syswrite LOG, "Execution of $opt finished successfully.\n" if $enable_log;
			my $temp = 95;
			$progress = $temp;
			$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
			$output_text->insert('end', "\nWriting output PDB File...\n",'boldy'); $output_text->SetCursor('end');
			syswrite LOG, "\nWriting output PDB File...\n" if $enable_log;
			eval {
				my @gaps;
				open IN, $temp_file_2  or die "error reading temp PDB file: $!";
				open OUT, '>', $abs_selected_pre_save_file or die "error writing output PDB file: $!";
				print OUT "REMARK   1 Output PDB File from Input preparation Step - DrugOn\nREMARK   1\n";
				while (my $line = <IN>)	{
					print OUT $line;
					#find gaps
					push @gaps,($line) if $line =~ /^REMARK\s+5\s+Gap/;
				}
				close OUT;
				close IN;
				# print "@gaps";
				$output_text->insert('end', "Done!\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Done.\n" if $enable_log;
				$temp = 100;
				$progress = $temp;
				$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
				# $output_text->insert('end', "\nDeleting Temporary Files...\n",'boldy'); $output_text->SetCursor('end');
				# syswrite LOG, "\nDeleting Temporary Files...\n" if $enable_log;	
				if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
				if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};				
				# $output_text->insert('end', "Done!\n\n",'boldy'); $output_text->SetCursor('end');
				# syswrite LOG, "Done.\n\n" if $enable_log;
				if (@gaps) {
					$fast_switches[0] = 3;
					$output_text->insert('end', "\nProtein was successfully processed but gaps were found in the structure...\n\n @gaps\n Gaps in the protein backbone may lead to errors later in the process \n \(Especially during EM and MD calculations\). Please take the time to correct these\n problems in the structure and run input preparation again to check your work.\n If you have a multi-chain protein file you can fairly safely ignore this message.\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "\\nProtein was successfully processed but gaps were found in the structure...\n\n @gaps\n Gaps in the protein backbone may lead to errors later in the process \n \(Especially during EM and MD calculations\). Please take the time to correct these\n problems in the structure and run input preparation again to check your work.\nIf you have a multi-chain protein file you can fairly safely ignore this message.\n" if $enable_log;
				} else {
					$fast_switches[0] = 1;
					$output_text->insert('end', "\nProtein was successfully processed.\n\n\tPlease make sure to at least have a quick check at the PDB file for any structural on naming issues\n\tthat may were reported by PDB2PQR, since they can cause headaches later on in the process.\n\tThey can be found at the Remarks Section, in the beginning of the PDB file.\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "\nProtein was successfully processed.\n" if $enable_log;
				}
			};
			if ($@) {
				$fast_switches[0] = 2;
				if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
				if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
				$output_text->insert('end', "\nERROR:\n  $@");$output_text->SetCursor('end');
				$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "ERROR:\n  $@Failure!\n\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n" if $enable_log;
			}
		} else {
			$fast_switches[0] = 2;
			if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
			if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
			$output_text->insert('end', "\nExecution of $opt failed.\n",'boldy'); $output_text->SetCursor('end');
			syswrite LOG, "\nExecution of $opt failed.\n" if $enable_log;
		}
		$preprocess_current_step = 0;
		$pre_button->configure(-state => 'normal');
		$pre_interrupt_button->configure(-state => 'disable');
		$Tab_frame->pageconfigure('EM',-state => 'normal');
		$Tab_frame->pageconfigure('Build',-state => 'normal');
		$Tab_frame->pageconfigure('pha',-state => 'normal');
		$progress = 0; $pre_steps=0;
		$pb_label->configure(-text => '0%');
		$running_id->cancel;
		$pb__bulb_label->configure(-image => $bulb_off_image);
		$running_bulb = 0;
		$output_text->insert('end', "\n----------\nFinished Preparing Receptor PDB File...\n----------\n\n",'red'); $output_text->SetCursor('end');
		syswrite LOG, "\n----------\nFinished Preparing Receptor PDB File...\n----------\n\n" if $enable_log;
	}
}

sub working_pre_remote {
	my ($cmd_handle,$opt,$temp_file,$temp_file_2,$abs_selected_pre_save_file) = @_;
	
	my $redline;
	my $stat = sysread $cmd_handle, $redline, 4096;
	unless ($stat == 0) {
		$output_text->insert('end', "$redline");
		$output_text->yview('end');
		syswrite LOG, $redline if $enable_log;
		&update_remaining_pre($redline,$opt);
	} else {
		$preprocess_current_step++;
		$mw->fileevent($cmd_handle ,'readable','');
		&tag_output_pre();
		my $link_base = &find_links_pre();
		if ($link_base eq '') {
			$fast_switches[0] = 2;
			if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
			if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
			$output_text->insert('end', "\nERROR handling pdb2pqr results from the server:\n  If you didn't interrupt the process\n  make sure there are no Internet connection problems.\n");$output_text->SetCursor('end');
			$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
			syswrite LOG, "ERROR handling pdb2pqr results from the server\nFailure!\n\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n" if $enable_log;
		} else {
			my $url = $link_base.'/stderr.txt';
			my $m = WWW::Mechanize->new();
			eval {
				$m->get($url);
				my $c = $m->content;
				die($c) if ($c =~ /error/gi);
				$url = $link_base.'/stdout.txt';
				$m->get($url);
				$c = $m->content;
				$output_text->insert('end', "\nPrinting pdb2pqr output details...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "\nPrinting pdb2pqr output details...\n\n" if $enable_log;
				$output_text->insert('end', "$c"); $output_text->SetCursor('end');
				syswrite LOG, "$c" if $enable_log;
				my ($v2,$d2,$out_file_neat) = File::Spec->splitpath($temp_file_2);
				$url = $link_base.'/'.$out_file_neat;
				$output_text->insert('end', "  Downloading Output PDB into temporary file $temp_file_2...\n"); $output_text->SetCursor('end');
				syswrite LOG, "  Downloading Output PDB into temporary file $temp_file_2...\n" if $enable_log;
				$m->get($url);
				$c = $m->content;
				open FH, ">", $temp_file_2 or die("error opening $temp_file_2");
				print FH $c;
				close FH;
			};
			if ($@) {
				$fast_switches[0] = 2;
				if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
				if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
				$output_text->insert('end', "\nError while handling Pdb2pqr server results:\n  $@");$output_text->SetCursor('end');
				$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Error while handling Pdb2pqr server results:\n  $@Failure!\n\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n" if $enable_log;
			}
			if (-e $temp_file_2) {
				$output_text->insert('end', "  Created temporary file $temp_file_2 successfully.\n\n"); $output_text->SetCursor('end');
				syswrite LOG, "  Created temporary file $temp_file_2 successfully.\n\n" if $enable_log;
				$output_text->insert('end', "Execution of $opt finished successfully.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Execution of $opt finished successfully.\n" if $enable_log;
				my $temp = 95;
				$progress = $temp;
				$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
				$output_text->insert('end', "\nWriting output PDB File...\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "\nWriting output PDB File...\n" if $enable_log;
				eval {
					my @gaps;
					open IN, $temp_file_2  or die "error reading temp PDB file: $!";
					open OUT, '>', $abs_selected_pre_save_file or die "error writing output PDB file: $!";
					print OUT "REMARK   1 Output PDB File from Input preparation Step - DrugOn\nREMARK   1\n";
					while (my $line = <IN>)	{
						print OUT $line;
						#find gaps
						push @gaps,($line) if $line =~ /^REMARK\s+5\s+Gap/;
					}
					close OUT;
					close IN;
					$output_text->insert('end', "Done!\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Done.\n" if $enable_log;
					$temp = 100;
					$progress = $temp;
					$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
					# $output_text->insert('end', "\nDeleting Temporary Files...\n",'boldy'); $output_text->SetCursor('end');
					# syswrite LOG, "\nDeleting Temporary Files...\n" if $enable_log;	
					if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
					if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};				
					# $output_text->insert('end', "Done!\n\n",'boldy'); $output_text->SetCursor('end');
					# syswrite LOG, "Done.\n\n" if $enable_log;
					if (@gaps) {
						$fast_switches[0] = 3;
						$output_text->insert('end', "\nProtein was successfully processed but gaps were found in the structure...\n\n @gaps\n Gaps in the protein backbone may lead to errors later in the process \n \(Especially during EM and MD calculations\). Please take the time to correct these\n problems in the structure and run input preparation again to check your work.\nIf you have a multi-chain protein file you can fairly safely ignore this message.\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "\\nProtein was successfully processed but gaps were found in the structure...\n\n @gaps\n Gaps in the protein backbone may lead to errors later in the process \n \(Especially during EM and MD calculations\). Please take the time to correct these\n problems in the structure and run input preparation again to check your work.\nIf you have a multi-chain protein file you can fairly safely ignore this message.\n" if $enable_log;
				} else {
						$fast_switches[0] = 1;
						$output_text->insert('end', "\nProtein was successfully processed.\n\n\tPlease make sure to at least have a quick check at the PDB file for any structural on naming issues\n\tthat may were reported by PDB2PQR, since they can cause headaches later on in the process.\n\tThey can be found at the Remarks Section, in the beginning of the PDB file.\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "\nProtein was successfully processed.\n" if $enable_log;
					}
				};
				if ($@) {
					$fast_switches[0] = 2;
					if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
					if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
					$output_text->insert('end', "\nERROR:\n  $@");$output_text->SetCursor('end');
					$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "ERROR:\n  $@Failure!\n\n----------\nPreparing Receptor Protein PDB File finished.\n----------\n\n" if $enable_log;
				}
			} else {
				$fast_switches[0] = 2;
				if (-e $temp_file) { system "rm $temp_file" and die "deleting $temp_file returned $?\n"};
				if (-e $temp_file_2) { system "rm $temp_file_2" and die "deleting $temp_file_2 returned $?\n"};	
				$output_text->insert('end', "\nExecution of $opt failed.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "\nExecution of $opt failed.\n" if $enable_log;
			}
		}
		$preprocess_current_step = 0;
		$pre_button->configure(-state => 'normal');
		$pre_interrupt_button->configure(-state => 'disable');
		$Tab_frame->pageconfigure('EM',-state => 'normal');
		$Tab_frame->pageconfigure('Build',-state => 'normal');
		$Tab_frame->pageconfigure('pha',-state => 'normal');
		$progress = 0; $pre_steps=0;
		$pb_label->configure(-text => '0%');
		$running_id->cancel;
		$pb__bulb_label->configure(-image => $bulb_off_image);
		$running_bulb = 0;
		$output_text->insert('end', "\n----------\nFinished Preparing Receptor PDB File...\n----------\n\n",'red'); $output_text->SetCursor('end');
		syswrite LOG, "\n----------\nFinished Preparing Receptor PDB File...\n----------\n\n" if $enable_log;
	}
}

sub tag_output_pre {
	my $error_tag = $output_text->search(-backwards,'ValueError','end');
	my $error_tag_2 = $output_text->search(-backwards,'pdb2pqr.py: error:','end');
	$output_text->FindAndReplaceAll(-exact,-case,'error:','ERROR:');
	if ($error_tag) {
		(my $line,my $index) = split(/\./,$error_tag,2);
		my $line2= $line + 1;
		$output_text->tagAdd('boldy',"$line.0","$line2.end");
	}
	if ($error_tag_2) {
		(my $line,my $index) = split(/\./,$error_tag_2,2);
		my $line2= $line + 1;
		$output_text->tagAdd('boldy',"$line.0","$line2.end");
	}
}

sub find_links_pre {
	my @links;
	my $status_tag = $output_text->search(-backwards,'Status: ','end');
	# my $error_tag_2 = $output_text->search(-backwards,'pdb2pqr.py: error:','end');
	# $output_text->FindAndReplaceAll(-exact,-case,'error:','ERROR:');
	if ($status_tag) {
		(my $line,my $index) = split(/\./,$status_tag,2);
		my $line_contents = $output_text->get("$line.0","$line.end");
		#print $line_contents;
		# todo maybe error handling for other statuses
		print "Erronious status returned from Pdb2pqrOpalService" unless $line_contents =~ /Status:  8/;
		#
		# my $line2= $line + 1;
		# $output_text->tagAdd('boldy',"$line.0","$line2.end");
	} else {
		return '';
	}
	$status_tag = $output_text->search(-backwards,'Base Output URL','end');
	if ($status_tag) {
		(my $line,my $index) = split(/\./,$status_tag,2);
		my $line_contents = $output_text->get("$line.0","$line.end");
		#print $line_contents;
		my ($base_html) = ($line_contents =~ /Base Output URL: (.*)/);
		#print $base_html;
		return $base_html;
	} else {
		return '';
	}
	
}

sub update_remaining_pre {
	my ($redline,$opt) = @_;
	my @list= ($redline =~ /Done/g);
	$pre_steps += scalar(@list);
	my $temp = 10 + (90 * ($pre_steps/($pre_complete_steps+1)));
	$progress = $temp;
	$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
}

sub browse_pre_file_callback {
	my ($opt) = @_;
	
	my $types=[];
	push @$types, ['PDB files', '*.pdb'];
	push @$types, ['All files', '*'];
	if ($opt eq 'in') {
		my $file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open PDB File', -multiple => 0);
		if (defined $file) {
			$selected_pre_file = $file;
		}
	} elsif ($opt eq 'out') {
		my $file = $mw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Save as PDB File', -multiple => 0);
		if (defined $file) {
			$selected_pre_save_file = $file;
		}
	}
}







# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Receptor Optimization																				#
# --------------------------------------------------------------------------------------------------------------#


sub gromacs_mdrun_callback {
	my ($opt) = @_;
	
	if ($opt eq 'mdrun') {
		unless (($selected_gro_in_file && -e $selected_gro_in_file) && $selected_gro_out_file && ($selected_mdrun_par_file && -e $selected_mdrun_par_file)) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide input and output PDB files for $opt, as well as the em.mdp parameter file!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			my $abs_selected_gro_out_file = File::Spec->rel2abs($selected_gro_out_file);
			$output_text->insert('end', "----------\nStarting Protein Receptor Optimization...\n----------\n\n",'red'); $output_text->SetCursor('end');
			syswrite LOG, "----------\nStarting Protein Receptor Optimization...\n----------\n\n" if $enable_log;
			my $ff_valid = &gro_translate($gromacs_ff);
			my $water_valid = $gromacs_watermodel;
			my $cmd_params = "-f $selected_gro_in_file -o -p -i -ignh -ff $ff_valid -water $water_valid";
			my $cmd;
			if (-e File::Spec->catfile('/','usr','bin','g_pdb2gmx')) {
				$cmd = 'g_pdb2gmx'.' '.$cmd_params;
			} else {
				$cmd = 'pdb2gmx'.' '.$cmd_params;	
			}
			#my $cmd = 'pdb2gmx'.' '.$cmd_params;
			my $flag;
			my $cmd_handle;
			my $oldwd = getcwd();
			chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
			$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
			chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
			unless ($flag){		
				$gromacs_current_step = 1;		
				$output_text->insert('end', "Running pdb2gmx...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Running pdb2gmx...\n\n" if $enable_log;
				#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
				syswrite LOG, "\$ $cmd\n" if $enable_log;
				$em_button->configure(-state => 'disable');
				$em_interrupt_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('Pre',-state => 'disable');
				$Tab_frame->pageconfigure('Build',-state => 'disable');
				$Tab_frame->pageconfigure('pha',-state => 'disable');
				
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
			} else {
				$gromacs_current_step = 0;
				$mw -> MsgBox( -title => 'Run pdb2gmx', -message=>"An error occured trying to run pdb2gmx!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
				$fast_switches[1] = 2;
			}
		}
	} elsif ($opt eq 'interrupt') {
		if (my $ans = $mw->messageBox( -icon => 'question',-message => "Are you sure you want to interrupt the running process?", -title => 'Interrupt process',-type => 'YesNo') =~ /Yes/i) {
			syswrite LOG, "\nInterrupted by User.\n" if $enable_log;
			$output_text->insert('end', "Interrupted by User.\n",'boldy'); $output_text->SetCursor('end');
			kill 1, $pid;
			$gromacs_has_error = 1;
			sleep(1);
			$fast_switches[1] = -1;
			&working_gro_fin();
		}
	} else {}
}

sub working_gro {
	my ($cmd_handle,$opt,$abs_selected_gro_out_file) = @_;
	
	my $redline;
	my $stat = sysread $cmd_handle, $redline, 4096;
	unless ($stat == 0) {
		$redline =~ s/of generating an error/of generating an ErroR/g;
		$redline =~ s/for error estimate/for ErroR estimate/g;
		if ($redline =~ /Step=/) {
			chomp($redline);
			$redline = $redline."\n";
		}
		$gromacs_has_error = 1 if ($redline =~ /error/);
		$gromacs_has_error = 1 if ($redline =~ /Invalid/);
		
		$output_text->insert('end', "$redline");
		$output_text->yview('end');
		syswrite LOG, $redline if $enable_log;
		&update_remaining_gro($redline,$opt);
	} else {
		$mw->fileevent($cmd_handle ,'readable','');
		&tag_output_gro();
		if ($gromacs_has_error == 1) {
			$gromacs_has_error = 0;
			$fast_switches[1] = 2;
			$output_text->insert('end', "\nExecution of EM step:$gromacs_current_step failed.\n",'boldy'); $output_text->SetCursor('end');
			syswrite LOG, "\nExecution of EM step:$gromacs_current_step failed.\n" if $enable_log;
			&working_gro_fin();
		} else {
			$gromacs_current_step++;
			LOOPA: for (my $i=0;$i<1;$i++) { # dummy for to redo if GOOOOOD??????
			if ($gromacs_current_step == 2) {
				$output_text->insert('end', "pdb2gmx finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "pdb2gmx finished successfully...\n\n" if $enable_log;
				my $cmd_params = "-f -o -d $sol_box_distance -bt $gromacs_boxtype";
				my $cmd;
				if (-e File::Spec->catfile('/','usr','bin','g_editconf')) {
					$cmd = 'g_editconf'.' '.$cmd_params;
				} else {
					$cmd = 'editconf'.' '.$cmd_params;
				}
				#my $cmd = 'editconf'.' '.$cmd_params;
				my $flag;
				my $cmd_handle;
				my $oldwd = getcwd();
				chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
				$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
				chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
				unless ($flag){		
					$output_text->insert('end', "Running editconf...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Running editconf...\n\n" if $enable_log;
					#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
					syswrite LOG, "\$ $cmd\n" if $enable_log;
					$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
				} else {
					$gromacs_current_step = 0;
					$mw -> MsgBox( -title => 'Run editconf', -message=>"An error occured trying to run editconf!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
					$fast_switches[1] = 2;
					&working_gro_fin();
				}
			} elsif ($gromacs_current_step == 3) {
				$output_text->insert('end', "editconf finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "editconf finished successfully...\n\n" if $enable_log;
				# find initial center
				my $center_tag = $output_text->search(-backwards,'    center      :','end');
				if ($center_tag) {
					(my $line,my $index) = split(/\./,$center_tag,2);
						my $ttt = $output_text->get("$line.0","$line.end");
						my ($nums) = $ttt =~ /:\s+(\S+\s+\S+\s+\S+)\s+\(nm\)/;
						$initial_center = $nums;
						# print $initial_center;
				} else {
					$initial_center = '0 0 0';
					print 'Error finding initial center. Will use default'."\n";
				}
				if ($em_checks[0] == 1) {
					my $in_file = 'out.gro';
					my $cs_var = '';
					$cs_var = 'tip4p.gro' if ($gromacs_watermodel eq 'tip4p');
					my $cmd_params = "-cp $in_file -cs $cs_var -p -o";
					my $cmd;
					if (-e File::Spec->catfile('/','usr','bin','g_genbox')) {
						$cmd = 'g_genbox'.' '.$cmd_params;
					} else {
						$cmd = 'genbox'.' '.$cmd_params;
					}
					#my $cmd = 'genbox'.' '.$cmd_params;
					my $flag;
					my $cmd_handle;
					my $oldwd = getcwd();
					chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
					$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
					chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
					unless ($flag){		
						$output_text->insert('end', "Running genbox...\n\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "Running genbox...\n\n" if $enable_log;
						#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
						syswrite LOG, "\$ $cmd\n" if $enable_log;
						$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
					} else {
						$gromacs_current_step = 0;
						$mw -> MsgBox( -title => 'Run genbox', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
						$fast_switches[1] = 2;
						&working_gro_fin();
					}
				} else {
					$gromacs_current_step++;
					redo LOOPA;
				}
			} elsif ($gromacs_current_step == 4) {
				if ($em_checks[0] == 1) {
					$output_text->insert('end', "genbox finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "genbox finished successfully...\n\n" if $enable_log;
				}
				if ($em_checks[1] == 1) {
					my $mdp_file = File::Spec->catfile($drugOns_directory,'Parameter_Files','dummy_em.mdp');
					my $in_file = 'out.gro';
					my $cmd_params = "-f $mdp_file -c $in_file -p -o";
					my $cmd;
					if (-e File::Spec->catfile('/','usr','bin','g_grompp')) {
						$cmd = 'g_grompp'.' '.$cmd_params;
					} else {
						$cmd = 'grompp'.' '.$cmd_params;
					}
					#my $cmd = 'grompp'.' '.$cmd_params;
					my $flag;
					my $cmd_handle;
					my $oldwd = getcwd();
					chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
					$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
					chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
					unless ($flag){		
						$output_text->insert('end', "Running genion grompp...\n\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "Running genion grompp...\n\n" if $enable_log;
						#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
						syswrite LOG, "\$ $cmd\n" if $enable_log;
						$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
					} else {
						$gromacs_current_step = 0;
						$mw -> MsgBox( -title => 'Run genion grompp', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
						$fast_switches[1] = 2;
						&working_gro_fin();
					}
				} else {
					$gromacs_current_step++;
					redo LOOPA;
				}
			} elsif ($gromacs_current_step == 5) {
				if ($em_checks[1] == 1) {
					$output_text->insert('end', "genion grompp finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "genion grompp finished successfully...\n\n" if $enable_log;
					my $in_tpr = 'topol.tpr';
					my $conc = 0.01;
					my $answer_file = File::Spec->catfile($drugOns_directory,'Parameter_Files','genion_group');
					my $cmd_params = "-s $in_tpr -o -p -pname NA -nname CL -conc $conc -neutral < $answer_file";
					my $cmd;
					if (-e File::Spec->catfile('/','usr','bin','g_genion')) {
						$cmd = 'g_genion'.' '.$cmd_params;
					} else {
						$cmd = 'genion'.' '.$cmd_params;
					}
					#my $cmd = 'genion'.' '.$cmd_params;
					my $flag;
					my $cmd_handle;
					my $oldwd = getcwd();
					chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
					$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
					chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
					unless ($flag){		
						$output_text->insert('end', "Running genion...\n\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "Running genion...\n\n" if $enable_log;
						#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
						syswrite LOG, "\$ $cmd\n" if $enable_log;
						$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
					} else {
						$gromacs_current_step = 0;
						$mw -> MsgBox( -title => 'Run genion', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
						$fast_switches[1] = 2;
						&working_gro_fin();
					}
				} else {
					$gromacs_current_step++;
					redo LOOPA;
				}
			} elsif ($gromacs_current_step == 6) {
				if ($em_checks[1] == 1) {
					$output_text->insert('end', "genion finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "genion finished successfully...\n\n" if $enable_log;
				}
				my $mdp_file = File::Spec->rel2abs($selected_mdrun_par_file);
				my $in_file = 'out.gro';
				my $cmd_params = "-f $mdp_file -c $in_file -p -o";
				my $cmd;
				if (-e File::Spec->catfile('/','usr','bin','g_grompp')) {
					$cmd = 'g_grompp'.' '.$cmd_params;
				} else {
					$cmd = 'grompp'.' '.$cmd_params;
				}
				#my $cmd = 'grompp'.' '.$cmd_params;
				my $flag;
				my $cmd_handle;
				my $oldwd = getcwd();
				chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
				$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
				chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
				unless ($flag){		
					$output_text->insert('end', "Running grompp...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Running grompp...\n\n" if $enable_log;
					#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
					syswrite LOG, "\$ $cmd\n" if $enable_log;
					$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
				} else {
					$gromacs_current_step = 0;
					$mw -> MsgBox( -title => 'Run grompp', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
					$fast_switches[1] = 2;
					&working_gro_fin();
				}
			} elsif ($gromacs_current_step == 7) {
				$output_text->insert('end', "grompp finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "grompp finished successfully...\n\n" if $enable_log;
				my $tpr_file = 'topol.tpr';
				my $in_file = 'out.gro';
				my $cmd_params = "-s $tpr_file -o -c -e -g -v";
				my $cmd;
				if (-e File::Spec->catfile('/','usr','bin','g_mdrun')) {
					$cmd = 'g_mdrun'.' '.$cmd_params;
				} else {
					$cmd = 'mdrun'.' '.$cmd_params;
				}
				#my $cmd = 'mdrun'.' '.$cmd_params;
				my $flag;
				my $cmd_handle;
				my $oldwd = getcwd();
				chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
				$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
				chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
				unless ($flag){		
					$output_text->insert('end', "Running mdrun...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Running mdrun...\n\n" if $enable_log;
					#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
					syswrite LOG, "\$ $cmd\n" if $enable_log;
					$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
				} else {
					$gromacs_current_step = 0;
					$mw -> MsgBox( -title => 'Run mdrun', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
					$fast_switches[1] = 2;
					&working_gro_fin();
				}	
			} elsif ($gromacs_current_step == 8) {
				$output_text->insert('end', "mdrun finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "mdrun finished successfully...\n\n" if $enable_log;
				my $in_file = 'confout.gro';
				my ($volume,$directories,$out_file) = File::Spec->splitpath($selected_gro_out_file);
				my $output_file = File::Spec->catfile($drugOns_directory,'.tmp',$out_file);
				my $cmd_params = "-f $in_file -o $output_file -pbc";
				my $cmd;
				if (-e File::Spec->catfile('/','usr','bin','g_editconf')) {
					$cmd = 'g_editconf'.' '.$cmd_params;
				} else {
					$cmd = 'editconf'.' '.$cmd_params;
				}
				#my $cmd = 'editconf'.' '.$cmd_params;
				my $flag;
				my $cmd_handle;
				my $oldwd = getcwd();
				chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
				$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
				chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
				unless ($flag){		
					$output_text->insert('end', "Running editconf...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Running editconf...\n\n" if $enable_log;
					#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
					syswrite LOG, "\$ $cmd\n" if $enable_log;
					$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
				} else {
					$gromacs_current_step = 0;
					$mw -> MsgBox( -title => 'Run editconf', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
					$fast_switches[1] = 2;
					&working_gro_fin();
				}
			} elsif ($gromacs_current_step == 9) {
				$output_text->insert('end', "editconf finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "editconf finished successfully...\n\n" if $enable_log;
				if ($em_checks[3] == 1) {
					my $answer_file;
					if ($gromacs_ff eq 'charmm27') {
						$answer_file = File::Spec->catfile($drugOns_directory,'Parameter_Files','g_energy_select_charmm');
					} else {
						$answer_file = File::Spec->catfile($drugOns_directory,'Parameter_Files','g_energy_select');
					}
					my $cmd_params = "-f -o <$answer_file";
					my $cmd = 'g_energy'.' '.$cmd_params;
					my $flag;
					my $cmd_handle;
					my $oldwd = getcwd();
					chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
					$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
					chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
					unless ($flag){		
						$output_text->insert('end', "Running g_energy...\n\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "Running g_energy...\n\n" if $enable_log;
						#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
						syswrite LOG, "\$ $cmd\n" if $enable_log;
						$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
					} else {
						$gromacs_current_step = 0;
						$mw -> MsgBox( -title => 'Run editconf', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
						$fast_switches[1] = 2;
						&working_gro_fin();
					}
				} else {
					$gromacs_current_step++;
					redo LOOPA;
				}
			} elsif ($gromacs_current_step == 10) {		
				if ($em_checks[3] == 1) {
					$output_text->insert('end', "g_energy finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "g_energy finished successfully...\n\n" if $enable_log;
					$output_text->insert('end', "Generating energy graph...",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Generating energy graph..." if $enable_log;
					my $oldwd = getcwd();
					chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
					system "$preferred_graph_viewer energy.xvg &" and die "preferred_graph_viewer error: $?\n";
					chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
					sleep 1;
					$output_text->insert('end', "Done.\n\n",'boldy'); $output_text->SetCursor('end');
					syswrite LOG, "Done.\n\n" if $enable_log;
					$gromacs_current_step++;
					redo LOOPA;
				} else {
					$gromacs_current_step++;
					redo LOOPA;
				}
			} elsif ($gromacs_current_step == 11) {
				if ($em_checks[2] == 1) {
					my ($volume,$directories,$out_file) = File::Spec->splitpath($selected_gro_out_file);
					my $in_file = File::Spec->catfile($drugOns_directory,'.tmp',$out_file);
					unless (-e $in_file) {
						$gromacs_current_step = 0;
						$mw -> MsgBox( -title => 'Clean PDB File', -message=>"An error occured trying to clean PDB File!.",-type=>'ok',-icon=>'error')->Show;
						$fast_switches[1] = 2;
						&working_gro_fin();
					} else {
						$output_text->insert('end', "Removing water/ions from the PDB File...\n\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "Removing water/ions from the PDB File...\n\n" if $enable_log;
						# my $temp_file = $in_file.'.2';
						my $temp_file = File::Spec->catfile($drugOns_directory,'.tmp','2_'.$out_file);
						eval {
							open IN, $in_file  or die "error reading input PDB: $! -";
							open OUT, '>', $temp_file or die "error writing output temp PDB file: $! -";
							while (my $line = <IN>)	{
								next if ($line =~ /SOL/);
								next if ($line =~ /NA\s+NA/);
								next if ($line =~ /CL\s+CL/);
								print OUT $line;
							}
							close OUT;
							close IN;
							$output_text->insert('end', "  Created temporary file $temp_file successfully.\n\n"); $output_text->SetCursor('end');
							syswrite LOG, "  Created temporary file $temp_file successfully.\n\n" if $enable_log;
							$output_text->insert('end', "Done!\n\n",'boldy'); $output_text->SetCursor('end');
							syswrite LOG, "Done.\n\n" if $enable_log;
						};	
						if ($@) {
							$output_text->insert('end', "ERROR:\n  $@");$output_text->SetCursor('end');
							$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
							$gromacs_current_step = 0;
							$fast_switches[1] = 2;
							&working_gro_fin();
							return;
						}
						$gromacs_current_step++;
						redo LOOPA;
					}
				} else {
					$gromacs_current_step++;
					redo LOOPA;
				}
			} elsif ($gromacs_current_step == 12) {
				$output_text->insert('end', "Shifting Protein to its initial center...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Shifting Protein to its initial center...\n\n" if $enable_log;
				my ($volume,$directories,$out_file) = File::Spec->splitpath($selected_gro_out_file);
				my $in_file = File::Spec->catfile($drugOns_directory,'.tmp',$out_file);
				if ($em_checks[2] == 1) {
					$in_file = File::Spec->catfile($drugOns_directory,'.tmp','2_'.$out_file);
				}
				unless (-e $in_file) {
					$gromacs_current_step = 0;
					$mw -> MsgBox( -title => 'Re-center PDB File', -message=>"An error occured trying to re-center PDB File. File Not Found!",-type=>'ok',-icon=>'error')->Show;
					$fast_switches[1] = 2;
					&working_gro_fin();
				} else {
					my $output_file = File::Spec->catfile($drugOns_directory,'.tmp','cent_'.$out_file);
					my $cmd_params = "-f $in_file -o $output_file -center $initial_center";
					my $cmd;
					if (-e File::Spec->catfile('/','usr','bin','g_editconf')) {
						$cmd = 'g_editconf'.' '.$cmd_params;
					} else {
						$cmd = 'editconf'.' '.$cmd_params;
					}
					#my $cmd = 'editconf'.' '.$cmd_params;
					my $flag;
					my $cmd_handle;
					my $oldwd = getcwd();
					chdir File::Spec->catdir($drugOns_directory,'.tmp') or die "$drugOns_directory/.tmp: $!\n";
					$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
					chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
					unless ($flag){		
						$output_text->insert('end', "Running editconf...\n\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "Running editconf...\n\n" if $enable_log;
						#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
						syswrite LOG, "\$ $cmd\n" if $enable_log;
						$mw->fileevent($cmd_handle,'readable', [\&working_gro,$cmd_handle,$opt,$abs_selected_gro_out_file]);
					} else {
						$gromacs_current_step = 0;
						$mw -> MsgBox( -title => 'Run editconf', -message=>"An error occured trying to run genbox!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
						$fast_switches[1] = 2;
						&working_gro_fin();
					}
				}
			} elsif ($gromacs_current_step == 13) {
				$output_text->insert('end', "editconf finished successfully...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "editconf finished successfully...\n\n" if $enable_log;
				$output_text->insert('end', "Creating output file...\n\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Creating output file...\n\n" if $enable_log;
				my ($volume,$directories,$out_file) = File::Spec->splitpath($selected_gro_out_file);
				my $in_file = File::Spec->catfile($drugOns_directory,'.tmp','cent_'.$out_file);
				unless (-e $in_file) {
					$gromacs_current_step = 0;
					$mw -> MsgBox( -title => 'Create PDB File', -message=>"An error occured trying to create PDB File!.",-type=>'ok',-icon=>'error')->Show;
					$fast_switches[1] = 2;
					&working_gro_fin();
				} else {
					eval {
						open IN, $in_file  or die "error reading input PDB: $! -";
						open OUT, '>', $selected_gro_out_file or die "error writing output PDB file: $! -";
						print OUT "REMARK   1 Output PDB File from Receptor Optimization Step - DrugOn\nREMARK   1\n";
						while (my $line = <IN>)	{
							print OUT $line;
						}
						close OUT;
						close IN;
						$output_text->insert('end', "  Created $selected_gro_out_file successfully.\n\n"); $output_text->SetCursor('end');
						syswrite LOG, "  Created $selected_gro_out_file successfully.\n\n" if $enable_log;
						$output_text->insert('end', "Done!\n\n",'boldy'); $output_text->SetCursor('end');
						syswrite LOG, "Done.\n\n" if $enable_log;
					};
					if ($@) {
						$output_text->insert('end', "ERROR:\n  $@");$output_text->SetCursor('end');
						$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
						$gromacs_current_step = 0;
						$fast_switches[1] = 2;
						&working_gro_fin();
						return;
					}
					$gromacs_current_step++;
					redo LOOPA;
				}
			} elsif ($gromacs_current_step == 14) {
				$output_text->insert('end', "Receptor optimization finished successfully.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Receptor optimization finished successfully.\n" if $enable_log;
				$fast_switches[1] = 1;
				&working_gro_fin();
			} else {}
			}
		}
	}	
}

sub working_gro_fin {	
	$gromacs_current_step = 0;
	$gromacs_has_error = 0;
	$em_button->configure(-state => 'normal');
	$em_interrupt_button->configure(-state => 'disable');
	$Tab_frame->pageconfigure('Pre',-state => 'normal');
	$Tab_frame->pageconfigure('Build',-state => 'normal');
	$Tab_frame->pageconfigure('pha',-state => 'normal');
	$progress = 0; $gromacs_steps=0;
	$pb_label->configure(-text => '0%');
	$running_id->cancel;
	$pb__bulb_label->configure(-image => $bulb_off_image);
	$running_bulb = 0;
	$output_text->insert('end', "\n----------\nFinished Protein Receptor Optimization...\n----------\n\n",'red'); $output_text->SetCursor('end');
	syswrite LOG, "\n----------\nFinished Protein Receptor Optimization...\n----------\n\n" if $enable_log;
	my $to_delete = File::Spec->catfile($drugOns_directory,'.tmp','*');
	system "rm $to_delete" and die "deleting $to_delete returned $?\n";
}

sub tag_output_gro {
	my $error_tag = $output_text->search(-backwards,'error','end');
	if ($error_tag) {
		(my $line,my $index) = split(/\./,$error_tag,2);
		my $line2= $line + 1;
		$output_text->tagAdd('boldy',"$line.0","$line2.end");
	}
}

sub update_remaining_gro {
	my ($redline,$opt) = @_;
	my $temp = min 100,0 + (100 * ($gromacs_current_step/($gromacs_complete_steps-3)));
	$progress = $temp;
	$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
}

sub gro_translate {
	my ($opt) = @_;
	
	if ($opt eq 'amber03p-amber94n') {
		return 'amber03';
	} elsif ($opt eq 'amber94') {
		return 'amber94';
	} elsif ($opt eq 'amber96p-amber94n') {
		return 'amber96';
	} elsif ($opt eq 'amber99p-amber94n') {
		return 'amber99';
	} elsif ($opt eq 'amber99SBp-amber94n') {
		return 'amber99sb';
	} elsif ($opt eq 'amber99SB-ILDNp-amber94n') {
		return 'amber99sb-ildn';
	} elsif ($opt eq 'amberGS') {
		return 'amberGS';
	} elsif ($opt eq 'charmm27') {
		return 'charmm27';
	} elsif ($opt eq 'gromos96 43a1') {
		return 'gromos43a1';
	} elsif ($opt eq 'gromos96 43a2') {
		return 'gromos43a2';
	} elsif ($opt eq 'gromos96 45a3') {
		return 'gromos45a3';
	} elsif ($opt eq 'gromos96 53a5') {
		return 'gromos53a5';
	} elsif ($opt eq 'gromos96 53a6') {
		return 'gromos53a6';
	} elsif ($opt eq 'opls-aa/l') {
		return 'oplsaa';
	} elsif ($opt eq 'tip3p') {
		return 'tip3p';
	} elsif ($opt eq 'tip4p') {
		return 'tip4p';
	} elsif ($opt eq 'spc') {
		return 'spc';
	} elsif ($opt eq 'spce') {
		return 'spce';
	} else {$output_text->insert('end', "Gromacs: Unknown $opt.\n"); $output_text->SetCursor('end');}
}

sub browse_gro_file_callback {
	my ($opt) = @_;
	
	my $types=[];
	push @$types, ['PDB files', '*.pdb'];
	push @$types, ['All files', '*'];
	if ($opt eq 'in') {
		my $file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open PDB File', -multiple => 0);
		if (defined $file) {
			$selected_gro_in_file = $file;
		}
	} elsif ($opt eq 'out') {
		my $file = $mw->getSaveFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Save as PDB File', -multiple => 0);
		if (defined $file) {
			$selected_gro_out_file = $file;
		}
	}
}



sub create_par_file_gro_callback {
	my ($opt) = @_;
	
	if ($opt eq 'mdrun') {
		&parameter_files_gro_window('mdrun');
	} else {}
}



sub browse_par_file_gro_callback {
	my ($opt) = @_;
	
	my $types=[];
	push @$types, ['Parameter files', '*.mdp'];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'mdrun') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_mdrun_par_file = $file;
		}
	} else {}
}


# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Build Ligands																		#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	LigBuilder 1.2																		#
# --------------------------------------------------------------------------------------------------------------#


sub ligbuilder_buttons_callback {
	my ($opt) = @_;
	
	if ($opt eq 'pocket') {
		unless ($selected_pocket_par_file && -e $selected_pocket_par_file) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide a parameter file for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			my $cmd;
			if ($ligbuilder_path) {
				$cmd = File::Spec->catfile($ligbuilder_path,'bin',$opt).' '.$selected_pocket_par_file;
			} else {
				$cmd = $opt.' '.$selected_pocket_par_file;
			}
			my $flag;
			my $cmd_handle;
			$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
			unless ($flag){
				syswrite LOG, "\$ $cmd\n" if $enable_log;
				$output_text->insert('end', "----------\nRunning $opt...\n----------\n\n",'red'); $output_text->SetCursor('end');
				#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$pocket_button->configure(-state => 'disabled');
				$grow_button->configure(-state => 'disabled');
				$link_button->configure(-state => 'disabled');
				$process_button->configure(-state => 'disabled');
				$interrupt_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('EM',-state => 'disabled');
				$Tab_frame->pageconfigure('Pre',-state => 'disabled');
				$Tab_frame->pageconfigure('pha',-state => 'disabled');
				$mw->fileevent($cmd_handle,'readable', [\&working,$cmd_handle, $opt]);
			}else{
				$mw -> MsgBox( -title => 'Run $opt', -message=>"An error occured trying to run $opt!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
				$fast_switches[2] = 2;
			}	
		}
	} elsif ($opt eq 'grow') {
		unless ($selected_grow_par_file && -e $selected_grow_par_file) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide a parameter file for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			
		  $output_text->insert('end', "----------\nRunning $opt...\n----------\n\n",'red'); $output_text->SetCursor('end');
		  my $spc_error = 0;
		  # add spc
		  if ($add_spc) {
			$output_text->insert('end', "Making all seed hydrogens growing sites...\n",'boldy'); $output_text->SetCursor('end');
		    syswrite LOG, "Making all seed hydrogens growing sites...\n" if $enable_log;
			eval {
				open IN, $selected_grow_par_file  or die "error reading input parameter file: $! -";
				my $seed_file_todo;
				while (my $line = <IN>) {
					if ($line =~ /^SEED_LIGAND_FILE/) {
						(my $a,$seed_file_todo) = split /\s+/,$line;
						last;
					}
				}
				close IN;
				my $seed_file_todo_abs = File::Spec->rel2abs($seed_file_todo);
				unless ($seed_file_todo_abs && -e $seed_file_todo_abs) {
					die "Seed parameter could not be verified as a valid file: $! -";
				} else {
					system("/usr/bin/perl -p -i.bak -e '".'s/H(\s+.+)H\s/H$1H\.spc /'."' $seed_file_todo_abs") and die("Error replacing H to H.spc: $! -")
				}
			};
			if ($@) {
				$spc_error = 1;
				$output_text->insert('end', "\nERROR:\n  $@");$output_text->SetCursor('end');
				$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "ERROR:\n  $@Failure!\n" if $enable_log;
				$output_text->insert('end', "\n----------\nExecution of $opt finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
				syswrite LOG, "\n\$ ----------\n\n" if $enable_log;
			} else {
				$output_text->insert('end', "Done\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Done\n" if $enable_log;
			}
		  }
		  unless ($spc_error) {
			my $cmd;
			if ($ligbuilder_path) {
				$cmd = File::Spec->catfile($ligbuilder_path,'bin',$opt).' '.$selected_grow_par_file;
			} else {
				$cmd = $opt.' '.$selected_grow_par_file;
			}
			my $flag;
			my $cmd_handle;
			$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
			unless ($flag){
				syswrite LOG, "\$ $cmd\n" if $enable_log;
				open IN, "$selected_grow_par_file" or die "Error trying to open $selected_grow_par_file!";
				foreach my $line (<IN>) {
					if ($line =~ /NUMBER_OF_GENERATION\s*(\d+)/ ) {
						$complete_steps[1] = $1;
						last;
					}
				}
				close IN;
				#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$pocket_button->configure(-state => 'disabled');
				$grow_button->configure(-state => 'disabled');
				$link_button->configure(-state => 'disabled');
				$process_button->configure(-state => 'disabled');
				$interrupt_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('EM',-state => 'disabled');
				$Tab_frame->pageconfigure('Pre',-state => 'disabled');
				$Tab_frame->pageconfigure('pha',-state => 'disabled');
				$mw->fileevent($cmd_handle,'readable', [\&working,$cmd_handle, $opt]);
				&tetrisbox if ($enable_fun == 1);
			}else{
				$mw -> MsgBox( -title => 'Run $opt', -message=>"An error occured trying to run $opt!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
				$fast_switches[3] = 2;
				$output_text->insert('end', "\n----------\nExecution of $opt finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
				syswrite LOG, "\n\$ ----------\n\n" if $enable_log;
			}
		  }
		}
	} elsif ($opt eq 'link') {
		unless ($selected_link_par_file && -e $selected_link_par_file) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide a parameter file for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			
		  $output_text->insert('end', "----------\nRunning $opt...\n----------\n\n",'red'); $output_text->SetCursor('end');
		  my $spc_error = 0;
		  # add spc
		  if ($add_spc) {
			$output_text->insert('end', "Making all seed hydrogens growing sites...\n",'boldy'); $output_text->SetCursor('end');
		    syswrite LOG, "Making all seed hydrogens growing sites...\n" if $enable_log;
			eval {
				open IN, $selected_link_par_file  or die "error reading input parameter file: $! -";
				my $seed_file_todo;
				while (my $line = <IN>) {
					if ($line =~ /^SEED_LIGAND_FILE/) {
						(my $a,$seed_file_todo) = split /\s+/,$line;
						last;
					}
				}
				close IN;
				my $seed_file_todo_abs = File::Spec->rel2abs($seed_file_todo);
				unless ($seed_file_todo_abs && -e $seed_file_todo_abs) {
					die "Seed parameter could not be verified as a valid file: $! -";
				} else {
					system("/usr/bin/perl -p -i.bak -e '".'s/H(\s+.+)H\s/H$1H\.spc /'."' $seed_file_todo_abs") and die("Error replacing H to H.spc: $! -")
				}
			};
			if ($@) {
				$spc_error = 1;
				$output_text->insert('end', "\nERROR:\n  $@");$output_text->SetCursor('end');
				$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "ERROR:\n  $@Failure!\n" if $enable_log;
				$output_text->insert('end', "\n----------\nExecution of $opt finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
				syswrite LOG, "\n\$ ----------\n\n" if $enable_log;
			} else {
				$output_text->insert('end', "Done\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "Done\n" if $enable_log;
			}
		  }
		  unless ($spc_error) {	
			my $cmd;
			if ($ligbuilder_path) {
				$cmd = File::Spec->catfile($ligbuilder_path,'bin',$opt).' '.$selected_link_par_file;
			} else {
				$cmd = $opt.' '.$selected_link_par_file;
			}
			my $flag;
			my $cmd_handle;
			$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
			unless ($flag){
				syswrite LOG, "\$ $cmd\n" if $enable_log;
				open IN, "$selected_link_par_file" or die "Error trying to open $selected_link_par_file!";
				foreach my $line (<IN>) {
					if ($line =~ /NUMBER_OF_GENERATION\s*(\d+)/ ) {
						$complete_steps[2] = $1;
						last;
					}
				}
				close IN;
				#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$pocket_button->configure(-state => 'disabled');
				$grow_button->configure(-state => 'disabled');
				$link_button->configure(-state => 'disabled');
				$process_button->configure(-state => 'disabled');
				$interrupt_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('EM',-state => 'disabled');
				$Tab_frame->pageconfigure('Pre',-state => 'disabled');
				$Tab_frame->pageconfigure('pha',-state => 'disabled');
				$mw->fileevent($cmd_handle,'readable', [\&working,$cmd_handle, $opt]);
				&tetrisbox if ($enable_fun == 1);
			}else{
				$mw -> MsgBox( -title => 'Run $opt', -message=>"An error occured trying to run $opt!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
				$fast_switches[3] = 2;
				$output_text->insert('end', "\n----------\nExecution of $opt finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
				syswrite LOG, "\n\$ ----------\n\n" if $enable_log;
			}
		  }
		}
	} elsif ($opt eq 'process') {
		unless ($selected_process_par_file && -e $selected_process_par_file) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide a parameter file for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			my $cmd;
			if ($ligbuilder_path) {
				$cmd = File::Spec->catfile($ligbuilder_path,'bin',$opt).' '.$selected_process_par_file;
			} else {
				$cmd = $opt.' '.$selected_process_par_file;
			}
			my $flag;
			my $cmd_handle;
			$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
			unless ($flag){
				syswrite LOG, "\$ $cmd\n" if $enable_log;
				$output_text->insert('end', "----------\nRunning $opt...\n----------\n\n",'red'); $output_text->SetCursor('end');
				#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$pocket_button->configure(-state => 'disabled');
				$grow_button->configure(-state => 'disabled');
				$link_button->configure(-state => 'disabled');
				$process_button->configure(-state => 'disabled');
				$interrupt_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('EM',-state => 'disabled');
				$Tab_frame->pageconfigure('Pre',-state => 'disabled');
				$Tab_frame->pageconfigure('pha',-state => 'disabled');
				$mw->fileevent($cmd_handle,'readable', [\&working,$cmd_handle, $opt]);
			}else{
				$mw -> MsgBox( -title => 'Run $opt', -message=>"An error occured trying to run $opt!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
				$fast_switches[4] = 2;
			}
		}
	} elsif ($opt eq 'interrupt') {
		if (my $ans = $mw->messageBox( -icon => 'question',-message => "Are you sure you want to interrupt the running process?", -title => 'Interrupt process',-type => 'YesNo') =~ /Yes/i) {
			syswrite LOG, "\nInterrupted by User.\n" if $enable_log;
			kill 1, $pid;
			$output_text->insert('end', "Interrupted by User.\n",'boldy'); $output_text->SetCursor('end');
			$pocket_button->configure(-state => 'normal');
			$grow_button->configure(-state => 'normal');
			$link_button->configure(-state => 'normal');
			$process_button->configure(-state => 'normal');
			$interrupt_button->configure(-state => 'disabled');
			$Tab_frame->pageconfigure('EM',-state => 'normal');
			$Tab_frame->pageconfigure('Pre',-state => 'normal');
			$Tab_frame->pageconfigure('pha',-state => 'normal');
			sleep(1);
			$fast_switches[2] = -1;
			$fast_switches[3] = -1;
			$fast_switches[4] = -1;
		}
	} else {}	
}

sub create_par_file_callback {
	my ($opt) = @_;
	
	if ($opt eq 'pocket') {
		&parameter_files_window('pocket');
	} elsif ($opt eq 'grow') {
		&parameter_files_window('grow');
	} elsif ($opt eq 'link') {
		&parameter_files_window('link');
	} elsif ($opt eq 'process') {
		&parameter_files_window('process');
	} else {}
}

sub browse_par_file_callback {
	my ($opt) = @_;
	
	my $types=[];
	push @$types, ['Parameter files', '*.index'];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'pocket') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_pocket_par_file = $file;
		}
	} elsif ($opt eq 'grow') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_grow_par_file = $file;
		}
	} elsif ($opt eq 'link') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_link_par_file = $file;
		}
	} elsif ($opt eq 'process') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_process_par_file = $file;
		}
	} else {}
}


# --------------------------------------------------------------------------------------------------------------#
#	LigBuilder 2.0																		#
# --------------------------------------------------------------------------------------------------------------#


sub ligbuilder_20_buttons_callback {
	my ($opt) = @_;
	
	if ($opt eq 'cavity') {
		unless ($selected_cavity_par_file && -e $selected_cavity_par_file) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide a parameter file for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			my $runopt = ($arch == 64) ? 'cavity64' : 'cavity32' ;
			my $cmd;
			if ($ligbuilder_20_path) {
				#$cmd = File::Spec->catfile($ligbuilder_20_path,'build',$runopt).' '.$selected_cavity_par_file;
				$cmd = File::Spec->catfile($ligbuilder_20_path,'bin',$runopt).' '.$selected_cavity_par_file;
			} else {
				$cmd = $runopt.' '.$selected_cavity_par_file;
			}
			my $flag;
			my $cmd_handle;
			$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
			unless ($flag){
				syswrite LOG, "\$ $cmd\n" if $enable_log;
				$output_text->insert('end', "----------\nRunning $opt...\n----------\n\n\n",'red'); $output_text->SetCursor('end');
				#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$cavity_button->configure(-state => 'disabled');
				$build_button->configure(-state => 'disabled');
				$makeindex_button->configure(-state => 'disabled');
				$interrupt_20_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('EM',-state => 'disabled');
				$Tab_frame->pageconfigure('Pre',-state => 'disabled');
				$Tab_frame->pageconfigure('pha',-state => 'disabled');
				$mw->fileevent($cmd_handle,'readable', [\&working,$cmd_handle, $opt]);
			}else{
				$mw -> MsgBox( -title => 'Run $opt', -message=>"An error occured trying to run $opt!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
			}	
		}
	} elsif ($opt eq 'build') {
		unless ($selected_build_par_file && -e $selected_build_par_file) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide a parameter file for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			my $runopt = ($arch == 64) ? 'build64' : 'build32' ;
			my $cmd;
			if ($ligbuilder_20_path) {
				#$cmd = File::Spec->catfile($ligbuilder_20_path,'build',$runopt).' '.$build_function.' '.$selected_build_par_file;
				$cmd = File::Spec->catfile($ligbuilder_20_path,'bin',$runopt).' '.$build_function.' '.$selected_build_par_file;
			} else {
				$cmd = $runopt.' '.$build_function.' '.$selected_build_par_file;
			}
			my $flag;
			my $cmd_handle;
			$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
			unless ($flag){
				syswrite LOG, "\$ $cmd\n" if $enable_log;
				$output_text->insert('end', "----------\nRunning $opt...\n----------\n\n",'red'); $output_text->SetCursor('end');
				#$command_history_text->insert('end', "\$ $cmd\n",'boldy'); $command_history_text->SetCursor('end');
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$cavity_button->configure(-state => 'disabled');
				$build_button->configure(-state => 'disabled');
				$makeindex_button->configure(-state => 'disabled');
				$interrupt_20_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('EM',-state => 'disabled');
				$Tab_frame->pageconfigure('Pre',-state => 'disabled');
				$Tab_frame->pageconfigure('pha',-state => 'disabled');
				$mw->fileevent($cmd_handle,'readable', [\&working,$cmd_handle, $opt]);
			}else{
				$mw -> MsgBox( -title => 'Run $opt', -message=>"An error occured trying to run $opt!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
			}	
		}
	} elsif ($opt eq 'interrupt') {
		if (my $ans = $mw->messageBox( -icon => 'question',-message => "Are you sure you want to interrupt the running process?", -title => 'Interrupt process',-type => 'YesNo') =~ /Yes/i) {
			syswrite LOG, "\nInterrupted by User.\n" if $enable_log;
			kill 1, $pid;
			$output_text->insert('end', "Interrupted by User.\n",'boldy'); $output_text->SetCursor('end');
			$cavity_button->configure(-state => 'normal');
			$build_button->configure(-state => 'normal');
			$makeindex_button->configure(-state => 'normal');
			$interrupt_20_button->configure(-state => 'disabled');
			$Tab_frame->pageconfigure('EM',-state => 'normal');
			$Tab_frame->pageconfigure('Pre',-state => 'normal');
			$Tab_frame->pageconfigure('pha',-state => 'normal');
		}
	} elsif ($opt eq 'index') {
		unless ($selected_build_db_folder && -d $selected_build_db_folder) {
			$mw -> MsgBox( -title => 'Run $opt', -message=>"You must provide a valid folder path for $opt!",-type=>'ok',-icon=>'warning')->Show;
		} else {
			$output_text->insert('end', "----------\nCreating INDEX File...\n----------\n\n",'red'); $output_text->SetCursor('end');
			syswrite LOG, "----------\nCreating INDEX File...\n----------\n\n" if $enable_log;
			eval {
				opendir(D, "$selected_build_db_folder") || die "Can't opedir $selected_build_db_folder: $!\n";
				my @list = readdir(D);
				closedir(D);
				my @list_s = sort @list;
				my $oldwd = getcwd();
				chdir "$selected_build_db_folder" or die "Can't cd to $selected_build_db_folder: $!\n";
				if (-e 'INDEX') {
					system "mv INDEX INDEX.bak" and die "renaming old  INDEX file returned $?\n"
				}
				open OUT, '>', 'INDEX' or die ("error creating INDEX file: $!");
				print OUT "#\n# Molecule library INDEX created with DrugOn\n#\n# The 1st column: fragment ID\n# The 2nd column: fragment file name\n# The 3th column: description\n#\n";
				my $count = 0;
				my $des = '';
				foreach my $f (@list_s) {
					if ( -e $f) {
						if ($f =~ /.mol2$/) {
							$count++;
							print OUT "$count\t$f\t$des\n";
						}
					}
				}
				print OUT "# END";
				close OUT;
				chdir "$oldwd" or die "Can't cd to $oldwd: $!\n";
				$output_text->insert('end', "INDEX File created successfully.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "INDEX File created successfully.\n" if $enable_log;
			};
			if ($@) {
				$output_text->insert('end', "\nERROR:\n  $@");$output_text->SetCursor('end');
				$output_text->insert('end', "\nFailure.\n",'boldy'); $output_text->SetCursor('end');
				syswrite LOG, "ERROR:\n  $@Failure!\n" if $enable_log;
			}
			$output_text->insert('end', "\n----------\nCreating INDEX File finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
			syswrite LOG, "\n----------\nCreating INDEX File finished.\n----------\n\n" if $enable_log;
		}
	} else {}
	
}

sub create_par_file_20_callback {
	my ($opt) = @_;
	
	if ($opt eq 'cavity') {
		&parameter_files_20_window('cavity');
	} elsif ($opt eq 'build') {
		&parameter_files_20_window('build');
	} else {}
}

sub browse_par_file_20_callback {
	my ($opt) = @_;
	
	my $types=[];
	push @$types, ['Parameter files', '*.input'];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'cavity') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_cavity_par_file = $file;
		}
	} elsif ($opt eq 'build') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_build_par_file = $file;
		}
	} elsif ($opt eq 'db_dir') {
		my $dir = $mw->DirSelect(-title => 'MOL2 DB Directory')->Show($working_directory);
		if (defined $dir) {
			if (-d $dir) {
				$selected_build_db_folder = $dir;
			}
		}
	} else {}
}


# --------------------------------------------------------------------------------------------------------------#
#	Common																				#
# --------------------------------------------------------------------------------------------------------------#


sub working {
	my ($cmd_handle,$opt) = @_;
	
	my $redline;
	my $stat = sysread $cmd_handle, $redline, 4096;
	unless ($stat == 0) {
		$redline =~ s/^(\s+Processing :)/\n$1/;
		
		$ligbuilder_has_error = 1 if $redline =~ /stdbuf: failed/;
		$ligbuilder_has_error = 1 if $redline =~ /errors have been detected/;
		$ligbuilder_has_error = 1 if $redline =~ /Error:/;
		$ligbuilder_has_error = 1 if $redline =~ /Invalid/;
		
		$output_text->insert('end', "$redline");
		$output_text->yview('end');
		syswrite LOG, $redline if $enable_log;
		&update_remaining($redline,$opt);
	} else {
		$mw->fileevent($cmd_handle ,'readable','');
		&tag_output();
		if ($ligbuilder_has_error == 1) {
			$fast_switches[2] = 2 if ($opt eq 'pocket');
			$fast_switches[3] = 2 if ($opt eq 'grow');
			$fast_switches[3] = 2 if ($opt eq 'link');
			$fast_switches[4] = 2 if ($opt eq 'process');
		} else {
			$fast_switches[2] = 1 if ($opt eq 'pocket');
			$fast_switches[3] = 1 if ($opt eq 'grow');
			$fast_switches[3] = 1 if ($opt eq 'link');
			$fast_switches[4] = 1 if ($opt eq 'process');
		}
		$pocket_button->configure(-state => 'normal');
		$grow_button->configure(-state => 'normal');
		$link_button->configure(-state => 'normal');
		$process_button->configure(-state => 'normal');
		$interrupt_button->configure(-state => 'disabled');
		$cavity_button->configure(-state => 'normal');
		$build_button->configure(-state => 'normal');
		$makeindex_button->configure(-state => 'normal');
		$interrupt_20_button->configure(-state => 'disabled');
		$Tab_frame->pageconfigure('EM',-state => 'normal');
		$Tab_frame->pageconfigure('Pre',-state => 'normal');
		$Tab_frame->pageconfigure('pha',-state => 'normal');
		$progress = 0; $steps=0;
		$pb_label->configure(-text => '0%');
		$running_id->cancel;
		$pb__bulb_label->configure(-image => $bulb_off_image);
		$running_bulb = 0;
		$output_text->insert('end', "\n----------\nExecution of $opt finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
		syswrite LOG, "\n\$ ----------\n\n" if $enable_log;
		# &close_tetris() if (($opt eq 'grow') || ($opt eq 'link') || ($opt eq 'build'));
		$ligbuilder_has_error = 0;
	}
}

sub update_remaining {
	my ($redline,$opt) = @_;
	if ($opt eq 'pocket') {
		my @list= ($redline =~ /Now/g);
		$steps += scalar(@list);
		my $temp = 100 * ($steps/$complete_steps[0]);
		$progress = $temp;
		$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
	} elsif ($opt eq 'grow') {
		if ($redline =~ /.*Generation\s*(\d+)/s) {
			$steps = $1;
			my $temp = 100 * ($steps/$complete_steps[1]);
			$progress = $temp;
			$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
		}
	} elsif ($opt eq 'link') {
		if ($redline =~ /.*generation\s*(\d+)/s) {
			$steps = $1;
			my $temp = 100 * ($steps/$complete_steps[2]);
			$progress = $temp;
			$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
		}
	} elsif ($opt eq 'process') {
		my @list= ($redline =~ /Now/g);
		$steps += scalar(@list);
		my $temp = 100 * ($steps/$complete_steps[3]);
		$progress = $temp;
		$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
	} elsif ($opt eq 'cavity') {
		my @list= ($redline =~ /Now/g);
		$steps += scalar(@list);
		my $temp = 100 * ($steps/$complete_steps[4]);
		$progress = $temp;
		$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
	} elsif ($opt eq 'build') {
		# NOT todo
	} else {}
	$mw->update;
}

sub tag_output {
	my $start_tag = $output_text->search(-backwards,'starts to run','end');
	my $start_tag_20 = $output_text->search(-backwards,'begining ...','end');
	my $error_tag = $output_text->search(-backwards,'errors have been detected','end');
	$output_text->FindAndReplaceAll(-exact,-case,'Error:','ERROR:');
	$output_text->FindAndReplaceAll(-exact,-case,'Invalid','INVALID');
	my $finish_tag = $output_text->search(-backwards,'done the job','end');
	if ($start_tag) {
		(my $line,my $index) = split(/\./,$start_tag,2);
		$output_text->tagAdd('boldy',"$line.0","$line.end");
	}
	if ($start_tag_20) {
		(my $line,my $index) = split(/\./,$start_tag_20,2);
		$output_text->tagAdd('boldy',"$line.0","$line.end");
	}
	if ($error_tag) {
		(my $line,my $index) = split(/\./,$error_tag,2);
		my $line2= $line + 1;
		$output_text->tagAdd('boldy',"$line.0","$line2.end");
	}
	if ($finish_tag) {
		(my $line,my $index) = split(/\./,$finish_tag,2);
		$output_text->tagAdd('boldy',"$line.0","$line.end");
	}
}






# --------------------------------------------------------------------------------------------------------------#
#																					#
# --------------------------------------------------------------------------------------------------------------#

# --------------------------------------------------------------------------------------------------------------#
#	Pharmacophore																	#
# --------------------------------------------------------------------------------------------------------------#

sub create_conf_file_callback {
	my ($opt) = @_;
	
	if ($opt eq 'pha') {
		&configure_pha_files_window('pha');
	} else {}
}

sub browse_conf_file_callback {
	my ($opt) = @_;
	
	my $types=[];
	push @$types, ['All files', '*'];
	my $file;
	if ($opt eq 'pha') {
		$file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types, -multiple => 0);
		if (defined $file) {
			$selected_pha_conf_file = $file;
		}
	} else {}
}




sub browse_pha_file_callback {
	my ($opt) = @_;

	my $types=[];
	push @$types, ['MOL2 files', '*.mol2'];
	push @$types, ['All files', '*'];
	if ($opt eq 'in1') {
		my $file1 = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open Protein File', -multiple => 0);
		if (defined $file1) {
			$selected_pha_pr_file = $file1;
		}
	}elsif ($opt eq 'in2') {
		my $file2 = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open Ligand File', -multiple => 0);
		if (defined $file2) {
			$selected_pha_li_file = $file2;
					}
				}		
				 
		my $input_flag =1;
}


sub  browse_pha_save_callback{


my $dir = $mw->DirSelect(-title => 'Select/Create Output Directory')->Show($working_directory);
		if (defined $dir) {
			$pha_save_file_dir= $dir;
		}
}




sub mv_files_callback{ 
	if ($pha_save_file_dir){
		
	my $results_file = $selected_pha_save_file;	
	my $Save_dir = File::Spec->catdir($pha_save_file_dir);
	chdir File::Spec->catdir($drugOns_directory,'Pharmacophore_files') or die "$drugOns_directory/.Pharmacophore_files: $!\n";
	system "mv $results_file $Save_dir" and die "Copying $results_file to $Save_dir returned: $?\n";
	$output_text->insert('end', "\nYour file has been moved successfully...\n\n ",'boldy');$output_text->SetCursor('end');
}
$pha_mv_button->configure(-state => 'disabled');
	}
	
	

sub config_file_callback{
	my ($opt) = @_;

		if($selected_pha_input_pr_file eq ".mol2" ||  $selected_pha_input_li_file eq ".mol2" ||  $selected_pha_save_file eq undef  || $selected_pha_input_pr_file eq undef || $selected_pha_input_li_file eq undef){
			$output_text->insert('end',"Put protein,ligand or Folder name...\n\n",'boldy'); $output_text->SetCursor('end');
			return;
		}else {
			if($pharmacophore_ff eq "default"){
		open(WCONF,">",File::Spec->catdir($drugOns_directory,'Pharmacophore_files/default')) || die " config didnt open.\n";  
				
				print WCONF "# scoring function and search settings\nscoring_function chemplp\nsearch_speed speed1\n";
				print WCONF "\n\n# input\nprotein_file $selected_pha_input_pr_file\n";
				print WCONF "ligand_file $selected_pha_input_li_file\n";
				print WCONF "\n\n# output\noutput_dir $selected_pha_save_file\n";
				print WCONF "\n\n# write single mol2 files (e.g. for RMSD calculation)\nwrite_multi_mol2 0\n";
				print WCONF "\n\n# binding site definition\nbindingsite_center -4.12761 57.2797 -10.175\nbindingsite_radius 12.8396\n";
				print WCONF "\n\n# cluster algorithm\ncluster_structures 10\ncluster_rmsd 2.0\n";
				print WCONF "\n\n# Constraints\nsphere_constraint 2.6 5.6 7.2 4.0 11\n";

			close(WCONF);
			}
		}
	}

sub pharmacophore_export_button_callback{    
	my ($opt) = @_;		
	if ($opt eq 'construct'){
		my $runopt;
		my $cmd;
		if($pharmacophore_ff eq 'default'){
			 $runopt =  'pharmACOphore --mode screen default' ;
		}elsif($pharmacophore_ff eq 'advanced'){
			 $runopt =  'pharmACOphore --mode screen advanced' ;
		}elsif($pharmacophore_ff eq 'custom'){
			&browse_custom_conf_file_callback;
			 $runopt = 'pharmACOphore --mode screen config' ;
		}
		if ($pharmACOphore_path) {
			chdir File::Spec->catdir($drugOns_directory,'Pharmacophore_files') or die "$drugOns_directory/.Pharmacophore_files: $!\n";
			$cmd = File::Spec->catfile($pharmACOphore_path,$runopt);
		} else {
			$cmd = $runopt;
		}
		my $flag;
		my $cmd_handle;
		$pid = open ($cmd_handle, "$stdbuf_str $cmd 2>&1 |") or $flag = $!;
		unless ($flag){
			syswrite LOG, "\$ $cmd\n" if $enable_log;
				$output_text->insert('end', "----------\nRunning pharmACOphore...\n----------\n\n\n",'red'); $output_text->SetCursor('end');
				$running_id = $mw->repeat($refresh_rate,\&toggle_running_icon);
				$pha_interrupt_button->configure(-state => 'normal');
				$Tab_frame->pageconfigure('EM',-state => 'disabled');
				$Tab_frame->pageconfigure('Pre',-state => 'disabled');
				$Tab_frame->pageconfigure('Build',-state => 'disabled');
				$mw->fileevent($cmd_handle,'readable', [\&working_pha,$cmd_handle]);
			}else{
				$mw -> MsgBox( -title => 'Run ', -message=>"An error occured trying to run pharmACOphore!\n\n$flag.",-type=>'ok',-icon=>'error')->Show;
				}
				
		}		
	elsif ($opt eq 'interrupt') {
		if (my $ans = $mw->messageBox( -icon => 'question',-message => "Are you sure you want to interrupt the running process?", -title => 'Interrupt process',-type => 'YesNo') =~ /Yes/i) {
			syswrite LOG, "\nInterrupted by User.\n" if $enable_log;
			kill 1, $pid;
			$output_text->insert('end', "Interrupted by User.\n",'boldy'); $output_text->SetCursor('end');
			$pha_interrupt_button->configure(-state => 'disabled');
			$pha_button->configure(-state => 'normal');
			$Tab_frame->pageconfigure('EM',-state => 'normal');
			$Tab_frame->pageconfigure('Pre',-state => 'normal');
			$Tab_frame->pageconfigure('Build',-state => 'normal');
			$Tab_frame->pageconfigure('pha',-state => 'normal');
			
		}
		}
		
		else{}

				
	}



sub browse_custom_conf_file_callback {
	my ($opt) = @_;

	my $types=[];
	push @$types, ['All files', '*'];
		my $CC_file = $mw->getOpenFile(-initialdir => $working_directory,-filetypes=>$types,-title => 'Open Custom Configuration File', -multiple => 0);
		if (defined $CC_file) {
			my $conf_dir = File::Spec->catdir($drugOns_directory,'Pharmacophore_files');
		        system "cp $CC_file $conf_dir/config" and die "Copying $CC_file to $conf_dir/config returned: $?\n";
		     
		}
	}
	
	
sub copy_pr_files_callback{
	if($selected_pha_pr_file eq undef || $selected_pha_li_file eq undef){
	$output_text->insert('end', "\nprotein or ligand input failed \n ",'boldy');$output_text->SetCursor('end');
	}else {
		my $Mol2_dir = File::Spec->catdir($drugOns_directory,'Pharmacophore_files');
		system "cp $selected_pha_pr_file $Mol2_dir" and die "Copying $selected_pha_pr_file to $Mol2_dir returned: $?\n";
			
		system "cp $selected_pha_li_file $Mol2_dir" and die "Copying $selected_pha_li_file to $Mol2_dir returned: $?\n";
			$output_text->insert('end', "\nprotein and ligand input has been finished successfully...\n\n ",'boldy');$output_text->SetCursor('end');
		}
	}

	
	
sub working_pha {
	my ($cmd_handle) = @_;
	
	my $redline;
	my $stat = sysread $cmd_handle, $redline, 4096;
	unless ($stat == 0) {
		$redline =~ s/^(\s+Processing :)/\n$1/;
		$output_text->insert('end', "$redline");
		$output_text->yview('end');
		syswrite LOG, $redline if $enable_log;
		&update_remaining_pha($redline);
	} else {
		$mw->fileevent($cmd_handle ,'readable','');
		&tag_output_pha();
		$pha_interrupt_button->configure(-state => 'disabled');
		$Tab_frame->pageconfigure('EM',-state => 'normal');
		$Tab_frame->pageconfigure('Pre',-state => 'normal');
		$Tab_frame->pageconfigure('Build',-state => 'normal');
		$Tab_frame->pageconfigure('pha',-state => 'normal');
		$pha_mv_button->configure(-state => 'normal');
		$pha_output_browse_button->configure(-state => 'normal');	
		$progress = 0;
		$pb_label->configure(-text => '0%');
		$running_id->cancel;
		$pb__bulb_label->configure(-image => $bulb_off_image);
		$running_bulb = 0;
		$output_text->insert('end', "\n----------\nExecution of pharmACOphore finished.\n----------\n\n",'red'); $output_text->SetCursor('end');
		syswrite LOG, "\n\$ ----------\n\n" if $enable_log;
	}
}

sub update_remaining_pha {
	my ($redline) = @_;
		my @list= ($redline =~ /Now/g);
		$steps += scalar(@list);
		my $temp = 90;
		$progress = $temp;
		$pb_label->configure(-text => sprintf("%3.1f%%",$temp));
	
	#$mw->update;
}

sub tag_output_pha {
	my $start_tag = $output_text->search(-backwards,'starts to run','end');
	my $start_tag_20 = $output_text->search(-backwards,'begining ...','end');
	my $error_tag = $output_text->search(-backwards,'errors have been detected','end');
	$output_text->FindAndReplaceAll(-exact,-case,'Error:','ERROR:');
	$output_text->FindAndReplaceAll(-exact,-case,'Invalid','INVALID');
	my $finish_tag = $output_text->search(-backwards,'done the job','end');
	if ($start_tag) {
		(my $line,my $index) = split(/\./,$start_tag,2);
		$output_text->tagAdd('boldy',"$line.0","$line.end");
	}
	if ($start_tag_20) {
		(my $line,my $index) = split(/\./,$start_tag_20,2);
		$output_text->tagAdd('boldy',"$line.0","$line.end");
	}
	if ($error_tag) {
		(my $line,my $index) = split(/\./,$error_tag,2);
		my $line2= $line + 1;
		$output_text->tagAdd('boldy',"$line.0","$line2.end");
	}
	if ($finish_tag) {
		(my $line,my $index) = split(/\./,$finish_tag,2);
		$output_text->tagAdd('boldy',"$line.0","$line.end");
	}
}
